/*
 Multi light paralax shader (simple : maximum 4 light's)
 Made by : SVSD_VAL
 Site    : http://svsd.mirgames.ru & http://delphigfx.mirgames.ru
 mail    : Valdim_05@mail.ru or SVSD_VAL@SibNet.ru
 ICQ     : 223-725-915
 Jabber  : svsd_val@jabber.ru or svsd_val@Jabber.Sibnet.ru
*/

//#define PARALLAX;
//#define ILLUM;
//#define SPECULAR;
//#define ENVIROMENT;

const int light_count = 4;

varying vec3 light_direction[light_count]; 
varying vec3 eye_direction;

uniform sampler2D color_map;
uniform sampler2D normal_map;
uniform sampler2D height_map;
uniform sampler2D illum_map;
uniform sampler2D specular_map;
uniform sampler2D env_map;

uniform float exponent;

void main()
{
   vec4 allcolor = vec4(0,0,0,0);
   vec3 v        = normalize(eye_direction);

	#ifdef PARALLAX
		float scale	=  0.04;
		float bias	= -0.03;

	        float height	= texture2D(height_map, gl_TexCoord[0].st).r;
	              height	= height * scale + bias;
	
	         vec2 texCoord	= gl_TexCoord[0].st + (height * v.xy);
	#else
	         vec2 texCoord	= gl_TexCoord[0].st;
	#endif

   vec4  base =           texture2D(color_map    , texCoord ).xyzw;
   vec3  bump = normalize(texture2D(normal_map   , texCoord ).xyz * 2.0 - 1.0);

	#ifdef SPECULAR
   vec4  spec =		  texture2D(specular_map , texCoord ).xyzw;
	#endif

	#ifdef ENVIROMENT
   vec3  env  =           texture2D(env_map      , reflect(-v, bump).xy * 0.5 + 0.5).xyz;
	#endif


   for (int i = 0; i < light_count ; i ++)
   {

	float distance_sqr	= dot(light_direction[i], light_direction[i]);
	float atten		= max(0.0, 1.0 - distance_sqr);

	vec3  l 	= light_direction[i] * inversesqrt(distance_sqr);
	float d 	= max(0.0, dot(l, bump) );
	float r 	= dot(reflect(-l, bump), v);
	float s		= pow( clamp(r, 0.0, 1.0), exponent);
	
	vec4 color 	= base * (gl_FrontLightProduct[i].ambient + gl_FrontLightProduct[i].diffuse *  d) * 2;


	  #ifdef SPECULAR
		color		+= spec * s;
	    #else
		color		+= gl_FrontLightProduct[i].diffuse * gl_FrontLightProduct[i].specular * s; 
	  #endif

		allcolor	+= color * atten;


	  #ifdef ENVIROMENT
		allcolor	+= vec4(env,0.0) * atten;
	  #endif;

   }
	  #ifdef ILLUM

		vec3  add_illum = texture2D(illum_map , texCoord).xyz;
		   gl_FragColor = vec4(allcolor.xyz + add_illum , base.w) ;
	     #else
		   gl_FragColor = vec4(allcolor.xyz , base.w) ;
	  #endif

}
