unit materials;
interface

uses windows,OpenGL15,Engine,shaders;

 Const
  Tex_Count        = 2;


 Type
  TMaterialRenderMode = (
      MAT_Z_ONLY,    // For shadow map updates (depth only)
      MAT_AMBIENT,   // For the ambient bass pass (textured, unlit)
      MAT_LIGHTING   // For the diffuse and specular pass (all-singing, all-dancing)
    );

   TTexture         = record
        W,H,BPP : Word;
        Data    : PByteArray;
   end;

   PMaterial        =^TMaterial;
   TMaterial        = Record
    UsedTextures    : Cardinal;
    Texture         : Array [0..Tex_Count] of record
                      used : boolean;
                     GL_ID : Cardinal;
                      end;

    Colors          : Array [0..3] of TVector;
    Blend           : Record
                       Enabled: Boolean;
                       Factor : Array [0..1] of cardinal;
                       Alpha  : Single;
                      end;
    Shader          : PShader;
    ShaderNum       : Cardinal;

    Name            : String;
  end;


var
  GL_USE_Textures       : boolean=true;
  CurShader             : PShader;

{ $define debugblend}
            {$ifdef debugblend}
  test                  : array [0..1] of integer;
            {$endif}

{-------------------------------------}
{$Region 'GL Materials'}
Function CreateDefaultMaterial:PMaterial;

Function LoadMatarial(FileName:String):Integer;

Function  SetBlend    (I:Integer):Cardinal;
Function BindMaterial( ID:Cardinal; RenderMode:TMaterialRenderMode):Cardinal;
Procedure DisableLastMaterial;
Procedure DisableMaterials();
Function RemoveMaterial(ID:Cardinal):boolean;
Procedure FreeMaterials;
Function GetMaterial(ID:Cardinal):PMaterial;

{$EndRegion}

{-------------------------------------}
{$Region 'GL Textures'}
procedure Texture_Enable (const ID: Cardinal; const Channel: Integer);
procedure Texture_Disable(const Channel: Integer);
function  CreateTexture  (const Width, Height, Format : Word; pData : Pointer) : Integer;
function TexNewID(clamp, mipmap: Boolean): Cardinal;
Procedure DeleteTexture(GL_ID:Cardinal);
Procedure FreeTextures;
Procedure AddToTextureList(const GLID:Cardinal);
{$EndRegion}

{-------------------------------------}

implementation
uses textures;

var
  MaterialList     : Array of PMaterial;
  LastMatrialID    : Cardinal;

  GL_Textures      : Array of Record
                        GL_ID:Cardinal;
                     end;

  const
  GLOBAL_AMBIENT: TVector4f = ( x: 0.1; y: 0.1; z: 0.1; w: 1 );

{$Region 'GL Textures'}

Procedure AddToTextureList(const GLID:Cardinal);
var i : integer;
begin
 I := Length(GL_Textures);
 SetLength(GL_Textures,i+1);
 GL_Textures[i].GL_ID  := glid;
end;

Procedure FreeTextures;
var i : integer;
begin
  for i := 0 to Length(GL_Textures)-1 do
  if glIsTexture(GL_Textures[i].GL_ID) then
  glDeleteTextures(1,@GL_Textures[i].GL_ID);
  SetLength(GL_Textures,0);
  FreeMaterials;
end;

Procedure DeleteTexture(GL_ID:Cardinal);
var i : integer;
begin
  for i := 0 to Length(GL_Textures)-1 do
  if GL_Textures[i].GL_ID = GL_ID then
  if glIsTexture(GL_Textures[i].GL_ID) then
  begin
    glDeleteTextures(1,@GL_Textures[i].GL_ID);
    GL_Textures[i] :=  GL_Textures[length(GL_Textures)-1];
    setlength (GL_Textures, length(GL_Textures)-1);
    exit;
  end;
end;


function TexNewID(clamp, mipmap: Boolean): Cardinal;
var
id:cardinal;
begin
//==      
  begin
    glGenTextures(1, @ID);
    glBindTexture(GL_TEXTURE_2D, ID);
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

    if clamp then
    begin
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
    end else
    begin
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    end;

    if mipmap then
    begin
      glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP_SGIS, GL_TRUE);
      glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
    end
    else
      glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

      glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);



    Result := ID;
    AddToTextureList(id);
  end;
end;


function CreateTexture(const Width, Height, Format : Word; pData : Pointer) : Integer;
var
  Texture : Cardinal;
begin
  Texture := TexNewID(False,GL_USE_MipMap);

// if GL_MaxAnisotropy < ANISOTROPYCount then ANISOTROPYCount:= GL_MaxAnisotropy;
// if ANISOTROPYCount>0 then glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, ANISOTROPYCount);
  case Format of
  GL_RGBA : gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, Width, Height, GL_RGBA, GL_UNSIGNED_BYTE,  pData);
  GL_RGB  : gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB , Width, Height, GL_RGB , GL_UNSIGNED_BYTE,  pData);
  GL_BGR  : gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB , Width, Height, GL_BGR, GL_UNSIGNED_BYTE,  pData);
  GL_BGRA : gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, Width, Height, GL_BGRA, GL_UNSIGNED_BYTE,  pData);
  GL_RGB8 : gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB8, Width, Height, GL_RGB , GL_UNSIGNED_BYTE,  pData);
  GL_LUMINANCE_ALPHA : gluBuild2DMipmaps(GL_TEXTURE_2D, 2, Width, Height, GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, pData);
  end;
  result :=Texture;
end;

procedure Texture_Enable(const ID: Cardinal; const Channel: Integer);
begin
  if not GL_USE_Textures then Exit;

  glActiveTextureARB(GL_TEXTURE0_ARB + Channel);
  glEnable(GL_TEXTURE_2D);
  //     -    
  if (ID = 0) or not glIsTexture(ID) then
        glBindTexture(GL_TEXTURE_2D,  0) else
        glBindTexture(GL_TEXTURE_2D, ID);
end;

procedure Texture_Disable(const Channel: Integer);
begin
  glActiveTextureARB(GL_TEXTURE0_ARB + Channel);
//  glBindTexture(GL_TEXTURE_2D, 0);
  glDisable(GL_TEXTURE_2D);
end;

{$EndRegion}

{$Region 'GL Materials'}
Function CreateMaterialFromFile(FileName:String):PMaterial;
var
 F: TextFile;
 S: String;
 P: TStrParam;
    m    : boolean;
  Use_Shader : String;

  DIR         : String;


begin
 New(Result);
//  CreateDefaultMaterial;

  AssignFile(F, FileName);
  Reset(F);

   m    := GL_USE_MipMap;
   GL_USE_MipMap:=true;
   DIR  :=                ExtractFilePath(FileName)+'Textures\';
   ZeroMemory(Result, SizeOf(TMaterial));

   With Result^ do
   begin
     Repeat
      readln(f,s);
      P := ParseStr(s);
      case Length(P) of
         2: begin
             if SameText(P[0],'SHADER') then Use_Shader := p[1];

             if SameText(P[0],'TEXTURE1') then
                 begin
                   LoadTexture(Dir+p[1],Texture[0].GL_ID);
                                        Texture[0].used :=true;
                 end;
             if SameText(P[0],'TEXTURE2') then
                 begin
                   LoadTexture(Dir+p[1],Texture[1].GL_ID);
                                        Texture[1].used :=true;
                 end;
             if SameText(P[0],'TEXTURE3') then
                 begin
                   LoadTexture(Dir+p[1],Texture[2].GL_ID);
                                        Texture[2].used :=true;
                 end;

             if SameText(P[0],'BLEND')             then Blend.Enabled := SameText(P[1],'TRUE');
            end;
        4: begin
           end;
      end;

   until eof(f);
  CloseFile(f);


   if (Use_Shader <> '') then
     Shader    := Get_Shader(Use_Shader);

     Colors[0]       := vec3f(0.2,0.2,0.2);
     Colors[1]       := vec3f(0.8,0.8,0.8);
     Colors[2]       := vec3f(1,1,1);
     Colors[3]       := vec3f(0.1,0.1,0.1);
   end;

       GL_USE_MipMap   := m;


end;

Function LoadMatarial(FileName:String):Integer;
var i : integer;
begin
  for i := 0 to Length(MaterialList) - 1 do
  if SameText(MaterialList[i]^.Name,FileName) then
  begin Result:=i; exit; end;

     i := Length(MaterialList);
       SetLength(MaterialList, i +1);
Result := I;

if fileexists(filename) then
     MaterialList[i] := CreateMaterialFromFile(FileName) else
      begin
       Engine.messagebox('Material file not found :'+#13+#10+ FileName,1);
       MaterialList[i] := CreateDefaultMaterial;
      end;

end;

Function RemoveMaterial(ID:Cardinal):boolean;
var i : integer;
begin
  for I := 0 to Tex_Count do
    DeleteTexture(MaterialList[id]^.Texture[i].GL_ID);

               MaterialList[id] :=  MaterialList[length(MaterialList)-1];
    setlength (MaterialList, length(MaterialList)-1);
end;

Function GetMaterial(ID:Cardinal):PMaterial;
begin
    Result := MaterialList[id];
end;

Procedure FreeMaterials;
var i, i2:integer;
begin
  for I  := 0 to Length(MaterialList) - 1 do
  for I2 := 0 to Tex_Count do
    DeleteTexture(MaterialList[i]^.Texture[i2].GL_ID);
    MaterialList := Nil;
end;

Function CreateDefaultMaterial:PMaterial;
begin
 New( Result);
 With Result^ do
 begin
   Colors[0]       := vec3f(0.2,0.2,0.2);
   Colors[1]       := vec3f(0.8,0.8,0.8);
   Colors[2]       := vec3f(1,1,1);
   Colors[3]       := vec3f(0.1,0.1,0.1);
   Blend.Enabled   := false;
   Blend.Factor[0] := GL_One;
   Blend.Factor[1] := GL_One;
   Blend.Alpha     := 1;
 end;
end;

Function BindMaterial( ID:Cardinal; RenderMode:TMaterialRenderMode):Cardinal;
const
       material_shininess: Array[0..0] of GLfloat = (32);
       black:array[0..3] of single=(0,0,0,0);

var
 MAT: PMaterial;
begin
 Mat := MaterialList[id];
 Result:=0;
 with Mat^ do
 begin
      LastMatrialID:=id;
 case RenderMode of
      MAT_Z_ONLY :  begin
                     glColor4fv(@black);
                    end;
      MAT_AMBIENT:  begin
                     glColor4fv(@GLOBAL_AMBIENT);
                     Texture_Enable(Texture[0].GL_ID,0);
                    end;
      MAT_LIGHTING: begin
                       glMaterialfv  (GL_FRONT,  GL_AMBIENT , @colors[0]);
                       glMaterialfv  (GL_FRONT,  GL_DIFFUSE , @colors[1]);
                       glMaterialfv  (GL_FRONT,  GL_SPECULAR, @colors[2]);
                       glMaterialfv  (GL_FRONT,  GL_EMISSION, @colors[3]);
                      // glMaterialfv(GL_FRONT, GL_SHININESS, @material_shininess[0]);
                       glLightModelfv(GL_LIGHT_MODEL_AMBIENT, @colors[0]);


                          Texture_Enable(Texture[0].GL_ID ,0);

                       if Texture[1].used then Texture_Enable(Texture[1].GL_ID,1);
                       if Texture[2].used then Texture_Enable(Texture[2].GL_ID,2);

                                (*
                       if Blend.Enabled then
                       begin
                         gldepthmask(false);
                         glenable(gl_blend);
                         gldisable(gl_cull_face);
            {$ifdef debugblend}
                         glBlendFunc(setblend(test[0]), SetBlend(test[1]));
            {$else}
                         glblendfunc(Blend.Factor[0] , Blend.Factor[1] );
            {$endif}

//                         GLColor3f(1,1,1);
                         GLColor4f(1,1,1,blend.Alpha)
                       end;
                         *)
                       CurShader := Shader;
                       if (shader <> nil) then
                       With Shader^ do
                       begin
                       Start;
                         GL_Shader_Tangent := tangent;
                       end;

                    end;

     end;
 end;
end;

Procedure DisableLastMaterial;
begin
       MaterialList[LastMatrialID]^.Shader^.Stop;
    if MaterialList[LastMatrialID]^.Blend.Enabled  then
        begin
        gldepthmask(true);
        GLDisable(gl_blend);
        end;

    Texture_Disable(2);
    Texture_Disable(1);
    Texture_Disable(0);
end;

Procedure DisableMaterials();
begin
   Stop_Shaders;

//    gldepthmask(true);
//    gldisable(gl_alpha_test);
//    glEnable(gl_cull_face);
//    GLDisable(gl_blend);

    Texture_Disable(2);
    Texture_Disable(1);
    Texture_Disable(0);
end;


Function SetBlend(I:Integer):Cardinal;
begin
 case i of
    0 : result:= GL_ZERO;
    1 : result:= GL_ONE;
    2 : result:= GL_SRC_COLOR;
    3 : result:= GL_ONE_MINUS_SRC_COLOR;
    4 : result:= GL_SRC_ALPHA;
    5 : result:= GL_ONE_MINUS_SRC_ALPHA;
    6 : result:= GL_DST_ALPHA;
    7 : result:= GL_ONE_MINUS_DST_ALPHA;
    8 : result:= GL_DST_COLOR;
    9 : result:= GL_ONE_MINUS_DST_COLOR;
    10: result:= GL_SRC_ALPHA_SATURATE;
  else  Result:= GL_One;
 end;
end;
{$EndRegion}

{$Region 'GL Save Texture To Bmp'}
Function SaveTexture(Fname:String;W, H ,bpp :Word;Data : PByteArray) : Integer;
var
 F : File Of Byte;

  BMP : packed record
    bfType          : Word;
    bfSize          : LongWord;
    bfReserved1     : Word;
    bfReserved2     : Word;
    bfOffBits       : LongWord;
    biSize          : LongWord;
    biWidth         : Integer;
    biHeight        : Integer;
    biPlanes        : Word;
    biBitCount      : Word;
    biCompression   : LongWord;
    biSizeImage     : LongWord;
    biXPelsPerMeter : Integer;
    biYPelsPerMeter : Integer;
    biClrUsed       : LongWord;
    biClrImportant  : LongWord;
  end;

begin
assignfile(f,Fname);
rewrite(f);

if bpp<3 then bpp:=3;
    with BMP do
    begin
      bfType          := $4D42;
      bfSize          := w * h * bpp + SizeOf(BMP);
      bfReserved1     := 0;
      bfReserved2     := 0;
      bfOffBits       := SizeOf(BMP);
      biSize          := SizeOf(BITMAPINFOHEADER);
      biWidth         := w;
      biHeight        := h;
      biPlanes        := 1;
      biBitCount      := bpp*8;
      biCompression   := 0;
      biSizeImage     := w * h * bpp;
      biXPelsPerMeter := 0;
      biYPelsPerMeter := 0;
      biClrUsed       := 0;
      biClrImportant  := 0;
      blockwrite(F, BMP, SizeOf(BMP));
    end;

    blockwrite(f,data^[0],w*h*bpp);
closefile(f);
end;

{$EndRegion}

{$Region 'Not nided'}
{
Procedure GenerateNormalMap(ID:Integer);
 var
    Y , X,tmp: Integer;

    XYPos: Array [0..5] of integer;
    FHeight, Height , len, dx, dy, nx, ny, nz : Single;
    xHeights : array [0..2] of single;
    const
    InvByte = 1 / 255;
    CHeight = 168;
    NormalMap = 1;
begin
 with mat.Maps[id] do
 begin
         mat.Maps[ NormalMap ].W   := w;
         mat.Maps[ NormalMap ].h   := h;
         mat.Maps[ NormalMap ].BPP := 3;

         Freemem (mat.Maps[ NormalMap ].data);

  GetMem(mat.Maps[ NormalMap ].data, w * h * 3 );


          fHeight := CHeight / 10;
       for y := 0 to H - 1 do
       begin
         XYPos[0] := (Y * W);
         XYPos[1] := ((y + 1) mod h) * W;

           for x := 0 to w - 1 do
           begin
             XYPos[2] := (x+XYPos[0]);

             tmp := XYPos[2]*3;
             xheights[0]  := (Data[ tmp ] + Data[ tmp+1 ] + Data[ tmp+2 ]) / 3;

             tmp := (XYPos[0] + (x+1) mod W) *3;
             xheights[1]  := ( Data[ tmp ] + Data[ tmp+1 ] + Data[ tmp+2 ] ) / 3;
             tmp := (XYPos[1] +  x) * 3;
             xheights[2]  := ( Data[ tmp ] + Data[ tmp+1 ] + Data[ tmp+2 ] ) / 3;

              Height  := (    XHeights[0]      * InvByte);

                   dx :=  (Height - XHeights[1] * InvByte) * FHeight;
                   dy :=  (Height - XHeights[2] * InvByte) * FHeight;

                  len := sqrt(dx * dx + dy * dy + 1);
               if len  = 0 then len := 1;

                   nz := 1 / len;
                   nx := dy * nz;
                   ny := dx * nz;

             mat.Maps[ NormalMap ].Data[ XYPos[2] *3 +2 ] := trunc((128 + 127 * nz) + 0.5);
             mat.Maps[ NormalMap ].Data[ XYPos[2] *3 +1 ] := trunc((128 + 127 * nx) + 0.5);
             mat.Maps[ NormalMap ].Data[ XYPos[2] *3 +0 ] := trunc((128 + 127 * ny) + 0.5);
           end;
       end;

          mat.Texture  [ NormalMap ] :=  CreateTexture(W,H, mat.Maps[ NormalMap ].data);

  end;
end;
}

{$EndRegion}
end.

