unit l_math;
{$Warnings off}
{$Hints On}
(*******************************)
(* SVSD_VAL                    *)
(* Last modify 01.03.08        *)
(*******************************)
(* Site : SVSD.MirGames.ru     *)
(* mail : ValDim_05@Mail.ru    *)
(*******************************)
interface

uses
 Windows;

const
 rad2deg      = 180/pi;
 deg2rad      = pi/180;
 Elasticity   = 0.05;
 Friction     = 0.05;
 M_PI         = 3.14159265358979323846;
 MATCH_FACTOR = 0.99888;
 M_Angle      = MATCH_FACTOR*(2*M_PI);
 V_Gravity    = 8*0.001;

type
 TVector2D = record
  X, Y : single;
 end;

 PVector = ^TVector;
 TVector = record
  X, Y, Z : single;
 end;

 TShaderVertex = record
    Tangent,               // COL0
    Binormal,              // COL1
    Normal: TVector;       // NRML
    Tex0: TVector2d;
    Pos: TVector;
 end;

 TVector3i = array[0..2] of Integer;

 TPlane = record
  Normal : TVector;
  V      : TVector;
  D      : single;
 end;

  TDistBool = record
    Dist: Single;
    Point,Vector: TVector;
    Constanta: SmallInt;
    Return: Boolean;
   end;

  function max(const x, y: single): single;
  function min(const x, y: single): single;

 // Math
  function ArcTan2(const Y, X: single): single;
  function ArcCos (const X: single): single;
  function Tan    (const X: Extended): Extended;

 // 3D Vector's
  function  Vector      (const X, Y, Z: single): TVector;
  procedure V_clear2    (var v: TVector);
  function  V_Clear     :TVector;

  function V_Add        (const v1, v2: TVector): TVector; assembler; register;
  function V_Sub        (const v1, v2: TVector): TVector; assembler; register;
  function V_Mult       (const v: TVector; const d: single): TVector; assembler; register;
  function V_Div        (const v: TVector; const d: single): TVector; assembler; register;
  function V_Dist       (const v1, v2: TVector): single;
  function V_Distq      (const v1, v2: TVector): single;
  function V_Length     (const v: TVector): single;
  function V_Normalize  (const v: TVector): TVector;
  function v_normalizesafe(const inv: TVector; out outv: TVector): Single;
  function V_Dot        (const v1, v2: TVector): single;
  function V_Cross      (const v1, v2: TVector): TVector;
  function V_Angle      (const v1, v2: TVector): single;
  function V_Interpolate(const v1, v2: TVector; const k: single): TVector;
  function V_MultV      (const A,B: TVector): TVector;assembler; register;
  function v_combine    (const v1, v2 : TVector; const scale : Single): TVector;
  function v_spacing    (const v1, v2: TVector): Single;

  Function v_RotateX    (const V : TVector; const ang : Single):TVector;
  Function v_RotateY    (const V : TVector; const ang : Single):TVector;
  Function v_RotateZ    (const V : TVector; const ang : Single):TVector;
  function v_Rotate     (const V,Rot:TVector):TVector;

// 2D Vector's
  function Vector2D     (const X, Y: single): TVector2D;
  function V_Rotate2D   (const v: TVector2D; const ang: single): TVector2D;
// Plane
  function P_Normal     (const v1, v2, v3: TVector): TVector;
  function P_Plane      (const v1, v2, v3: TVector): TPlane;
  function P_Plane2     (const Normal,v1: TVector): TPlane;
  function P_Dist       (const Pos: TVector; const Plane: TPlane): single;
  function P_Classify   (const Pos: TVector; const Plane: TPlane): single;
  function P_InsideTri  (const v, v1, v2, v3: TVector): boolean;
///////
  function EdgeSphereCollision(const Center,v1, v2, v3: TVector; const Radius: Single): Boolean;
  function ClosestPointOnLine (const vA, vB, Point: TVector): TVector;
///////
  function V_Min              (const v1,v2: TVector): boolean;
  Function VAABBMax     (const V1,V2:TVector):TVector;
  Function VAABBMin     (const V1,V2:TVector):TVector;
  function V_nan        (const V,V2:TVector):TVector;
  function IsNaN        (const v : single) : boolean;
  Function ChangeAcc(const Acc,Normal:TVector):TVector;

//////
  Function  Plane_Intersection(const plane: TPlane; const LineStart, LineEnd: TVector): boolean;
  Function  Plane_GetIntersection(const plane: TPlane; const LineStart, LineEnd: TVector): TVector;
  function  LineVsPolygon (const v1,v2,v3, LB,LE,vNormal : TVector):boolean;
  function  LineVsPolygon2(const v1,v2,v3, LB,LE,vNormal : TVector;Var vInt : TVector):boolean;

  Function uCubeVsPoint   (const Box,Point :TVector; Size:Single):Boolean;
  Function uBoxVsPoint    (const Box,BoxSize,Point :TVector):Boolean;
  function uAABBVsPoint   (const Minx,Maxx,Pos : TVector): boolean;
  function uBoxVsBox      (const Box1,Box2, Box1Size,Box2Size :TVector): boolean;
  Function uAABBVsAABB    (const Minx1,Maxx1,Minx2,Maxx2:TVector): boolean;
  function uAABBVsSphere  (const Minx,Maxx,Pos : TVector;R:Single): boolean;
  function uCubeVsLine    (const BP,LBEGIN,LEND : TVector;BS :single): boolean;
  function uAABBVsLine    (const Mins,Maxs,StartPoint, EndPoint : TVector): boolean;
  function uSphereVsPoint (const Sphere ,Point : TVector; R: Single): boolean;
  Function uSphereVsSphere(const Sphere1,Sphere2 : TVector; R1,R2 : Single) : boolean;
  function uSphereVsLine  (const Sphere ,LB,LE : TVector;R: Single): boolean;
  function uSphereVsLine2 (const Sphere, P1,P2 : TVector;R: Single): boolean;

////////////
  //
  procedure CalculateTandB      (var T, B: TVector; const st1, st2: TVector2d; const P, Q: TVector);
  procedure CalculateTangents   (var v1, v2, v3: TShaderVertex);
  function  dotVecLengthSquared3(v: TVector): Single;
  procedure dotVecNormalize3    (var v: TVector);

implementation

Function ChangeAcc(const Acc,Normal:TVector):TVector;
Var
D    : Single;
VN,VT: TVector;
begin

D := (acc.x * Normal.x) + (acc.y * Normal.y) + (acc.z * Normal.z);
     Vn := V_Mult(Normal,D);
     Vt := V_Sub (Acc,Vn);
 Result := v_add (v_mult(Vt ,(1.0 - Friction) ),v_mult(Vn, -Elasticity));
end;

function IsNaN(const v : single) : boolean;
asm
  fld   dword ptr v    // 0: v
  fxam                 // see what we've got, C0 set if NaN
  fstsw ax             // store C0-C3 in ax,
  sahf                 //   then in EFLAGS register: C0 -> CF
  mov   Result,0       // assume false
  jae   @NoNaN         // jump if CF not set
  mov   Result,1       // set result to true
@NoNaN:
  ffree st(0)          // clear FPU register
end;

function max(const x, y: single): single;
begin
if x > y then
 Result := x
else
 Result := y;
end;

function min(const x, y: single): single;
begin
if x < y then
 Result := x
else
 Result := y;
end;

function Tan(const X: Extended): Extended;
{  Tan := Sin(X) / Cos(X) }
asm
        FLD    X
        FPTAN
        FSTP   ST(0)      { FPTAN pushes 1.0 after result }
        FWAIT
end;

(***      ***)
function ArcTan2(const Y, X: single): single;
asm
 FLD     Y
 FLD     X
 FPATAN
 FWAIT
end;

function ArcCos(const X: single): single;
asm
{
Result := ArcTan2(Sqrt(1 - X * X), X);
}
       FLD     X
       fmul    ST(0),ST(0)
       FLD1
       FSub    st(0), st(1)
       FSQRT
       FLD     X
       FPATAN
       FWAIT
       FSTP ST(1)
end;

(***      ***)

//  
function Vector(const X, Y, Z: single): TVector;
begin
Result.X := X;
Result.Y := Y;
Result.Z := Z;
end;

Function V_Clear: TVector;
begin
  ZeroMemory(@Result, SizeOf(TVector));
end;

procedure v_clear2(var v: TVector);
asm
  XOR EDX, EDX
  MOV [EAX], EDX
  MOV [EAX+4], EDX
  MOV [EAX+8], EDX
end;

{------------------------------------------------------------------------------}
function V_Min(const v1,v2: TVector): boolean;
begin
if (v1.X < v2.X) and
   (v1.Y < v2.Y) and
   (v1.Z < v2.Z) then result:=true else result:=false;
end;

Function VAABBMin(const V1,V2:TVector):TVector;
begin
if (v1.x < v2.X) then result.x := v1.X else result.x := v2.X;
if (v1.y < v2.y) then result.y := v1.y else result.y := v2.y;
if (v1.z < v2.z) then result.z := v1.z else result.z := v2.z;
end;

Function VAABBMax(const V1,V2:TVector):TVector;
begin
if (v1.x > v2.X) then result.x := v1.X else result.x := v2.X;
if (v1.y > v2.y) then result.y := v1.y else result.y := v2.y;
if (v1.z > v2.z) then result.z := v1.z else result.z := v2.z;
end;

function V_nan(const V,V2:TVector):TVector;
begin
    asm
      mov [ecx  ], 0
      mov [ecx+4], 0
      mov [ecx+8], 0
    end;
if not isnan(v.x) then result.x :=v.x else result.x :=v2.x;
if not isnan(v.y) then result.y :=v.y else result.y :=v2.y;
if not isnan(v.z) then result.z :=v.z else result.z :=v2.z;
end;
{------------------------------------------------------------------------------}

function V_Sub(const V1, V2: TVector): TVector; assembler; register;
{Result.X := V1.X-V2.X;
 Result.Y := V1.Y-V2.Y;
 Result.Z := V1.Z-V2.Z;}
asm
      FLD DWORD PTR [EAX]    // V1.X
      FSUB DWORD PTR [EDX]   // - V2.X
      FSTP DWORD PTR [ECX]   // Result :=
      FLD DWORD PTR [EAX + 4]
      FSUB DWORD PTR [EDX + 4]
      FSTP DWORD PTR [ECX + 4]
      FLD DWORD PTR [EAX + 8]
      FSUB DWORD PTR [EDX + 8]
      FSTP DWORD PTR [ECX + 8]
end;

function V_ADD(const V1, V2: TVector): TVector; assembler; register;
{Result.X := V1.X+V2.X;
 Result.Y := V1.Y+V2.Y;
 Result.Z := V1.Z+V2.Z;}
asm

      FLD DWORD PTR [EAX]
      FADD DWORD PTR [EDX]
      FSTP DWORD PTR [ECX]
      FLD DWORD PTR [EAX + 4]
      FADD DWORD PTR [EDX + 4]
      FSTP DWORD PTR [ECX + 4]
      FLD DWORD PTR [EAX + 8]
      FADD DWORD PTR [EDX + 8]
      FSTP DWORD PTR [ECX + 8]
end;
function v_mult(const v: TVector; const D: Single): TVector;
{Result.X := V.X*D;
 Result.Y := V.Y*D;
 Result.Z := V.Z*D;}
asm
      FLD  DWORD PTR [EAX]
      FMUL DWORD PTR [EBP+8]
      FSTP DWORD PTR [EDX]
      FLD  DWORD PTR [EAX+4]
      FMUL DWORD PTR [EBP+8]
      FSTP DWORD PTR [EDX+4]
      FLD  DWORD PTR [EAX+8]
      FMUL DWORD PTR [EBP+8]
      FSTP DWORD PTR [EDX+8]
end;

function V_MultV(const A, B: TVector): TVector; assembler; register;
{Result.X := A.X*B.X;
 Result.Y := A.Y*B.Y;
 Result.Z := A.Z*B.Z;}
asm
      FLD DWORD PTR [EAX]
      FMul DWORD PTR [EDX]
      FSTP DWORD PTR [ECX]
      FLD DWORD PTR [EAX + 4]
      FMul DWORD PTR [EDX + 4]
      FSTP DWORD PTR [ECX + 4]
      FLD DWORD PTR [EAX + 8]
      FMul DWORD PTR [EDX + 8]
      FSTP DWORD PTR [ECX + 8]
end;
function v_dot(const v1, v2: TVector): Single;
// EAX contains address of V1
// EDX contains address of V2
// ST(0) contains the result
{Result := v1.X * v2.X + v1.Y * v2.Y + v1.Z * v2.Z;}
asm
      FLD DWORD PTR [EAX]
      FMUL DWORD PTR [EDX]
      FLD DWORD PTR [EAX + 4]
      FMUL DWORD PTR [EDX + 4]
      FADDP
      FLD DWORD PTR [EAX + 8]
      FMUL DWORD PTR [EDX + 8]
      FADDP
end;

function V_Div(Const V: TVector;Const D:Single): TVector; assembler; register;
{if d=0 then exit;
 Result.X := V.X/D;
 Result.Y := V.Y/D;
 Result.Z := V.Z/D;}
asm
      mov [edx  ], 0
      mov [edx+4], 0
      mov [edx+8], 0
      cmp [ebp+8],0
          jz @@exit1

      FLD  DWORD PTR [EAX]
      Fdiv DWORD PTR [EBP+8]
      FSTP DWORD PTR [EDX]
      FLD  DWORD PTR [EAX+4]
      Fdiv DWORD PTR [EBP+8]
      FSTP DWORD PTR [EDX+4]
      FLD  DWORD PTR [EAX+8]
      Fdiv DWORD PTR [EBP+8]
      FSTP DWORD PTR [EDX+8]
 @@exit1:
end;

function V_Cross(Const V1, V2: TVector): TVector;
{Result.X := v1.Y * v2.Z - v1.Z * v2.Y;
 Result.Y := v1.Z * v2.X - v1.X * v2.Z;
 Result.Z := v1.X * v2.Y - v1.Y * v2.X;}
var Temp: TVector;
asm

      PUSH EBX                      //  EBX,   v    
      LEA EBX, [Temp]
      FLD DWORD PTR [EDX + 8]       // C      FPU
      FLD DWORD PTR [EDX + 4]
      FLD DWORD PTR [EDX + 0]
      FLD DWORD PTR [EAX + 8]
      FLD DWORD PTR [EAX + 4]
      FLD DWORD PTR [EAX + 0]

      FLD ST(1)                     // ST(0) := V1.Y
      FMUL ST, ST(6)                // ST(0) := V1.Y * V2.Z
      FLD ST(3)                     // ST(0) := V1.Z
      FMUL ST, ST(6)                // ST(0) := V1.Z * V2.Y
      FSUBP ST(1), ST               // ST(0) := ST(1)-ST(0)
      FSTP DWORD [EBX]              // Temp.X := ST(0)
      FLD ST(2)                     // ST(0) := V1.Z
      FMUL ST, ST(4)                // ST(0) := V1.Z * V2.X
      FLD ST(1)                     // ST(0) := V1.X
      FMUL ST, ST(7)                // ST(0) := V1.X * V2.Z
      FSUBP ST(1), ST               // ST(0) := ST(1)-ST(0)
      FSTP DWORD [EBX + 4]          // Temp.Y := ST(0)
      FLD ST                        // ST(0) := V1.X
      FMUL ST, ST(5)                // ST(0) := V1.X * V2.Y
      FLD ST(2)                     // ST(0) := V1.Y
      FMUL ST, ST(5)                // ST(0) := V1.Y * V2.X
      FSUBP ST(1), ST               // ST(0) := ST(1)-ST(0)
      FSTP DWORD [EBX + 8]          // Temp.Z := ST(0)
      FSTP ST(0)                    //   FPU
      FSTP ST(0)
      FSTP ST(0)
      FSTP ST(0)
      FSTP ST(0)
      FSTP ST(0)
      MOV EAX, [EBX]                //    Temp  Result
      MOV [ECX], EAX
      MOV EAX, [EBX + 4]
      MOV [ECX + 4], EAX
      MOV EAX, [EBX + 8]
      MOV [ECX + 8], EAX
      POP EBX
end;
(*
function V_Length(Const V: tvector): Single; assembler;
{Result := sqrt(sqr(v.X) + sqr(v.Y) + sqr(v.Z));}
asm
        mov edx, 2
        FLDZ
@@Loop: FLD  DWORD PTR [EAX  +  4 * EDX]
        FMUL ST, ST
        FADDP
        SUB  EDX, 1
        JNL  @@Loop
        FSQRT
end;
*)
function v_length(const v: TVector): Single;
// EAX contains address of V
// result is passed in ST(0)
{Result := sqrt(sqr(v.X) + sqr(v.Y) + sqr(v.Z));}
asm
      FLD  DWORD PTR [EAX]
      FMUL ST, ST
      FLD  DWORD PTR [EAX+4]
      FMUL ST, ST
      FADDP
      FLD  DWORD PTR [EAX+8]
      FMUL ST, ST
      FADDP
      FSQRT
end;

//    
function V_Dist(const v1, v2: TVector): single;
// EAX contains address of v1
// EDX contains highest of v2
// Result  is passed on the stack
// Result := sqrt(sqr(v2.X - v1.X) + sqr(v2.Y - v1.Y) +sqr(v2.Z - v1.Z));
asm
      FLD  DWORD PTR [EAX]
      FSUB DWORD PTR [EDX]
      FMUL ST, ST
      FLD  DWORD PTR [EAX+4]
      FSUB DWORD PTR [EDX+4]
      FMUL ST, ST
      FADD
      FLD  DWORD PTR [EAX+8]
      FSUB DWORD PTR [EDX+8]
      FMUL ST, ST
      FADD
      FSQRT
end;

//     
//    
function V_Distq(const v1, v2: TVector): single;
// EAX contains address of v1
// EDX contains highest of v2
// Result  is passed on the stack
// Result := sqr(v2.X - v1.X) + sqr(v2.Y - v1.Y) +sqr(v2.Z - v1.Z);
asm
      FLD  DWORD PTR [EAX]
      FSUB DWORD PTR [EDX]
      FMUL ST, ST
      FLD  DWORD PTR [EAX+4]
      FSUB DWORD PTR [EDX+4]
      FMUL ST, ST
      FADD
      FLD  DWORD PTR [EAX+8]
      FSUB DWORD PTR [EDX+8]
      FMUL ST, ST
      FADD
end;


function v_spacing(const v1, v2: TVector): Single;
// EAX contains address of v1
// EDX contains highest of v2
// Result  is passed on the stack
// Result:=Abs(v2.x-v1.x)+Abs(v2.y-v1.y)+Abs(v2.z-v1.z);
asm
      FLD  DWORD PTR [EAX]
      FSUB DWORD PTR [EDX]
      FABS
      FLD  DWORD PTR [EAX+4]
      FSUB DWORD PTR [EDX+4]
      FABS
      FADD
      FLD  DWORD PTR [EAX+8]
      FSUB DWORD PTR [EDX+8]
      FABS
      FADD
end;

function v_combine(const v1, v2 : TVector; const scale : Single): TVector;
// EAX contains address of v1
// EDX contains address of v2
// EBP+8 contains address of scale
// ECX contains address of Result
asm
      FLD  DWORD PTR [EDX]
      FMUL DWORD PTR [EBP+8]
      FADD DWORD PTR [EAX]
      FSTP DWORD PTR [ECX]

      FLD  DWORD PTR [EDX+4]
      FMUL DWORD PTR [EBP+8]
      FADD DWORD PTR [EAX+4]
      FSTP DWORD PTR [ECX+4]

      FLD  DWORD PTR [EDX+8]
      FMUL DWORD PTR [EBP+8]
      FADD DWORD PTR [EAX+8]
      FSTP DWORD PTR [ECX+8]
end;

function V_Normalize(const v: TVector): TVector;
// EAX contains address of V1
// EDX contains the result
// Result:=v * (1 / (v * v) )
{
var s : Extended;
begin
s :=  1 / sqrt(sqr(v.X) + sqr(v.Y) + sqr(v.Z));
Result.x := v.x * s;
Result.y := v.y * s;
Result.z := v.z * s;
end;
}
asm
      FLD  DWORD PTR [EAX]
      FMUL ST, ST
      FLD  DWORD PTR [EAX+4]
      FMUL ST, ST
      FADD
      FLD  DWORD PTR [EAX+8]
      FMUL ST, ST
      FADD
      FSQRT

      FLD1
      FDIVR
      FLD  ST
      FMUL DWORD PTR [EAX]
      FSTP DWORD PTR [EDX]
      FLD  ST
      FMUL DWORD PTR [EAX+4]
      FSTP DWORD PTR [EDX+4]
      FMUL DWORD PTR [EAX+8]
      FSTP DWORD PTR [EDX+8]
end;

function v_normalizesafe(const inv: TVector; out outv: TVector): Single;
var len: Single;
begin
  len := v_length(inv);
  if len = 0 then
  begin
    v_clear2(outv);
    Result := 0;
    Exit;
  end;

  Result := len;
  len    := 1/len;
  outv   := v_mult(inv,len);
end;

function V_Angle(const v1, v2: TVector): single;
begin
Result := ArcCos(V_Dot(V_Normalize(v1), V_Normalize(v2)));
end;

function V_Interpolate(const v1, v2: TVector; const k: single): TVector;
begin
Result.X := v1.X + (v2.X - v1.X)/k;
Result.Y := v1.Y + (v2.Y - v1.Y)/k;
Result.Z := v1.Z + (v2.Z - v1.Z)/k;
end;


Function v_RotateX(const V : TVector; const ang : Single):TVector;
var
  radAng : Single;
begin
  radAng := ang * (PI / 180);

  Result.x := v.x;
  Result.y := (v.y * cos(radAng)) - (v.z * sin(radAng));
  Result.z := (v.y * sin(radAng)) + (v.z * cos(radAng));
end;

Function v_RotateY(const V : TVector; const ang : Single):TVector;
var
  radAng : Single;
begin
  radAng := ang * (PI / 180);

  Result.y := v.y;
  Result.x := (v.x * cos(radAng)) - (v.z * sin(radAng));
  Result.z := (v.z * sin(radAng)) + (v.x * cos(radAng));
end;

Function v_RotateZ(const V : TVector; const ang : Single):TVector;
var
  radAng : Single;
begin
  radAng := ang * (PI / 180);

  Result.z := v.z;
  Result.x := (v.x * cos(radAng)) - (v.y * sin(radAng));
  Result.y := (v.y * sin(radAng)) + (v.x * cos(radAng));
end;

function v_Rotate(const V,Rot:TVector):TVector;
begin
 Result := v_RotateZ(v_RotateY(v_RotateX(V,Rot.x),rot.y),rot.z);
end;

// 2D Vector's
function Vector2D(const X, Y: single): TVector2D;
begin
Result.X := X;
Result.Y := Y;
end;

function V_Rotate2D(const v: TVector2D; const ang: single): TVector2D;
var
 l : single;
begin
l := sqrt(sqr(v.X) + sqr(v.Y));
Result.X := cos(ang) * l;
Result.Y := sin(ang) * l;
end;

// Plane
function P_Normal(const v1, v2, v3: TVector): TVector;
begin
Result := V_Normalize(V_Cross( V_Sub(v3, v2), V_Sub(v2, v1)  ) ) ;
end;

function P_Plane(const v1, v2, v3: TVector): TPlane;
begin
Result.Normal := P_Normal(v1, v2, v3);
Result.D      := -V_Dot(Result.Normal, v1);
Result.V      := v1;
end;

function P_Plane2(const Normal,v1: TVector): TPlane;
begin
result.Normal := Normal;
result.d      := -V_Dot(Normal, v1);
Result.V      := v1;
end;

function P_Dist(const Pos: TVector; const Plane: TPlane): single;
begin
Result := (V_Dot(Plane.Normal, Pos) + Plane.D);
end;

function P_Classify(const Pos: TVector; const Plane: TPlane): single;
begin
Result := Plane.Normal.X * (pos.x - plane.V.x) + Plane.Normal.Y * (pos.y - plane.V.y) + Plane.Normal.Z * (pos.z - plane.V.z);
//Result := V_Dot(Plane.Normal, V_Sub(Pos, Plane.V));

end;

function P_InsideTri(const v, v1, v2, v3: TVector): boolean;
Const MATCH_FACTOR = 0.999;
var
 i      : Integer;
 Angle  : Double;
 vec    : array [0..2] of TVector;
begin
Result := False;
vec[0] := V_Sub(v1, v);
vec[1] := V_Sub(v2, v);
vec[2] := V_Sub(v3, v);
Angle := 0;
for i := 0 to 2 do
 Angle := Angle + V_Angle(vec[i], vec[(i + 1) mod 3]);
	if(Angle >= (MATCH_FACTOR * (1.999 * PI)) )	then
 Result := true;
end;


function ClosestPointOnLine(const vA, vB, Point: TVector): TVector;
var
 Vector1, Vector2, Vector3: TVector;
 d, t: double;
begin
Vector1 := V_Sub(Point, vA);
Vector2 := V_Normalize(V_Sub(vB, vA));
d := V_Dist(vA, vB);
t := V_Dot(Vector2, Vector1);

if t <= 0 then
 begin
 Result := vA;
 Exit;
 end;

if t >= d then
 begin
 Result := vB;
 Exit;
 end;

Vector3 := V_Mult(Vector2, t);
Result := V_Add(vA, Vector3);
end;

function EdgeSphereCollision(const Center,v1, v2, v3: TVector; const Radius: Single): Boolean;
var i: Integer;
    Point: TVector;
    Distance: Single;
 v : array [0..2] of TVector;
begin
v[0] := v1;
v[1] := v2;
v[2] := v3;
for i := 0 to 2 do
 begin
 Point := ClosestPointOnLine(v[i], v[(i + 1) mod 3], Center);
 Distance := V_Dist(Point, Center);
 if Distance < Radius then
  begin
  Result := true;
  Exit;
  end;
 end;
Result := false;
end;

Function  Plane_Intersection(const plane: TPlane; const LineStart, LineEnd: TVector): boolean;
var
	Sign , Sign1 : Single;
begin
	sign  := P_Dist( LineStart , plane);
	sign1 := P_Dist( LineEnd   , plane);
	result := ( ((sign < 0) and (sign1 > 0)) or ((sign > 0) and (sign1 <=0)) );
end;

Function  Plane_GetIntersection(const plane: TPlane; const LineStart, LineEnd: TVector): TVector;
var
  ptDirection : TVector;
  denominator : single;
  D           : single;
begin
	ptDirection := V_sub( LineEnd, LineStart );
	denominator := V_Dot( plane.Normal, ptDirection );
        Result:=LineEnd;

	if denominator = 0 then exit;

	D := ( plane.D - V_Dot( plane.normal, LineStart )) / denominator;
	ptDirection := V_mult( ptDirection,  D);
	result      := V_add(LineStart, ptDirection);
end;

function LineVsPolygon(const v1,v2,v3, LB,LE,vNormal : TVector):boolean;
var
  vIntersection : TVector;
  originDistance : Extended;
  distance1 : Extended;
  distance2 : Extended;
  vVector1 : TVector;
  vVector2 : TVector;
  vPoint : TVector;
  vLineDir : TVector;
  Numerator : Extended;
  Denominator : Extended;
  dist : Extended;
  Angle,tempangle : Extended; // Initialize the angle
	vA, vB : TVector;						// Create temp vectors
  I : integer;
  dotProduct : Extended;
  vectorsMagnitude : Extended;
  vPoly            : array [0..2] of TVector;
  verticeCount : integer;
begin
  vPoint  := vector(0,0,0);
  vLineDir:= vector(0,0,0);
  Angle := 0;
  vpoly[0] := v1;
  vpoly[1] := v2;
  vpoly[2] := v3;
  verticeCount:=3;

  vVector1 := v_sub(vPoly[2],vPoly[0]);
  vVector2 := v_sub(vPoly[1],vPoly[0]);

  originDistance := -1 * V_Dot(vNormal,vPoly[0]);

	distance1 := V_Dot(vNormal,lb) + originDistance;// Cz + D
	distance2 := V_Dot(vNormal,le) + originDistance;// Cz + D
	if(distance1 * distance2 >= 0) then
  begin
	  result := false;
    exit;
  end;
  vLineDir := v_sub(le,lb);    // Get the X value of our new vector
  v_normalize(vLineDir);
	Numerator := -1 * (V_Dot(vNormal,lb) + originDistance);
	Denominator := V_Dot(vNormal,vLineDir);

	if( Denominator = 0) then	 // Check so we don't divide by zero
  begin
		vIntersection := lb; // Return an arbitrary point on the line
  end
  else
  begin

  	dist := Numerator / Denominator;
  	vPoint := V_ADD(lb,V_Mult(vLineDir,dist));
  	vIntersection := vPoint;								// Return the intersection point
  end;

  // Go in a circle to each vertex and get the angle between
  for i := 0 to verticeCount-1 do
  begin

    vA := v_sub(vPoly[i],vIntersection);
    vB := V_sub(vPoly[(i + 1) mod verticeCount],vIntersection);

    dotProduct       := V_Dot(vA,vB);
	  vectorsMagnitude := V_Length(VA)* V_Length(VB);
    tempangle        := arccos( dotProduct / vectorsMagnitude );
	  if(isnan(tempangle)) then
  		tempangle := 0;
	  Angle := Angle + tempangle;
  end;

	if(Angle >= (M_Angle) ) then
  begin
		result := TRUE;
    exit;
  end;

	result := false; // There was no collision, so return false
end;

function LineVsPolygon2(const v1,v2,v3, LB,LE,vNormal : TVector;Var vInt : TVector):boolean;
var
  vIntersection : TVector;
  originDistance : Extended;
  distance1 : Extended;
  distance2 : Extended;
  vVector1 : TVector;
  vVector2 : TVector;
  vPoint : TVector;
  vLineDir : TVector;
  Numerator : Extended;
  Denominator : Extended;
  dist : Extended;
  Angle,tempangle : Extended; // Initialize the angle
	vA, vB : TVector;						// Create temp vectors
  I : integer;
  dotProduct : Extended;
  vectorsMagnitude : Extended;
  vPoly            : array [0..2] of TVector;
  verticeCount : integer;
begin
  vint:=vector(0,0,0);
  vPoint  := vector(0,0,0);
  vLineDir:= vector(0,0,0);
  Angle := 0;
  vpoly[0] := v1;
  vpoly[1] := v2;
  vpoly[2] := v3;
  verticeCount:=3;

  vVector1 := v_sub(vPoly[2],vPoly[0]);
  vVector2 := v_sub(vPoly[1],vPoly[0]);

  originDistance := -1 * V_Dot(vNormal,vPoly[0]);

	distance1 := V_Dot(vNormal,lb) + originDistance;// Cz + D
	distance2 := V_Dot(vNormal,le) + originDistance;// Cz + D
	if(distance1 * distance2 >= 0) then
  begin
	  result := false;
    exit;
  end;
  vLineDir := v_sub(le,lb);    // Get the X value of our new vector
  v_normalize(vLineDir);
	Numerator := -1 * (V_Dot(vNormal,lb) + originDistance);
	Denominator := V_Dot(vNormal,vLineDir);

	if( Denominator = 0) then	 // Check so we don't divide by zero
  begin
		vIntersection := lb; // Return an arbitrary point on the line
  end
  else
  begin

  	dist := Numerator / Denominator;
  	vPoint := V_ADD(lb,V_Mult(vLineDir,dist));
  	vIntersection := vPoint;								// Return the intersection point
//  vint := vIntersection;
  end;

  // Go in a circle to each vertex and get the angle between
  for i := 0 to verticeCount-1 do
  begin

    vA := v_sub(vPoly[i],vIntersection);
    vB := V_sub(vPoly[(i + 1) mod verticeCount],vIntersection);

    dotProduct       := V_Dot(vA,vB);
	  vectorsMagnitude := V_Length(VA)* V_Length(VB);
    tempangle        := arccos( dotProduct / vectorsMagnitude );
	  if(isnan(tempangle)) then
  		tempangle := 0;
	  Angle := Angle + tempangle;
  end;

	if(Angle >= (M_Angle) ) then
  begin
		result := TRUE;
    vint := vIntersection;
    exit;
  end;
  vInt := vector(0,0,0);
	result := false; // There was no collision, so return false
end;

////////////////////////////// LINE POLYGON COLLISION \\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
Function uCubeVsPoint(const Box,Point :TVector; Size:Single):Boolean;
Begin
  If (abs(Point.x - Box.x)< Size) And
     (abs(Point.y - Box.y)< Size) And
     (abs(Point.z - Box.z)< Size) Then Result := True Else Result := False;
end;

Function uBoxVsPoint(const Box,BoxSize,Point :TVector):Boolean;
Begin
  IF (abs(Point.x - Box.x)< BoxSize.x) And
     (abs(Point.y - Box.y)< BoxSize.y) And
     (abs(Point.z - Box.z)< BoxSize.z) Then Result := True Else Result := False;
end;


function uAABBVsPoint(const Minx,Maxx,Pos :TVector): boolean;
begin
IF (Pos.X >= Minx.X) and (Pos.X <= Maxx.X) and
   (Pos.y >= Minx.y) and (Pos.y <= Maxx.y) and
   (Pos.z >= Minx.z) and (Pos.z <= Maxx.z) then result := true else result:=false;
end;

function uBoxVsBox(const Box1,Box2,
                    Box1Size,Box2Size :TVector): boolean;
begin
 if (box1.X + box1size.X < Box2.X) or
    (box1.y + box1size.y < Box2.y) or
    (box1.z + box1size.z < Box2.z) or

    (Box2.x + box2size.X < box1.X) or
    (Box2.y + box2size.y < box1.y) or
    (Box2.z + box2size.z < box1.z) then
     Result:=false else result:=true;
end;

Function uAABBVsAABB(const Minx1,Maxx1,Minx2,Maxx2:TVector): boolean;
begin
result:=true;
		if (Minx1.x > Maxx2.x) or (Maxx1.x < Minx2.x) or (Minx1.y > Maxx2.y) or
  		 (Maxx1.y < Minx2.y) or (Minx1.z > Maxx2.z) or (Maxx1.z < Minx2.z) then result:=false;
end;


function uAABBVsSphere(const Minx,Maxx,Pos :TVector;R:Single): boolean;
var
d :single;
begin
   d := 0;

      //      AABB,
      if (Pos.x < Minx.x) then
         //      
         d:=d+ abs(Pos.x - Minx.x);

      //      AABB,
      if (pos.x > Maxx.x) then
          //      
          d:=d+ abs(Pos.x - Maxx.x);

(******************************************************************************)
      if (Pos.y < Minx.y) then d :=d + abs(Pos.y - Minx.y);
      if (pos.y > Maxx.y) then d :=d + abs(Pos.y - Maxx.y);
(******************************************************************************)
      if (Pos.z < Minx.z) then d :=d + abs(Pos.z - Minx.z);
      if (pos.z > Maxx.z) then d :=d + abs(Pos.z - Maxx.z);
(******************************************************************************)

   result := (d  <=  R);

end;


function uCubeVsLine(const BP,LBEGIN,LEND:TVector;BS :single): boolean;
Var
  MID,
  DIR,
  T   : TVector;
  HL,
  R   : Single;

begin
  //  
  Mid.x := lbegin.x+(lend.x-lbegin.x)/2;
  Mid.y := lbegin.y+(lend.y-lbegin.y)/2;
  Mid.z := lbegin.z+(lend.z-lbegin.z)/2;

  //  
  dir.x := (lend.x-lbegin.x);
  dir.y := (lend.y-lbegin.y);
  dir.z := (lend.z-lbegin.z);

  //  
  hl := sqrt(sqr(dir.x)+sqr(dir.y)+sqr(dir.z));
  //  
  if hl <> 0 then
  begin
    dir.x := dir.x / hl;
    dir.y := dir.y / hl;
    dir.z := dir.z / hl;

    hl    := hl / 2;
  end;

  //      
   t.x := BP.x -mid.x;
   t.y := BP.y -mid.y;
   t.z := BP.z -mid.z;

    // ,      X,Y,Z 
   if ( (abs(T.x) > BS + hl*abs(dir.x)) or
        (abs(T.y) > BS + hl*abs(dir.y)) or
        (abs(T.z) > BS + hl*abs(dir.z)) ) then begin result := false ; exit; end;

   //  X ^ dir
    r := BS*abs(dir.z) + BS*abs(dir.y);
    if ( abs(T.y*dir.z - T.z*dir.y) > r ) then begin result := false ; exit; end;

   //  Y ^ dir
    r := BS* abs(dir.z) + BS* abs(dir.x);
    if ( abs(T.z*dir.x - T.x*dir.z) > r ) then begin result := false ; exit; end;

   //  Z ^ dir
    r := BS*abs(dir.y) + BS*abs(dir.x);
    if ( abs(T.x*dir.y - T.y*dir.x) > r ) then begin result := false ; exit; end;

   result := true;

end;

function uAABBVsLine(const Mins,Maxs,StartPoint, EndPoint : TVector): boolean;
var
  dir, lineDir, ld, lineCenter, center, extents, cross: TVector;
begin
  Result := False;

  center := v_mult(v_add(Mins, Maxs), 0.5);
  extents := v_sub(Maxs, center);
  lineDir := v_mult(v_sub(EndPoint, StartPoint), 0.5);
  lineCenter := v_add(StartPoint, lineDir);
  dir := v_sub(lineCenter, center);

  ld.x := Abs(lineDir.x);
  if Abs(dir.x) > (extents.x + ld.x) then Exit;

  ld.y := Abs(lineDir.y);
  if Abs(dir.y) > (extents.y + ld.y) then Exit;

  ld.z := Abs(lineDir.z);
  if Abs(dir.z) > (extents.z + ld.z) then Exit;

  cross := v_cross(lineDir, dir);

  if Abs(cross.x) > ((extents.y * ld.z) + (extents.z * ld.y)) then Exit;
  if Abs(cross.y) > ((extents.x * ld.z) + (extents.z * ld.x)) then Exit;
  if Abs(cross.z) > ((extents.x * ld.y) + (extents.y * ld.x)) then Exit;

  Result := True;
end;


function uSphereVsPoint(const Sphere,Point :TVector; R: Single): boolean;
Var
  dist: Single;
begin
//     
  dist:=sqrt(sqr(Sphere.X-Point.X)+sqr(Sphere.Y-Point.Y)+sqr(Sphere.Z-Point.Z));
//        
  if dist<=R then result:=true else result:=false;
end;

Function uSphereVsSphere(const Sphere1,Sphere2 : TVector; R1,R2 : Single) : boolean;
Var
R,RR  : Single;
X,Y,Z : Single;
begin
  R:= (R1 + R2);
  //   
  X := Sphere2.x - Sphere1.X;
  Y := Sphere2.Y - Sphere1.Y;
  Z := Sphere2.Z - Sphere1.Z;

  rr := sqrt(x*x +y*y +z*z);
  if rr < r then Result := True else Result := false;
  //  r  rr   
end;

function uSphereVsLine(const Sphere, LB,LE : TVector;R: Single): boolean;
var
 Point,
 Vector1,
 Vector2,
 Dir    : TVector;
 D, T   : Single;
begin
Result  :=false;

//      
Vector1 := v_sub(Sphere,Lb);
//  
Dir     := v_sub(Le,Lb);
//  
D       := v_length(dir);
//  
if d >0 then
  begin
    Vector2  := v_mult(Dir, 1/d);
  end;

//       
d := v_dist(Lb,le);

//          
t := (Vector1.X * Vector2.X) + (Vector1.Y * Vector2.Y) + (Vector1.Z * Vector2.Z);

//    
if t+r <= 0 then  Exit;
if t-r >= d then  Exit;

//     
Point := v_adD(lb,v_mult(Vector2,t));
//       ..
//        
  if v_dist(sphere,point) <=R then
Result := true;
end;

function uSphereVsLine2(const Sphere, P1,P2 : TVector;R: Single): boolean;
var
  v, dir: TVector;
  d, len: Single;
begin
  dir := v_sub(p2, p1);
  len := v_normalizesafe(dir, dir);

  v := v_sub(Sphere, p1);
  d := v_dot(v, dir);

  Result := False;

  if d > len + R then Exit;
  if d < -R then Exit;

  len := v_dist(v_combine(p1, dir, d), sphere);

  if len > R then Exit;

  Result := True;
end;


///////////////////////////////////////////////////////////////////////////////
//Shaders//////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
procedure CalculateTandB(var T, B: TVector; const st1, st2: TVector2d; const P, Q: TVector);
var
  s: Single;
  st: array [0..1, 0..1] of Single;
  pq: array [0..2, 0..1] of Single;
begin

  // Calculate tangent and binormal for a triangle with the given edges P and Q.
  s := 1 / ((st1.x * st2.y) - (st2.x * st1.y));

  st[0,0] := st2.y;   st[1,0] := -st1.y;
  st[0,1] := -st2.x;  st[1,1] := st1.x;

  pq[0,0] := P.X;   pq[1,0] := P.Y;   pq[2,0] := P.Z;
  pq[0,1] := Q.X;   pq[1,1] := Q.Y;   pq[2,1] := Q.Z;

  T.X := s * (st[0,0]*pq[0,0] + st[1,0]*pq[0,1]);
  T.Y := s * (st[0,0]*pq[1,0] + st[1,0]*pq[1,1]);
  T.Z := s * (st[0,0]*pq[2,0] + st[1,0]*pq[2,1]);

  B.X := s * (st[0,1]*pq[0,0] + st[1,1]*pq[0,1]);
  B.Y := s * (st[0,1]*pq[1,0] + st[1,1]*pq[1,1]);
  B.Z := s * (st[0,1]*pq[2,0] + st[1,1]*pq[2,1]);

end;

procedure CalculateTangents(var v1, v2, v3: TShaderVertex);
var
  P, Q: TVector;
  st1, st2: TVector2d;
  T, B: TVector;
begin

  // Calculate triangle's edges:
  P.X := v2.Pos.X - v1.Pos.X;
  P.Y := v2.Pos.Y - v1.Pos.Y;
  P.Z := v2.Pos.Z - v1.Pos.Z;

  Q.X := v3.Pos.X - v1.Pos.X;
  Q.Y := v3.Pos.Y - v1.Pos.Y;
  Q.Z := v3.Pos.Z - v1.Pos.Z;

  // Calculate S and T vectors:
  st1.x := v2.Tex0.x - v1.Tex0.x;
  st1.y := v2.Tex0.y - v1.Tex0.y;

  st2.x := v3.Tex0.x - v1.Tex0.x;
  st2.y := v3.Tex0.y - v1.Tex0.y;

  // Calculate the tangent and binormal:
  CalculateTandB(T, B, st1, st2, P, Q);

  { Add them to the tangents and binormals already defined at each vertex. At
    the end of the processing, we'll calculate the average tangent and binormal
    at each vector from this. }
  v1.Tangent.X := v1.Tangent.X + T.X;     v1.Binormal.X := v1.Binormal.X + B.X;
  v1.Tangent.Y := v1.Tangent.Y + T.Y;     v1.Binormal.Y := v1.Binormal.Y + B.Y;
  v1.Tangent.Z := v1.Tangent.Z + T.Z;     v1.Binormal.Z := v1.Binormal.Z + B.Z;

  v2.Tangent.X := v2.Tangent.X + T.X;     v2.Binormal.X := v2.Binormal.X + B.X;
  v2.Tangent.Y := v2.Tangent.Y + T.Y;     v2.Binormal.Y := v2.Binormal.Y + B.Y;
  v2.Tangent.Z := v2.Tangent.Z + T.Z;     v2.Binormal.Z := v2.Binormal.Z + B.Z;

  v3.Tangent.X := v3.Tangent.X + T.X;     v3.Binormal.X := v3.Binormal.X + B.X;
  v3.Tangent.Y := v3.Tangent.Y + T.Y;     v3.Binormal.Y := v3.Binormal.Y + B.Y;
  v3.Tangent.Z := v3.Tangent.Z + T.Z;     v3.Binormal.Z := v3.Binormal.Z + B.Z;

  { IMPORTANT: If your model has seams where the texture mapping is mirrored,
    the tangents of the vertices on those seams will cancel each other out. This
    will lead to a null vector which will mess your lighting up big time. For
    truly robust bump mapping, such cases must be detected and the affected
    vertices should be duplicated. There are other special cases, but this is
    the worst one. NVIDIA has a utility called NVMeshMender that can help you
    fix your meshes. }

end;

function dotVecLengthSquared3(v: TVector): Single;
begin
  Result := v.x*v.x + v.y*v.y + v.z*v.z;
end;

procedure dotVecNormalize3(var v: TVector);
var
  L: Single;
const
  DOT_ORIGIN3: TVector = (x: 0; y: 0; z: 0);
begin

  L := dotVecLengthSquared3(v);
  if L = 0 then v := DOT_ORIGIN3
  else begin
    L := sqrt(L);
    v.x := v.x / L;
    v.y := v.y / L;
    v.z := v.z / L;
  end;

end;


end.
