#include "Car.h"

#include "Utils/Ini.h"
#include "Utils/OGLutils.h"
#include "Omega/OMEGA_functions.h"
#include "Utils/font.h"


BulletsManager	bulletsManager;


Car::Car()
{ 
	currentWeapon = MACHINEGUN;
	Health = 100;
	Active = true; 
	killsCount = 0;
	deathCount = 0;
}

void Car::setName(const char* Name)
{
	strcpy(name, Name);
}

void Car::load(const char* filename, const char* car_type)
{
	IniFile      f(filename);
	CarSpeed   = f.GetFloat(car_type,"MaxSpeed",0.0f);
	Power      = f.GetFloat(car_type,"Power",0.0f);
	kAF		   = f.GetFloat(car_type,"Friction",0.0f);
	sens       = f.GetFloat(car_type,"sensitivity", 0.0f);
	carSize.x  = f.GetFloat(car_type,"size_x", 0.0f);
	carSize.y  = f.GetFloat(car_type,"size_y", 0.0f);

	char* tmp;
	tmp = f.GetString(car_type, "TextureNormal", "");
	tex_car	   = _cache.LoadIDFromCache(tmp);
	tmp = f.GetString(car_type, "TextureBroke", "");
	tex_badcar	   = _cache.LoadIDFromCache(tmp);
}

void Car::render()
{
	if ( Active && cam.boxVisible(pos, vec2(carSize.x+carSize.y)) )
	{
		glPushMatrix();
			glTranslatef(pos.x, pos.y, 0.0f);

			glPushMatrix();

				//  
				glRotatef(Alpha * rad2deg, 0.0f, 0.0f, 1.0f);


				setTexture(	tex_car );

				glBegin(GL_QUADS);
				glTexCoord2f(0.0f, 1.0f); glVertex2f( - carSize.x, -carSize.y);
				glTexCoord2f(0.0f, 0.0f); glVertex2f( - carSize.x, carSize.y);
				glTexCoord2f(1.0f, 0.0f); glVertex2f( carSize.x, carSize.y);
				glTexCoord2f(1.0f, 1.0f); glVertex2f( carSize.x, -carSize.y);
				glEnd();


				//   
				glColor4f(1.0f, 1.0f, 1.0f, 1.0f - pow(Health * 0.01f, 3.0f));

				setTexture(	tex_badcar );

				glBegin(GL_QUADS);
				glTexCoord2f(0.0f, 1.0f); glVertex2f( - carSize.x, -carSize.y);
				glTexCoord2f(0.0f, 0.0f); glVertex2f( - carSize.x, carSize.y);
				glTexCoord2f(1.0f, 0.0f); glVertex2f( carSize.x, carSize.y);
				glTexCoord2f(1.0f, 1.0f); glVertex2f( carSize.x, -carSize.y);
				glEnd();

			glPopMatrix();


			glPushMatrix();	//  

				float s = 60.0f * Health*0.01f;

				glColor4f(1.0f - Health * 0.006f, pow(Health * 0.01f, 2.0f), 0.0f, 1.0f - pow(Health * 0.01f, 7.0f));

				glTranslatef(-30, -50, 0.0f); 

				glDisable(GL_TEXTURE_2D);

				glBegin(GL_QUADS);
				glVertex2f( 0.0f, 0.0f);
				glVertex2f( s, 0.0f);
				glVertex2f( s, 6.0f);
				glVertex2f( 0.0f, 6.0f);
				glEnd();

				glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

				glEnable(GL_TEXTURE_2D);

			glPopMatrix();

/*
			//  
			tex_gun.bind();
			vec2 an(Target - pos + offset);
			glRotatef(Angle(an) * rad2deg, 0.0f, 0.0f, 1.0f);

			float gun_size = 32.0f;
			glBegin(GL_QUADS);
			glTexCoord2f(0.0f, 1.0f); glVertex2f( - gun_size+10, - gun_size);
			glTexCoord2f(0.0f, 0.0f); glVertex2f( - gun_size+10, gun_size);
			glTexCoord2f(1.0f, 0.0f); glVertex2f( gun_size+10, gun_size);
			glTexCoord2f(1.0f, 1.0f); glVertex2f( gun_size+10, - gun_size);
			glEnd();
*/

		glPopMatrix();

		glColor4f(1.0f, 1.0f, 1.0f, 0.7f);
		font.drawText(vec2(pos.x-30.0f, pos.y-48.f), 20, "%s", name);
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	}
}

void Car::shoot(vec2& target)
{
	if (currentWeapon < 0 || currentWeapon>WEAPONS_COUNT) currentWeapon = MACHINEGUN;
	Bullet bl;


	if ((U_GetTime() - lastShootTime) > bulletsManager.paramWeapon[currentWeapon].reloadTime) //  
	{
		lastShootTime = (float)U_GetTime();

		if ( shopCount[currentWeapon] <= 0 )		//   
		{
//			playSound(snd_no_patrons, pos);
			return; 
		}

		for (int i=0; i < bulletsManager.paramWeapon[currentWeapon].bullCount; i++)
		{

			//     

			float variation = bulletsManager.paramWeapon[currentWeapon].variation;
			float speed     = bulletsManager.paramWeapon[currentWeapon].speed;
			float speedEps  = bulletsManager.paramWeapon[currentWeapon].speedEps;


			//     
			vec2 v = normalize(target - pos);
				 v.x += (frand(variation) - variation * 0.5f);
				 v.y += (frand(variation) - variation * 0.5f);

				 bl.vel = v * (speed + (Speed>0?Speed:0) + (frand(speedEps*2.0f)-speedEps));



			switch (currentWeapon)
			{
			case MACHINEGUN:
				{

					bl.pos = pos;
					bl.startPos = pos;
					bl.angle = Angle(bl.vel);	
					playSound(snd_machine_gun, pos);

					break;
				}

			case FIREFLOW:
				{
					float ang = Angle(bl.vel);
					bl.pos = pos + vec2(cosf(ang), sinf(ang)) * vec2(frand(5.4f));
					bl.startPos = pos;
					bl.angle = frand(two_pi);
					bl.startColor = vec3(0.0f, 0.0f, 0.9f);
					bl.endColor   = vec3(1.0f, 0.62f, -0.9f);
					bl.vel = v * speed + (frand(speedEps*2.0f)-speedEps);

					break;
				}

			case ROCKETLAUNCHER:
				{
					bl.pos = pos;
					bl.startPos = pos;
					bl.angle = Angle(bl.vel);
					playSound(snd_rocket_start, pos);

					break;
				}

			case SHOTGUN:
				{

					bl.pos = pos;
					bl.startPos = pos;
					bl.angle = Angle(bl.vel);	
					playSound(snd_machine_gun, pos);

					break;
				}

			case MINE:
				{
					bl.pos = pos;
					bl.startPos = pos;
					bl.vel = vec2(0);
					playSound(snd_mine_activate, pos);

					break;
				}
			}


			bl.dead = false;	// 
			bl.timeToDead = (float)bulletsManager.paramWeapon[currentWeapon].bullLifeTime;  //  
			bl.type = currentWeapon;		//    
			bl.owner = this;				// 

			//  
			bl.blt_smoke = new PFSmoke();
			particlesEngine.AddEmitter(bl.blt_smoke);

			//  
			bulletsManager.addBullet(bl);
		}
		shopCount[currentWeapon]--; //   
	}
}

void Car::setWeapon(WeaponID w)
{
	currentWeapon = w;
}


void Car::putMine()
{
	WeaponID tmp = currentWeapon;
	currentWeapon = MINE;
	shoot(vec2(0.0f));
	currentWeapon = tmp;
}


void Car::reset()
{
	//  
	killsCount = 0;
	deathCount = 0;
	Active = true;
	Speed = 0.0f;
	Health = 100;
	shopCount[MACHINEGUN] = bulletsManager.paramWeapon[MACHINEGUN].shopSize;
	shopCount[FIREFLOW] = 0;
	shopCount[ROCKETLAUNCHER] = 0;
	shopCount[SHOTGUN] = 0;
	shopCount[MINE] = 3;
	currentWeapon = MACHINEGUN;
}

bool Car::isTyres()
{											  // 38    8,    
	if ( abs(difference( Alpha, velAngle )) > 0.66f && Speed > 8.0f )
		return true;
	else
		return false;
}

