#ifndef __CAR_PHYSIC__
#define __CAR_PHYSIC__
#pragma once

#include <math.h>
#include "Math/vector.h"


//     [0..2*pi]
inline float correctAngle(float angle)
{
	float unit = floor(angle/two_pi);
	float part = unit*two_pi;
	return  (angle - part);
}

//   
inline float difference(float a1, float a2)
{
	a1 = correctAngle(a1);
	a2 = correctAngle(a2);

	if ( (a1 - a2) > pi ) a2 += two_pi;
	if ( (a2 - a1) > pi ) a2 -= two_pi;

	return a2 - a1;
}


class CarPhysics
{
public:

	bool			   Active;
	float			   Health; // 

	float Speed;			   //
	vec2 pos;
	float Alpha;			   //      Z

	vec2 force;		//      
	
	float velAngle;


	bool bTurnLeft;
	bool bTurnRight;
	bool bForward;
	bool bBackward;


	//  
	vec2  carSize;
	float sens;              // 
	float kAF;				 // 
	float Power;			 // 
	float CarSpeed;			 // 
public:

	void update(float dt)
	{
		if (Health < 0) Active = false;
		if (Active)
		{
			if (bForward)
			{   // 
				Speed += Power * dt;
				Speed = MIN(Speed, CarSpeed);
			}
			else
			if (bBackward)
			{   // 
				Speed -= Power * dt;
				Speed = MAX(Speed, -CarSpeed);
			}
			else
				Speed *= kAF;


			if (bTurnRight)
			{	
				Alpha += sens * dt;
			}
			else
				if (bTurnLeft)
				{
					Alpha -= sens * dt;
				}


			float t = abs(Alpha - velAngle)*dt*0.1f;
			velAngle = velAngle + (Alpha - velAngle)*t;



			vec2 velVec (cosf(velAngle), sinf(velAngle)); //  
			vec2 corpVec(cosf(Alpha), sinf(Alpha));		 //   

			//  
			bTurnLeft = false;
			bTurnRight = false;
			bForward = false;
			bBackward = false;

			//pos += force;
			pos += Speed * velVec * dt;

			//force *= kAF*0.8f;
		}
	}

	void turnLeft()
	{
		bTurnLeft = true;
	}

	void turnRight()
	{
		bTurnRight = true;
	}

	void pressGas()
	{
		bForward = true;
	}

	void pressBreak()
	{
		bBackward = true;
	}
};

#endif