#include "GameMenu.h"
#include "World.h"
#include "Resources.h"
#include "Utils/EditorDraw.h"


struct ScreenResolution
{
	int width;
	int height;
};

ScreenResolution screenResolutions[8] ={{800,600},
										{1024, 768},
										{1152, 864},
										{1280, 600},
										{1280, 768},
										{1280, 1024},
										{1600, 900},
										{1680, 1050}};


GUIEngine mainMenuGui;
GUIEngine optionsMenuGui;
GUIEngine prepareMenuGui;

Bot bots;

int  botsCount = 5;
int  botsCountLabel;

int  curMap = 1;
int  mapLabel;

int playerCar = 1;
int playerCarLabel;


int currentResolution=0;
int labelResolution;


// 
int vsyncCheckBox;
int fullScreenCheckBox;


void onOptionsEvent()
{
	gameState = MS_OPTIONS;
}

void onNewGameEvent()
{
	gameState = MS_PREPARE;
}


void onApplySettings()
{
	vSync = ((GUICheckBox*)optionsMenuGui.elements[vsyncCheckBox])->checked;
	fullScreen = ((GUICheckBox*)optionsMenuGui.elements[fullScreenCheckBox])->checked;

	WINDOW_WIDTH = screenResolutions[currentResolution].width;
	WINDOW_HEIGHT = screenResolutions[currentResolution].height;

	applyConfig();
	saveConfig();
	initGUI();
	gameState = MS_MAINMENU;
}

void onCancleSettings()
{
	gameState = MS_MAINMENU;
}


void onGoEvent()
{	
	//  
	for (int i=0; i<bulletsManager.bullets.getCount(); i++)
	{
		bulletsManager.bullets[i].kill();
	}

	for (int i=0; i<particlesEngine.emitters.getCount(); i++)
	{
		particlesEngine.emitters[i]->active = false;
	}
	
	//   
	World::tyres = new PFTiresTrail();
	particlesEngine.AddEmitter(World::tyres);

	// 
	World::carSmoke = new PFSmoke();
	particlesEngine.AddEmitter(World::carSmoke);


	char s[256];
	sprintf(s, "Maps/%d.txt", curMap);
	world.map.loadMap(s);

	cam.maxSizeX = world.map.width;
	cam.maxSizeY = world.map.height;

	world.bots.setCount(0);

	//  
	bots.reset();
	bots.pos = World::map.respawnPoints[(int)frand(World::map.respawnPoints.getCount())];

	//       
	if ( playerCar == 1 )
		bots.load("Config/Cars.ini", "Car1");
	else
		if ( playerCar == 2 )
			bots.load("Config/Cars.ini", "Car2");
		else
			bots.load("Config/Cars.ini", "Car3");

	world.bots.add(bots);
	strcpy(world.bots[0].name, "Player");



	//  
	for (int i=1; i<=botsCount; i++)
	{
		bots.reset();
		bots.pos = World::map.respawnPoints[(int)frand(World::map.respawnPoints.getCount())];

		float f = frand(4);
		if ( f < 1.0f)
			bots.load("Config/Cars.ini", "Car1");
		else
			if ( f >=1.0f && f<=2.0f )
				bots.load("Config/Cars.ini", "Car2");
			else
				bots.load("Config/Cars.ini", "Car3");

		bots.shootTarget = &world.bots[i-1];

		world.bots.add(bots);
		strcpy(world.bots[i].name, botNames[(int)frand(botsNamesCount)]);
	}


	gameState = MS_GAMEPLAY;
}

void onMapIndUp()
{
	curMap++;
	if (curMap>3) curMap = 1;
	prepareMenuGui.elements[mapLabel]->setText("%d", curMap);	
}

void onMapIndDown()
{
	curMap--;
	if (curMap<1) curMap = 3;
	prepareMenuGui.elements[mapLabel]->setText("%d", curMap);	
}

void onBotIndUp()
{
	botsCount++;
	if (botsCount>26) botsCount = 26;
	prepareMenuGui.elements[botsCountLabel]->setText("%d", botsCount);	
}

void onBotIndDown()
{
	botsCount--;
	if (botsCount<0) botsCount = 0;
	prepareMenuGui.elements[botsCountLabel]->setText("%d", botsCount);
}

void onPlayerCarIndUp()
{
	playerCar++;
	if (playerCar>3) playerCar = 3;
	prepareMenuGui.elements[playerCarLabel]->setText("%d", playerCar);	
}

void onPlayerCarIndDown()
{
	playerCar--;
	if (playerCar<1) playerCar = 1;
	prepareMenuGui.elements[playerCarLabel]->setText("%d", playerCar);
}

void setResolutionLabel()
{
	optionsMenuGui.elements[labelResolution]->setText("%d X %d", screenResolutions[currentResolution].width, screenResolutions[currentResolution].height);
}

void setCurrentResolutionToLabel()
{
	for (int i=0; i<8; i++)
		if ( WINDOW_WIDTH == screenResolutions[i].width && WINDOW_HEIGHT == screenResolutions[i].height )
		{
			optionsMenuGui.elements[labelResolution]->setText("%d X %d", screenResolutions[i].width, screenResolutions[i].height);
			currentResolution = i;
			break;
		}
}

void onLeftResolution()
{
	currentResolution--;
	if (currentResolution<0) currentResolution=7;
	setResolutionLabel();
}

void onRightResolution()
{
	currentResolution++;
	if (currentResolution>7) currentResolution=0;
	setResolutionLabel();
}

void createMainMenuButton(vec2& pos, vec2& size, const char* text, void (*onEvent)())
{
	GUIButton* btn = new GUIButton();
	btn->pos = pos;
	btn->size = size;
	btn->tex_normal = tex_panel_normal.id();
	btn->tex_press =  tex_panel_press.id();
	btn->tex_rollover =  tex_panel_rollover.id();
	btn->OnClickEvent = onEvent;
	btn->Initialize(text);
	mainMenuGui.AddElement(btn);
}

void createPrepareButton(vec2& pos, vec2& size, const char* text, void (*onEvent)())
{
	GUIButton* btn = new GUIButton();
	btn->pos = pos;
	btn->size = size;
	btn->tex_normal = tex_panel_normal.id();
	btn->tex_press =  tex_panel_press.id();
	btn->tex_rollover =  tex_panel_rollover.id();
	btn->OnClickEvent = onEvent;
	btn->Initialize(text);
	prepareMenuGui.AddElement(btn);
}

void createOptionsButton(vec2& pos, vec2& size, const char* text, void (*onEvent)())
{
	GUIButton* btn = new GUIButton();
	btn->pos = pos;
	btn->size = size;
	btn->tex_normal = tex_panel_normal.id();
	btn->tex_press =  tex_panel_press.id();
	btn->tex_rollover =  tex_panel_rollover.id();
	btn->OnClickEvent = onEvent;
	btn->Initialize(text);
	optionsMenuGui.AddElement(btn);
}

int createOptionsCheckBox(vec2& pos, vec2& size, const char* text)
{
	GUICheckBox* btn = new GUICheckBox();
	btn->pos = pos;
	btn->size = size;
	btn->tex_normal = tex_panel_normal.id();
	btn->tex_press =  tex_panel_press.id();
	btn->Initialize(text);
	return optionsMenuGui.AddElement(btn);
}

int createMainMenuLabel(vec2& pos, float size, const char* format, ...)
{
	GUILabel* lbl = new GUILabel();
	lbl->textPos = pos;
	lbl->size.x = size;

	va_list ap;
	va_start(ap, format);
	vsprintf(lbl->text, format, ap);
	va_end(ap);
	
	return mainMenuGui.AddElement(lbl);
}

int createPrepareLabel(vec2& pos, float size, const char* format, ...)
{
	GUILabel* lbl = new GUILabel();
	lbl->textPos = pos;
	lbl->size.x = size;

	va_list ap;
	va_start(ap, format);
	vsprintf(lbl->text, format, ap);
	va_end(ap);
	
	return prepareMenuGui.AddElement(lbl);
}

int createOptionsLabel(vec2& pos, float size, const char* format, ...)
{
	GUILabel* lbl = new GUILabel();
	lbl->textPos = pos;
	lbl->size.x = size;

	va_list ap;
	va_start(ap, format);
	vsprintf(lbl->text, format, ap);
	va_end(ap);
	
	return optionsMenuGui.AddElement(lbl);
}


void initGUI()
{
	for (int i=0; i<mainMenuGui.elements.getCount(); i++)
		delete mainMenuGui.elements[i];
	mainMenuGui.elements.setCount(0);

	for (int i=0; i<optionsMenuGui.elements.getCount(); i++)
		delete optionsMenuGui.elements[i];
	optionsMenuGui.elements.setCount(0);

	for (int i=0; i<prepareMenuGui.elements.getCount(); i++)
		delete prepareMenuGui.elements[i];
	prepareMenuGui.elements.setCount(0);


	createMainMenuButton(vec2(WINDOW_WIDTH - 200, WINDOW_HEIGHT*0.5f - 25), vec2(150, 25), "new game", onNewGameEvent);
	createMainMenuButton(vec2(WINDOW_WIDTH - 200, WINDOW_HEIGHT*0.5f + 5), vec2(150, 25), "options", onOptionsEvent);
	createMainMenuButton(vec2(WINDOW_WIDTH - 200, WINDOW_HEIGHT*0.5f + 100), vec2(150, 25), "exit", OE_Quit);
	createMainMenuLabel(vec2(0,100), 50, "Machine Fighting");



	createPrepareButton(vec2(WINDOW_WIDTH - 200, WINDOW_HEIGHT - 50), vec2(150, 25), "go", onGoEvent);

	//  
	createPrepareButton(vec2(WINDOW_WIDTH*0.5f, WINDOW_HEIGHT*0.5f), vec2(25, 25), "^", onMapIndUp);
	createPrepareButton(vec2(WINDOW_WIDTH*0.5f, WINDOW_HEIGHT*0.5f+25.0f), vec2(25, 25), "v", onMapIndDown);
	mapLabel = createPrepareLabel(vec2(WINDOW_WIDTH*0.5f-100,WINDOW_HEIGHT*0.5f+12.5f), 25, "1");
	createPrepareLabel(vec2(WINDOW_WIDTH*0.5f-250,WINDOW_HEIGHT*0.5f+12.5f), 25, "map");

	//   
	createPrepareButton(vec2(WINDOW_WIDTH*0.5f, WINDOW_HEIGHT*0.5f+50), vec2(25, 25), "^", onBotIndUp);
	createPrepareButton(vec2(WINDOW_WIDTH*0.5f, WINDOW_HEIGHT*0.5f+75.0f), vec2(25, 25), "v", onBotIndDown);
	botsCountLabel = createPrepareLabel(vec2(WINDOW_WIDTH*0.5f-100,WINDOW_HEIGHT*0.5f+62.5f), 25, "5");
	createPrepareLabel(vec2(WINDOW_WIDTH*0.5f-250,WINDOW_HEIGHT*0.5f+62.5f), 25, "bots count");

	//  
	createPrepareButton(vec2(WINDOW_WIDTH*0.5f, WINDOW_HEIGHT*0.5f-50), vec2(25, 25), "^", onPlayerCarIndUp);
	createPrepareButton(vec2(WINDOW_WIDTH*0.5f, WINDOW_HEIGHT*0.5f-25), vec2(25, 25), "v", onPlayerCarIndDown);
	playerCarLabel = createPrepareLabel(vec2(WINDOW_WIDTH*0.5f-100,WINDOW_HEIGHT*0.5f-35), 25, "1");
	createPrepareLabel(vec2(WINDOW_WIDTH*0.5f-250,WINDOW_HEIGHT*0.5f-35), 25, "player car");



	// 
	createOptionsButton(vec2(WINDOW_WIDTH*0.5f-100, WINDOW_HEIGHT*0.5f+200), vec2(100, 25), "Apply", onApplySettings);
	createOptionsButton(vec2(WINDOW_WIDTH*0.5f, WINDOW_HEIGHT*0.5f+200), vec2(100, 25), "Cancel", onCancleSettings);

	vsyncCheckBox = createOptionsCheckBox(vec2(WINDOW_WIDTH*0.5f-100, WINDOW_HEIGHT*0.5f-125), vec2(25, 25), "vSync");
	((GUICheckBox*)optionsMenuGui.elements[vsyncCheckBox])->checked = vSync;

	fullScreenCheckBox = createOptionsCheckBox(vec2(WINDOW_WIDTH*0.5f-100, WINDOW_HEIGHT*0.5f-100), vec2(25, 25), "fullScreen");
	((GUICheckBox*)optionsMenuGui.elements[fullScreenCheckBox])->checked = fullScreen;

	createOptionsButton(vec2(WINDOW_WIDTH*0.5f-100, WINDOW_HEIGHT*0.5f-50), vec2(25, 25), "<", onLeftResolution);
	createOptionsButton(vec2(WINDOW_WIDTH*0.5f+100, WINDOW_HEIGHT*0.5f-50), vec2(25, 25), ">", onRightResolution);
	labelResolution = createOptionsLabel(vec2(WINDOW_WIDTH*0.5f-75,WINDOW_HEIGHT*0.5f-50), 25, "unknown");
	setCurrentResolutionToLabel();
}

void updateMainMenu()
{
	mainMenuGui.update();
}

void renderMainMenu()
{
	setTexture(tex_back_menu->ID);
	drawTexturedQuad(vec2(0.0f), vec2(WINDOW_WIDTH, WINDOW_HEIGHT), vec4(1.0f));
	mainMenuGui.Render();
}

void updateOptionsMenu()
{
	optionsMenuGui.update();
}

void renderOptionsMenu()
{
	setTexture(tex_back_other->ID);
	drawTexturedQuad(vec2(0.0f), vec2(WINDOW_WIDTH, WINDOW_HEIGHT), vec4(1.0f));
	optionsMenuGui.Render();
}


void updatePrepareMenu()
{
	prepareMenuGui.update();
}

void renderPrepareMenu()
{
	setTexture(tex_back_other->ID);
	drawTexturedQuad(vec2(0.0f), vec2(WINDOW_WIDTH, WINDOW_HEIGHT), vec4(1.0f));

	prepareMenuGui.Render();
	if ( playerCar == 1 )
		tex_car1_normal.bind();
	else
		if ( playerCar == 2 )
			tex_car2_normal.bind();
		else
			tex_car3_normal.bind();

	drawTexturedQuad(vec2(WINDOW_WIDTH*0.5f-250, WINDOW_HEIGHT*0.5f-120), vec2(128, 64), vec4(1.0f));
}

