#include "BMP.h"
#include "../Utils/Log.h"


#pragma warning(disable: 4996) // fopen
#pragma warning(disable: 4018) // unsigned


void* LoadDIBitmap(const char *filename, BITMAPINFO **info)	 	 
{
	FILE			*fp;		// file pointer 
	void			*bits;		// Bitmap pixel bits 
	long			bitsize,	// Size of bitmap
				    infosize;	// Size of header information 
	BITMAPFILEHEADER	header;	// File header

	//   
	if ((fp = fopen(filename, "rb")) == NULL)
	{
		Log.Write("File not found !\n",filename);
		return (NULL);
	}
	//  
	if (fread(&header, sizeof(BITMAPFILEHEADER), 1, fp) < 1)
	{
		Log.Write("Can nt found file header!\n");
		fclose(fp);
		return (NULL);
	}

	if (header.bfType != 'MB')
	{
	    Log.Write("It is not BMP file!\n");
		fclose(fp);
		return (NULL);
	}

	infosize = header.bfOffBits - sizeof(BITMAPFILEHEADER);
	if ((*info = (BITMAPINFO *)malloc(infosize)) == NULL)
	{
		Log.Write("  !\n");
		fclose(fp);
		return (NULL);
	}

	if (fread(*info, 1, infosize, fp) < infosize)
	{
	    Log.Write("Can nt read file header!\n");
		free(*info);
		fclose(fp);
		return (NULL);
	}

	if ((bitsize = (*info)->bmiHeader.biSizeImage) == 0)
	bitsize = ((*info)->bmiHeader.biWidth *
		   (*info)->bmiHeader.biBitCount + 7) / 8 *
	  abs((*info)->bmiHeader.biHeight);

	if ((bits = malloc(bitsize)) == NULL)
	{
		Log.Write("  !\n");
		free(*info);
		fclose(fp);
		return (NULL);
	};
	if (fread(bits, 1, bitsize, fp) < bitsize)
	{
		Log.Write("Can nt read file!\n");
		free(*info);
		free(bits);
		fclose(fp);
		return (NULL);
	};
	fclose(fp);
	return (bits);
}

unsigned char *ConvertRGB(BITMAPINFO *info,	// I - Original bitmap information 
					void *bits)	        // I - Original bitmap pixels 
{
  int		i, j,			// Looping vars 
  			bitsize,		// Total size of bitmap 
			width;			// Aligned width of bitmap 
  unsigned char	*newbits;		// New RGB bits 
  unsigned char	*from, *to;		// RGB looping vars 

  // Allocate memory for the RGB bitmap...

  width   = 3 * info->bmiHeader.biWidth;
  bitsize = width * info->bmiHeader.biHeight;
  if ((newbits = (unsigned char *)calloc(bitsize, 1)) == NULL)
    return NULL;

  // Copy the original bitmap to the new array, converting as necessary.
        if (info->bmiHeader.biBitCount == 24)
		{
         
         // Swap red & blue in a 24-bit image...
         
        for (i = 0; i < info->bmiHeader.biHeight; i ++)
			 for (j = 0, from = ((unsigned char *)bits) + i * width, /* 1 */
	                                 to = newbits + i * width; 
				  j < info->bmiHeader.biWidth;				   /* 2 */
			      j++, from += 3, to += 3){					   /* 3 */
              to[0] = from[2];
              to[1] = from[1];
              to[2] = from[0];
			};
		};
  return newbits;
}


CBmp::CBmp()
{
	m_SizeX=0;
	m_SizeY=0;
	m_Bpp=0;
	m_dest=NULL;
}

int CBmp::Release()
{
	if(m_dest != NULL)
	{
		free(m_dest);
		m_dest = NULL;
		return -1;
	}
	return 0;
}

bool CBmp::ReadFile(const char *str)
{
	BITMAPINFO  *info;        // Bitmap information 
	void		*bits;		  // Bitmap pixel bits 

	bits = LoadDIBitmap(str, &info);
	if (bits == NULL) {
	  return false;  
	}
	m_dest = ConvertRGB(info, bits);

	m_SizeX = info->bmiHeader.biWidth;
	m_SizeY = info->bmiHeader.biHeight;

	m_Bpp = info->bmiHeader.biBitCount;
	
	free(info);
	free(bits);

	return true; //  !
}