
#include <math.h>

#include "IPicture.h"
#include "../Utils/Log.h"

#pragma warning(disable: 4996)

unsigned char *CPicture::LoadOther(const char *szPathName, int &width, int &height)
{
	OLECHAR		wszPath[MAX_PATH+1];									//     (WCHAR)
	char		szPath[MAX_PATH+1];										//    
	long		lWidth;													// Width In Logical Units
	long		lHeight;												// Height In Logical Units
	long		lWidthPixels;											// Width In Pixels
	long		lHeightPixels;											// Height In Pixels
	GLint		glMaxTexDim;											//   


	//   
	FILE			*fp;
	if ((fp = fopen(szPathName, "rb")) == NULL)
	{
		Log.Write("   !\n",szPathName);
		return 0;
	}

	//------------

	GetCurrentDirectory(MAX_PATH, szPath);								// Get Our Working Directory
	strcat(szPath, "\\");												// Append "\" After The Working Directory
	strcat(szPath, szPathName);											// Append The PathName

	MultiByteToWideChar(CP_ACP, 0, szPath, -1, wszPath, MAX_PATH);
	HRESULT hr = OleLoadPicturePath(wszPath, 0, 0, 0, IID_IPicture, (void**)&pPicture);

	if(FAILED(hr))														//   
	{
		Log.Write("   !\n");
		return 0;
	}

	hdcTemp = CreateCompatibleDC(GetDC(0));								// Create The Windows Compatible Device Context
	if(!hdcTemp)														// Did Creation Fail?
	{
		pPicture->Release();											// Decrements IPicture Reference Count
		return 0;													    // Return False (Failure)
	}

	glGetIntegerv(GL_MAX_TEXTURE_SIZE, &glMaxTexDim);					// Get Maximum Texture Size Supported
	
	pPicture->get_Width(&lWidth);										// Get IPicture Width (Convert To Pixels)
	lWidthPixels	= MulDiv(lWidth, GetDeviceCaps(hdcTemp, LOGPIXELSX), 2540);
	pPicture->get_Height(&lHeight);										// Get IPicture Height (Convert To Pixels)
	lHeightPixels	= MulDiv(lHeight, GetDeviceCaps(hdcTemp, LOGPIXELSY), 2540);

	// Resize Image To Closest Power Of Two
	if (lWidthPixels <= glMaxTexDim) // Is Image Width Less Than Or Equal To Cards Limit
		lWidthPixels = 1 << (int)floor((log((double)lWidthPixels)/log(2.0f)) + 0.5f); 
	else  // Otherwise  Set Width To "Max Power Of Two" That The Card Can Handle
		lWidthPixels = glMaxTexDim;
 
	if (lHeightPixels <= glMaxTexDim) // Is Image Height Greater Than Cards Limit
		lHeightPixels = 1 << (int)floor((log((double)lHeightPixels)/log(2.0f)) + 0.5f);
	else  // Otherwise  Set Height To "Max Power Of Two" That The Card Can Handle
		lHeightPixels = glMaxTexDim;
	
	//	Create A Temporary Bitmap
	BITMAPINFO	bi = {0};												// The Type Of Bitmap We Request
	DWORD		*pBits = 0;												// Pointer To The Bitmap Bits

	bi.bmiHeader.biSize			= sizeof(BITMAPINFOHEADER);				// Set Structure Size
	bi.bmiHeader.biBitCount		= 32;									// 32 Bit
	bi.bmiHeader.biWidth		= lWidthPixels;							// Power Of Two Width
	bi.bmiHeader.biHeight		= lHeightPixels;						// Make Image Top Up (Positive Y-Axis)
	bi.bmiHeader.biCompression	= BI_RGB;								// RGB Encoding
	bi.bmiHeader.biPlanes		= 1;									// 1 Bitplane

	//	Creating A Bitmap This Way Allows Us To Specify Color Depth And Gives Us Imediate Access To The Bits
	hbmpTemp = CreateDIBSection(hdcTemp, &bi, DIB_RGB_COLORS, (void**)&pBits, 0, 0);
	
	if(!hbmpTemp)														// Did Creation Fail?
	{
		DeleteDC(hdcTemp);												// Delete The Device Context
		pPicture->Release();											// Decrements IPicture Reference Count
		return 0;													// Return False (Failure)
	} 

	SelectObject(hdcTemp, hbmpTemp);									// Select Handle To Our Temp DC And Our Temp Bitmap Object

	// Render The IPicture On To The Bitmap
	pPicture->Render(hdcTemp, 0, 0, lWidthPixels, lHeightPixels, 0, lHeight, lWidth, -lHeight, 0);

	// Convert From BGR To RGB Format
	for(long i = 0; i < lWidthPixels * lHeightPixels; i++)				// Loop Through All Of The Pixels
	{
		BYTE* pPixel	= (BYTE*)(&pBits[i]);							// Grab The Current Pixel
		BYTE  temp		= pPixel[0];									// Store 1st Color In Temp Variable (Blue)
		pPixel[0]		= pPixel[2];									// Move Red Value To Correct Position (1st)
		pPixel[2]		= temp;											// Move Temp Value To Correct Blue Position (3rd)

/*		
		if ((pPixel[0]==0) && (pPixel[1]==0) && (pPixel[2]==0))		
			pPixel[3]	=   0;										
		else															
			pPixel[3]	= 255;
*/
	}

	width = lWidthPixels;
	height= lHeightPixels;

	return (unsigned char*)pBits;
}



CPicture::CPicture()
{
	m_SizeX=0;
	m_SizeY=0;
	m_dest=NULL;
}

bool CPicture::ReadFile(const char *str)
{
	m_dest = LoadOther(str, m_SizeX, m_SizeY);
	if (m_dest != 0) return true;
	else		return false;
}

int CPicture::Release()
{
	if(m_dest != NULL)
	{
		DeleteObject(hbmpTemp);		    //  Object
		DeleteDC(hdcTemp);				//  Device Context

		pPicture->Release();			//    IPicture

		m_dest = NULL;
		return -1;
	}
	return 0;
}
