#include "Map.h"
#include "Utils/EditorDraw.h"


// ----------------------------------------------------------------
// Bonus
//

const float BONUS_TIME = 500.0f;
const float FADE_TIME  = 30.0f;
const float RESP_TIME  = 30.0f;

void Bonus::disable()
{
	active = false;
	respawn = false;
	respawnTime = 0.0f;
	phase = 0.0f;
	fading = true;
}

void Bonus::update(float dt)
{
	if ( active == true) return;
	else
		if ( fading )	//  
		{
			phase += dt;
			if (phase>FADE_TIME) 
			{
				fading = false;
				phase = 0.0f;
			}
		}
		else
			if ( respawn )		//  
			{
				phase += dt;
				if (phase > RESP_TIME)
				{
					active = true;
				}
				return;
			}
			else	//    
			{
				respawnTime += dt;
				if (respawnTime > BONUS_TIME)
				{
					respawn = true;
				}
			}
}

void Bonus::render()
{
	vec4 alpha(1.0f, 1.0f, 1.0f, 1.0f);
	if (fading) alpha = vec4( 1.0f - phase/FADE_TIME );	//  
	else
	if (respawn) alpha = vec4( phase/RESP_TIME );

	if ( fading == true || respawn == true || active == true )
	{
		vec2 size (32.0f * clamp(pow(alpha.x,3.0f), 0.0f, 1.0f));

		glPushMatrix();
		glTranslated(pos.x-cam.p.x, pos.y -cam.p.y, 0.0f);
		glRotatef(cosf(U_GetTime()*0.003f)*50.0f, 0.0f, 0.0f, 1.0f);
		
		switch (type)
		{
		case BNS_MACHINEGUN:
			{
				tex_bullet_bonus.bind();
				drawTexturedQuad(-size+size*0.5f, size, alpha);

				break;
			}

		case BNS_FIREFLOW:
			{
				tex_fire_bonus.bind();
				drawTexturedQuad(-size+size*0.5f, size, alpha);
				break;
			}

		case BNS_ROCKETLAUNCHER:
			{
				tex_rocket_bonus.bind();
				drawTexturedQuad(-size+size*0.5f, size, alpha);
				break;
			}

		case BNS_MINE:
			{
				tex_mine_bonus.bind();
				drawTexturedQuad(-size+size*0.5f, size, alpha);
				break;
			}

		case BNS_MEDKIT:
			{
				tex_medkit_bonus.bind();
				drawTexturedQuad(-size+size*0.5f, size, alpha);
				break;
			}

		case BNS_SHOTGUN:
			{
				tex_shotgun_bonus.bind();
				drawTexturedQuad(-size+size*0.5f, size, alpha);
				break;
			}

		}// swticth

		//  
		size = vec2(40.0f);

		glRotatef(U_GetTime()*0.2f, 0.0f, 0.0f, 1.0f);
		tex_sphere.bind();
		drawTexturedQuad(-size+size*0.5f, size,  alpha);
		glPopMatrix();
	} // if
}


// ----------------------------------------------------------------
// Map
//

bool Map::loadMap(const char* filename) //  
{
	respawnPoints.clear();
	for (int i=0; i<objects.getCount(); i++)
	{
		objects[i].polygons.reset();
		objects[i].rim.reset();
	}
	objects.reset();
	bonuses.reset();


	FILE* f;

	f = fopen(filename,"r");
	if (f == NULL)
	{
		return false;
	}

	// 
	int t = 0;
	fscanf(f, "%d\n", &t);
	width = t;
	fscanf(f, "%d\n", &t);
	height = t;

	
	unsigned int num = 0;

	//  
	fscanf(f, "%d\n", &num);
	for (unsigned int i=0; i < num; i++)
	{
		vec2 pt;
		fscanf(f, "%f %f\n", &(pt.x), &(pt.y));
		respawnPoints.add(pt);
	}


	// 
	BonusID TYPE;
	fscanf(f, "%d\n", &num);
	for (unsigned int i=0; i < num; i++)
	{
		vec2 pt;
		fscanf(f, "%f %f %d\n", &(pt.x), &(pt.y), &TYPE);
		bonuses.add(Bonus(pt, TYPE));
	}


	// 

	MapObject mp;
	fscanf(f, "%d\n", &num);
	for (unsigned int i=0; i< num; i++)
	{
		objects.add(mp);


		char t[256];

		//   
		fscanf(f, "%s/n", t);
		objects[i].tex = _cache.LoadIDFromCache(t); 

		unsigned int d = 0;

		//    
		fscanf(f, "%d\n", &d);
		objects[i].collided = (d>0?true:false);


		//  
		fscanf(f, "%d\n", &d);
		for (unsigned int j=0; j < d; j++)
		{
			GraphicTriangle gt;
			objects[i].polygons.add(gt);
			float f1 = 0.0f, f2 = 0.0f;

			// 
			fscanf(f, "%f %f\n", &f1, &f2);
			objects[i].polygons[j].v1.x=f1;
			objects[i].polygons[j].v1.y=f2;

			fscanf(f, "%f %f\n", &f1, &f2);
			objects[i].polygons[j].v2.x=f1;
			objects[i].polygons[j].v2.y=f2;

			fscanf(f, "%f %f\n", &f1, &f2);
			objects[i].polygons[j].v3.x=f1;
			objects[i].polygons[j].v3.y=f2;

			//  
			fscanf(f, "%f %f\n", &f1, &f2);
			objects[i].polygons[j].t1.x=f1;
			objects[i].polygons[j].t1.y=f2;

			fscanf(f, "%f %f\n", &f1, &f2);
			objects[i].polygons[j].t2.x=f1;
			objects[i].polygons[j].t2.y=f2;

			fscanf(f, "%f %f\n", &f1, &f2);
			objects[i].polygons[j].t3.x=f1;
			objects[i].polygons[j].t3.y=f2;


			float f3 = 0.0f, f4 = 0.0f;

			//  
//			fscanf(f, "%f %f %f %f\n", &f1, &f2, &f3, &f4);
			fscanf(f, "%f\n", &f4);
//			objects[i].polygons[j].c1.x=f1;
//			objects[i].polygons[j].c1.y=f2;
//			objects[i].polygons[j].c1.z=f3;
			objects[i].polygons[j].c1.w=f4;

//			fscanf(f, "%f %f %f %f\n", &f1, &f2, &f3, &f4);
			fscanf(f, "%f\n", &f4);
//			objects[i].polygons[j].c2.x=f1;
//			objects[i].polygons[j].c2.y=f2;
//			objects[i].polygons[j].c2.z=f3;
			objects[i].polygons[j].c2.w=f4;

//			fscanf(f, "%f %f %f %f\n", &f1, &f2, &f3, &f4);
			fscanf(f, "%f\n", &f4);
//			objects[i].polygons[j].c3.x=f1;
//			objects[i].polygons[j].c3.y=f2;
//			objects[i].polygons[j].c3.z=f3;
			objects[i].polygons[j].c3.w=f4;

		}
		calculateAABB(&(objects[i]), &(objects[i].aabb));
	}

	fclose(f);

	return true;
}

bool Map::saveMap(const char* filename)
{
	char buff[255];
	FILE* f;

	sprintf(buff, "%s%s", filename, ".txt");

	f = fopen(buff,"w");
	if (f == NULL) return false;

	// 
	fprintf(f, "%d\n", width);
	fprintf(f, "%d\n", height);

	//  
	fprintf(f, "%d\n", respawnPoints.getCount());
	for (unsigned int i=0; i < respawnPoints.getCount(); i++)
	{
		fprintf(f, "%f %f\n", respawnPoints[i].x, respawnPoints[i].y);
	}

	// 
	fprintf(f, "%d\n", bonuses.getCount());
	for (unsigned int i=0; i < bonuses.getCount(); i++)
	{
		fprintf(f, "%f %f %d\n", bonuses[i].pos.x, bonuses[i].pos.y, bonuses[i].type);
	}

	// 
	fprintf(f, "%d\n", objects.getCount()-1);
	for (unsigned int i=0; i< objects.getCount()-1; i++)
	{
		//   
		fprintf(f, "%s\n", _cache.LoadNameFromCache(objects[i].tex));
		
		//    
		fprintf(f, "%d\n", objects[i].collided);

		//  
		fprintf(f, "%d\n", objects[i].polygons.getCount());
		for (unsigned int j=0; j < objects[i].polygons.getCount(); j++)
		{
			// 
			fprintf(f, "%f %f\n", objects[i].polygons[j].v1.x, objects[i].polygons[j].v1.y);
			fprintf(f, "%f %f\n", objects[i].polygons[j].v2.x, objects[i].polygons[j].v2.y);
			fprintf(f, "%f %f\n", objects[i].polygons[j].v3.x, objects[i].polygons[j].v3.y);

			//  
			fprintf(f, "%f %f\n", objects[i].polygons[j].t1.x, objects[i].polygons[j].t1.y);
			fprintf(f, "%f %f\n", objects[i].polygons[j].t2.x, objects[i].polygons[j].t2.y);
			fprintf(f, "%f %f\n", objects[i].polygons[j].t3.x, objects[i].polygons[j].t3.y);

			//  
/*			
			fprintf(f, "%f %f %f %f\n", objects[i].polygons[j].c1.x, objects[i].polygons[j].c1.y, objects[i].polygons[j].c1.z, objects[i].polygons[j].c1.w);
			fprintf(f, "%f %f %f %f\n", objects[i].polygons[j].c2.x, objects[i].polygons[j].c2.y, objects[i].polygons[j].c2.z, objects[i].polygons[j].c2.w);
			fprintf(f, "%f %f %f %f\n", objects[i].polygons[j].c3.x, objects[i].polygons[j].c3.y, objects[i].polygons[j].c3.z, objects[i].polygons[j].c3.w);
*/
			//  
			fprintf(f, "%f\n", objects[i].polygons[j].c1.w);
			fprintf(f, "%f\n", objects[i].polygons[j].c2.w);
			fprintf(f, "%f\n", objects[i].polygons[j].c3.w);
		}

	}

	fclose(f);

	return true;
}


void Map::render()
{   

	//  
	float dx=cam.p.x/128.0f;
	float dy=cam.p.y/128.0f;

	float coeff_x = cam.width/128.0f;
	float coeff_y = cam.height/128.0f;

	tex_ter_dirt.bind();
	glBegin(GL_QUADS);
	glTexCoord2f(0.0f + dx, 0.0f + dy); glVertex2f(0, 0);
	glTexCoord2f(coeff_x + dx, 0.0f + dy);	glVertex2f(cam.width, 0);
	glTexCoord2f(coeff_x + dx, coeff_y + dy); glVertex2f(cam.width, cam.height);
	glTexCoord2f(0.0f + dx, coeff_y + dy);	glVertex2f(0, cam.height);
	glEnd();


	vec3 alpha(1.0f, 1.0f, 1.0f);
	for (unsigned int i=0; i < objects.getCount(); i++)
	{	
		if (cam.boxVisible(objects[i].aabb.v1, objects[i].aabb.v2)) //   
		{
			for (unsigned int j=0; j<objects[i].polygons.getCount(); j++)
			{
				alpha.x = objects[i].polygons[j].c1.w;
				alpha.y = objects[i].polygons[j].c2.w;
				alpha.z = objects[i].polygons[j].c3.w;

				setTexture(objects[i].tex);
				drawTexturedTriangle(&(objects[i].polygons[j]), false, alpha);
			}
		}
	}
}

void Map::renderBonuses()
{
	for ( unsigned int i=0; i<bonuses.getCount(); i++ )
	{
		bonuses[i].render();	
	}
}



