#ifndef __MAP__
#define __MAP__

#include "Utils/TextureManager.h"
#include "Utils/Array.h"
#include "Math/Collisions.h"
#include "Utils/oglutils.h"
#include "Utils/Camera.h"

#include "Resources.h"




typedef enum BonusID //  
{
	BNS_MACHINEGUN,
	BNS_FIREFLOW,
	BNS_ROCKETLAUNCHER,
	BNS_SHOTGUN,
	BNS_MINE,
	BNS_MEDKIT
};

class Bonus // 
{
public:
	BonusID type;
	vec2 pos;
	bool active;

	float respawnTime;		//    
	float phase;
	bool  respawn;			// ,    
	bool  fading;			// 

public:
	Bonus(vec2& p, BonusID t): pos(p), type(t), active(true){}

	void update(float dt);
	void disable();
	void render();
};


class GraphicTriangle: public Triangle		//  
{
public:
	vec2 t1;	//  
	vec2 t2;
	vec2 t3;

	vec4 c1;	//  
	vec4 c2;
	vec4 c3;
};



class MapObject		 //  
{
public:
	Array<GraphicTriangle> polygons;		//  
	Array<Line>			   rim;				//      
	bool collided;							//    
	int					   tex;
	AABB				   aabb;  	

public:
	MapObject()
	{
		tex = 0;
		collided = false;
	}
};



class Map		// 
{
public:
	int width;
	int height;

	Array<vec2> respawnPoints; //  
	Array<MapObject> objects;  // 
	Array<Bonus> bonuses;	   // 

	int	tex_ground;

public:
	bool loadMap(const char* filename);
	bool saveMap(const char* filename);

	void render();
	void renderBonuses();

};



#endif

