#include "Collisions.h"


bool colLineCircle(vec2& x1, vec2& x2, vec2& center, float radius, vec2* normal, float* depth)
{
	float a, b, c, len;
	vec2 line = x2 - x1;
	c = length(line);
	a = length(x1 - center);
	b = length(x2 - center);

	float bb = b*b;

	len = (bb + c*c - a*a)/(2.0f*c); 
	*depth = sqrtf(bb - len*len);    //  

	bool collision = false;

	if ((*depth) <= radius) 
	{
		*depth = radius - (*depth); //  
		collision = true;
	}
	else
	{
		collision = false;
	}

	line = normalize(line);
	if (Line(x1, x2).orient(center) > 0) //    
	{
		*normal = vec2(-line.y, line.x);
	}
	else
	{
		*normal = vec2(line.y, -line.x);
	}


	if (x2.x>x1.x)
	{
		if ((center.x > x2.x+radius) || (center.x < x1.x-radius))
		{
			collision = false;
		}
	}
	else
	{
		if ((center.x < x2.x-radius) || (center.x > x1.x+radius))
		{
			collision = false;
		}
	}

	if (x2.y>x1.y)
	{
		if ((center.y > x2.y+radius) || (center.y < x1.y-radius))
		{
			collision = false;
		}
	}
	else
	{
		if ((center.y < x2.y-radius) || (center.y > x1.y+radius))
		{
			collision = false;
		}
	}

	return collision;

	return true;
}


bool colLineCircle2(vec2& x1, vec2& x2, vec2& center, float radius, vec2* normal, float* depth)
{	//  
	vec2 c = closestPointOnLine(x1, x2, center);
	vec2 v = center-c;
	float len = length(v);
	if ( len>radius ) return false;
	*depth = radius - len;
	*normal = normalize(v);
	return true;
}


vec2 closestPointOnLine(vec2& vA, vec2& vB, vec2& vPoint)
{
  vec2 vVector1 = vPoint - vA;
  vec2 vVector2 = normalize(vB - vA);
  double d = length(vA - vB);
  double t = dot(vVector2, vVector1);

  if (t <= 0.0f) return vA;
  if (t >= d   ) return vB;
   
  vec2 vVector3 = vVector2 * t;
  vec2 vClosestPoint = vA + vVector3;

  return vClosestPoint;
}


float lineDistance(vec2& a, vec2& b, vec2& c)
{
	vec2 closest = closestPointOnLine(a, b, c);
	return length(c-closest);
}



// ---------------------------------------------------------------------
// Triangle
//


vec2 Triangle::getCenter()
{
	return (v1 + v2 + v3) / 3.0f;
}

bool Triangle::pointInside(vec2& v)
{
	vec2 center = getCenter();
	float val_origin, val_test;

	// 1
	val_origin = sign(Line(v1, v2).orient(center));
	val_test   = sign(Line(v1, v2).orient(v));
	if (val_origin != val_test) return false;

	// 2
	val_origin = sign(Line(v2, v3).orient(center));
	val_test   = sign(Line(v2, v3).orient(v));
	if (val_origin != val_test) return false;

	// 3
	val_origin = sign(Line(v3, v1).orient(center));
	val_test   = sign(Line(v3, v1).orient(v));
	if (val_origin != val_test) return false;
	
	return true;		
}

bool Triangle::pointInside(vec2& v, vec2* normal, float* depth)
{
	if ( !pointInside(v) ) return false;

	float min = 9999999999.0f;
	float d = 0.0f;
	vec2 a(0.0f), b(0.0f);

	d = Line(v1, v2).pointDistance(v);
	if ( d < min )
	{
		min = d;
		a = v1;
		b = v2;
	}

	d = Line(v2, v3).pointDistance(v);
	if ( d < min )
	{
		min = d;
		a = v2;
		b = v3;
	}

	d = Line(v3, v1).pointDistance(v);
	if ( d < min )
	{
		min = d;
		a = v3;
		b = v1;
	}

	vec2 norm(a-b);
	norm = vec2(-norm.y, norm.x);
	norm = normalize(norm);

	*normal= norm;
	*depth = min; // 

	return true;
}

bool Triangle::colCircle(vec2& v, float radius, vec2* normal, float* depth)
{
	bool collided = false;
	float m = 0.0f;
	vec2  n(0.0f);

	if (colLineCircle2(v1, v2, v, radius, &n, depth))
	{
		if ( *depth > m )
		{
			m = *depth;
			*normal = n;
		}
		collided = true;
	}

	if (colLineCircle2(v2, v3, v, radius, &n, depth))
	{
		if ( *depth > m )
		{
			m = *depth;
			*normal = n;
		}
		collided = true;
	}

	if (colLineCircle2(v3, v1, v, radius, &n, depth))
	{
		if ( *depth > m )
		{
			m = *depth;
			*normal = n;
		}
		collided = true;
	}
	*depth = m;

	return collided;
}




// ---------------------------------------------------------------------
// AABB
//

bool AABB::pointInside(vec2& v)
{
	return (v.x > v1.x-v2.x) && (v.x < v1.x + v2.x) && (v.y > v1.y-v2.y) && (v.y < v1.y + v2.y);
}

bool AABB::pointInside(vec2& v, float alpha)
{
	mat2 rot = rotate(alpha);
	vec2 _v = v1 + (v-v1)*rot;

	return (_v.x > v1.x-v2.x) && (_v.x < v1.x + v2.x) && (_v.y > v1.y-v2.y) && (_v.y < v1.y + v2.y);
}

bool AABB::colLine(vec2& a, vec2& b)
{																	// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	if (Line(v1, vec2(v1.x, v2.y)).lineIntersect(a, b)) return true;
	if (Line(v2, vec2(v1.x, v2.y)).lineIntersect(a, b)) return true;
	if (Line(v2, vec2(v2.x, v1.y)).lineIntersect(a, b)) return true;
	if (Line(v1, vec2(v2.x, v1.y)).lineIntersect(a, b)) return true;

	return false;
}

bool AABB::colLine(vec2& a, vec2& b, vec2* normal, float* depth)
{
	return false;
}

bool AABB::colCircle(Circle& c)
{
	if ( (c.center.x + c.radius > v1.x - v2.x) && 
		 (c.center.x - c.radius < v1.x + v2.x) && 
		 (c.center.y + c.radius > v1.y - v2.y) && 
		 (c.center.y - c.radius < v1.y + v2.y) ) 
	return true;
	return false;
}




// ---------------------------------------------------------------------
// Line
//

bool Line::lineIntersect(vec2& c, vec2& d)
{
	// dot(ab_|_,ac)*dot(ab_|_,ad) <= 0
	// and
	// dot(cd_|_,da)*dot(cd_|_,db) <= 0
	return  (((c.x-a.x)*(b.y-a.y)-(c.y-a.y)*(b.x-a.x))*((d.x-a.x)*(b.y-a.y)-(d.y-a.y)*(b.x-a.x))<=0) &&
			(((a.x-c.x)*(d.y-c.y)-(a.y-c.y)*(d.x-c.y))*((b.x-c.x)*(d.y-c.y)-(b.y-c.y)*(d.x-c.x))<=0);
}

float Line::pointDistance(vec2& c)
{
	// dot(a,b)/|a|
	vec2 d(a.x-b.x, a.y-b.y);
	float D = d.x * (c.y - a.y) - d.y * (c.x - a.x);
	return fabs(D / length(d));
}


float Line::orient(vec2& c)
{
	// dot(a_|_,b)
	return (a.x - c.x)*(b.y - c.y) - (a.y - c.y) * (b.x - c.x);
}


// ---------------------------------------------------------------------
// Ray
//



// ---------------------------------------------------------------------
// Circle
//
bool Circle::pointInside(vec2& p)
{
	vec2 v = p - center;
	float dist = dot(v, v);
	if ( dist < radius*radius )
		return true;
	else
		return false;
}


bool Circle::colCircle(vec2& p, float r)
{
	float r2 = r + radius;
	vec2  d = p - center;
	float len = dot(d, d);

	if ( len < r2*r2 )
		return true;
	else
		return false;
}


bool Circle::colCircle(vec2& p, float r, vec2* depth)
{
	float r2 = r + radius;
	vec2  d = p - center;
	float len = dot(d, d);

	if ( len < r2*r2 )
	{
		*depth = normalize(d) * (r2-sqrtf(len));
		return true;
	}
	else
		return false;
}


