// --------------------------------------------------------
//      2D  3D .
// 
// :
// 2D, 3D, 4D    - ;
// 2x2, 3x3, 4x4 - ;
//
//  - ViZgl(   Humus ;)
//

#ifndef _VECTOR_H_
#define _VECTOR_H_

#include <stdlib.h>
#include <math.h>
#include <limits.h>
#include <float.h>

const float pi = 3.1415926535897932384626433832795f;
const float two_pi = 6.283185307179586476925286766559f;
const float pi_div2 = 1.5707963267948966192313216916398f;
const float deg2rad = pi / 180.0f;
const float rad2deg = 180.f / pi;


// 
#define POSITIVE_X 0
#define NEGATIVE_X 1
#define POSITIVE_Y 2
#define NEGATIVE_Y 3
#define POSITIVE_Z 4
#define NEGATIVE_Z 5


#pragma warning(disable:4005)


inline float MIN(float a, float b)
{
	return (a<b)? a : b;
}

inline float MAX(float a, float b)
{
	return (a>b)? a : b;
}


//  
inline float frand(float x = 1.0f)
{
	return (rand() / (float) RAND_MAX) * x;
}


// ------------------------------------------------------------
// 2D 
//

struct vec2 {
	float x, y;

	vec2(){}
	vec2(const float ixy){
		x = y = ixy;
	}
	vec2(const float ix, const float iy){
		x = ix;
		y = iy;
	}
	operator float *() const { return (float *) &x; }
	float& operator [] (int index){ return *(&x + index); }

	inline void operator += (const float s) { x += s; y += s; }
	inline void operator += (const vec2 &v) { x += v.x; y += v.y; }
	inline void operator -= (const float s) { x -= s; y -= s; }
	inline void operator -= (const vec2 &v) { x -= v.x; y -= v.y; }
	inline void operator *= (const float s) { x *= s; y *= s; }
	inline void operator *= (const vec2 &v) { x *= v.x; y *= v.y; }
	inline void operator /= (const float s) { x /= s; y /= s; };
	inline void operator /= (const vec2 &v) { x /= v.x; y /= v.y; };

	vec2& operator = ( const vec2& v ) { x = v.x; y = v.y; return *this; }
	vec2& operator = ( float f ) { x = y = f; return *this; }
	vec2  operator - () const { return vec2( -x, -y); }
};

vec2 operator + (const vec2 &u, const vec2 &v);
vec2 operator + (const vec2 &v, const float s);
vec2 operator + (const float s, const vec2 &v);

vec2 operator - (const vec2 &u, const vec2 &v);
vec2 operator - (const vec2 &v, const float s);
vec2 operator - (const float s, const vec2 &v);

vec2 operator - (vec2 &v);

vec2 operator * (const vec2 &u, const vec2 &v);
vec2 operator * (const float s, const vec2 &v);
vec2 operator * (const vec2 &v, const float s);

vec2 operator / (const vec2 &u, const vec2 &v);
vec2 operator / (const vec2 &v, const float s);
vec2 operator / (const float s, const vec2 &v);

bool operator == (const vec2 &u, const vec2 &v);

float Angle(vec2& v);		//    Ox

// ------------------------------------------------------------
// 3D 
//

struct vec3 {
	float x, y, z;

	vec3(){}
	vec3(const float ixyz){
		x = y = z = ixyz;
	}
	vec3(const float ix, const float iy, const float iz){
		x = ix;
		y = iy;
		z = iz;
	}
	vec3(const vec2 iv, const float iz){
		x = iv.x;
		y = iv.y;
		z = iz;
	}
	vec3(const float ix, const vec2 iv){
		x = ix;
		y = iv.x;
		z = iv.y;
	}
	operator float *() const { return (float *) &x; }
	float &operator[](int index){ return *(((float *) &x) + index); }

	inline vec2 xy() const { return vec2(x, y); }
	inline vec2 yz() const { return vec2(y, z); }
	inline vec2 xz() const { return vec2(x, z); }

	inline void operator += (const float s) { x += s; y += s; z += s; }
	inline void operator += (const vec3 &v) { x += v.x; y += v.y; z += v.z; }
	inline void operator -= (const float s) { x -= s; y -= s; z -= s; }
	inline void operator -= (const vec3 &v) { x -= v.x; y -= v.y; z -= v.z; }
	inline void operator *= (const float s) { x *= s; y *= s; z *= s; }
	inline void operator *= (const vec3 &v) { x *= v.x; y *= v.y; z *= v.z; }
	inline void operator /= (const float s) { x /= s; y /= s; z /= s; }
	inline void operator /= (const vec3 &v) { x /= v.x; y /= v.y; z /= v.z; }

	vec3& operator = ( const vec3& v ) { x = v.x; y = v.y; z = v.z; return *this; }
	vec3& operator = ( float f ) { x = y = z = f; return *this; }
	vec3  operator - () const { return vec3( -x, -y, -z ); }
};

vec3 operator + (const vec3 &u, const vec3 &v);
vec3 operator + (const vec3 &v, const float s);
vec3 operator + (const float s, const vec3 &v);

vec3 operator - (const vec3 &u, const vec3 &v);
vec3 operator - (const vec3 &v, const float s);
vec3 operator - (const float s, const vec3 &v);

vec3 operator - (vec3 &v);

vec3 operator * (const vec3 &u, const vec3 &v);
vec3 operator * (const float s, const vec3 &v);
vec3 operator * (const vec3 &v, const float s);

vec3 operator / (const vec3 &u, const vec3 &v);
vec3 operator / (const vec3 &v, const float s);
vec3 operator / (const float s, const vec3 &v);

bool operator == (const vec3 &u, const vec3 &v);


// ------------------------------------------------------------
// 4D 
//

struct vec4 {
	float x, y, z, w;

	vec4(){}
	vec4(const float ixyzw){
		x = y = z = w = ixyzw;
	}
	vec4(const float ix, const float iy, const float iz, const float iw){
		x = ix;
		y = iy;
		z = iz;
		w = iw;
	}
	vec4(const vec2 iv, const float iz, const float iw){
		x = iv.x;
		y = iv.y;
		z = iz;
		w = iw;
	}
	vec4(const float ix, const vec2 iv, const float iw){
		x = ix;
		y = iv.x;
		z = iv.y;
		w = iw;
	}
	vec4(const float ix, const float iy, const vec2 iv){
		x = ix;
		y = iy;
		z = iv.x;
		w = iv.y;
	}
	vec4(const vec2 iv0, const vec2 iv1){
		x = iv0.x;
		y = iv0.y;
		z = iv1.x;
		w = iv1.y;
	}
	vec4(const vec3 iv, const float iw){
		x = iv.x;
		y = iv.y;
		z = iv.z;
		w = iw;
	}
	vec4(const float ix, const vec3 iv){
		x = ix;
		y = iv.x;
		z = iv.y;
		w = iv.z;
	}
	operator float *() const { return (float *) &x; }
	float &operator[](int index){ return *(((float *) &x) + index); }

	inline vec2 xy() const { return vec2(x, y); }
	inline vec2 xz() const { return vec2(x, z); }
	inline vec2 xw() const { return vec2(x, w); }
	inline vec2 yz() const { return vec2(y, z); }
	inline vec2 yw() const { return vec2(y, w); }
	inline vec2 zw() const { return vec2(z, w); }
	inline vec3 xyz() const { return vec3(x, y, z); }
	inline vec3 yzw() const { return vec3(y, z, w); }

	inline void operator += (const float s) { x += s; y += s; z += s; w += s; }
	inline void operator += (const vec4 &v) { x += v.x; y += v.y; z += v.z; w += v.w; }
	inline void operator -= (const float s) { x -= s; y -= s; z -= s; w -= s; }
	inline void operator -= (const vec4 &v) { x -= v.x; y -= v.y; z -= v.z; w -= v.w; }
	inline void operator *= (const float s) { x *= s; y *= s; z *= s; w *= s; }
	inline void operator *= (const vec4 &v) { x *= v.x; y *= v.y; z *= v.z; w *= v.w; }
	inline void operator /= (const float s) { x /= s; y /= s; z /= s; w /= s; }
	inline void operator /= (const vec4 &v) { x /= v.x; y /= v.y; z /= v.z; w /= v.w; }

	vec4& operator = ( const vec4& v ) { x = v.x; y = v.y; z = v.z; w = v.w; return *this; }
	vec4& operator = ( float f ) { x = y = z = w = f; return *this; }
	vec4  operator - () const { return vec4( -x, -y, -z, -w ); }
};

vec4 operator + (const vec4 &u, const vec4 &v);
vec4 operator + (const vec4 &v, const float s);
vec4 operator + (const float s, const vec4 &v);

vec4 operator - (const vec4 &u, const vec4 &v);
vec4 operator - (const vec4 &v, const float s);
vec4 operator - (const float s, const vec4 &v);

vec4 operator - (vec4 &v);

vec4 operator * (const vec4 &u, const vec4 &v);
vec4 operator * (const float s, const vec4 &v);
vec4 operator * (const vec4 &v, const float s);

vec4 operator / (const vec4 &u, const vec4 &v);
vec4 operator / (const vec4 &v, const float s);
vec4 operator / (const float s, const vec4 &v);

bool operator == (const vec4 &u, const vec4 &v);


// ------------------------------------------------------------
//     
//

//  
float dot(const vec2 &u, const vec2 &v);
float dot(const vec3 &u, const vec3 &v);
float dot(const vec4 &u, const vec4 &v);


//  
float lerp(const float u, const float v, const float x);
vec2  lerp(const vec2 &u, const vec2 &v, const float x);
vec3  lerp(const vec3 &u, const vec3 &v, const float x);
vec4  lerp(const vec4 &u, const vec4 &v, const float x);
vec2  lerp(const vec2 &u, const vec2 &v, const vec2 &x);
vec3  lerp(const vec3 &u, const vec3 &v, const vec3 &x);
vec4  lerp(const vec4 &u, const vec4 &v, const vec4 &x);

//  
float cerp(const float u0, const float u1, const float u2, const float u3, float x);
vec2  cerp(const vec2 &u0, const vec2 &u1, const vec2 &u2, const vec2 &u3, float x);
vec3  cerp(const vec3 &u0, const vec3 &u1, const vec3 &u2, const vec3 &u3, float x);
vec4  cerp(const vec4 &u0, const vec4 &u1, const vec4 &u2, const vec4 &u3, float x);

//  
float sign(const float v);
vec2  sign(const vec2 &v);
vec3  sign(const vec3 &v);
vec4  sign(const vec4 &v);

// 
vec2  abs(const vec2 &v);
vec3  abs(const vec3 &v);
vec4  abs(const vec4 &v);

//    
float clamp(const float v, const float c0, const float c1);
vec2  clamp(const vec2 &v, const float c0, const float c1);
vec2  clamp(const vec2 &v, const vec2 &c0, const vec2 &c1);
vec3  clamp(const vec3 &v, const float c0, const float c1);
vec3  clamp(const vec3 &v, const vec3 &c0, const vec3 &c1);
vec4  clamp(const vec4 &v, const float c0, const float c1);
vec4  clamp(const vec4 &v, const vec4 &c0, const vec4 &c1);

//    [0,1]
#define saturate(x) clamp(x, 0, 1)

//  
vec2 normalize(const vec2 &v);
vec3 normalize(const vec3 &v);
vec4 normalize(const vec4 &v);

// 
float length(const vec2 &v);
float length(const vec3 &v);
float length(const vec4 &v);

// (reflection)
vec3 reflect(const vec3 &v, const vec3 &normal);

//   
float distance(const vec2 &u, const vec2 &v);
float distance(const vec3 &u, const vec3 &v);
float distance(const vec4 &u, const vec4 &v);

//     
float planeDistance(const vec3 &normal, const float offset, const vec3 &point);
float planeDistance(const vec4 &plane, const vec3 &point);

float sCurve(const float t);

//  
vec3 cross(const vec3 &u, const vec3 &v);

//    
float lineProjection(const vec3 &line0, const vec3 &line1, const vec3 &point);

//  
unsigned int toRGBA(const vec4 &u);
unsigned int toBGRA(const vec4 &u);

#endif





#ifndef _MATRIX_
#define _MATRIX_

// ------------------------------------------------------------
//  2x2
//

struct mat2 {
private:
	float e[2][2];

public:
	mat2()
	{}

	mat2(float _e11, float _e12, float _e21, float _e22)
	{
		e[0][0] = _e11;
		e[0][1] = _e12;
		e[1][0] = _e21;
		e[1][1] = _e22;
	}

	mat2(vec2 &row0, vec2 &row1)	
	{
		e[0][0] = row0.x;
		e[0][1] = row0.y;
		e[1][0] = row1.x;
		e[1][1] = row1.y;
	}

	float  operator()(int i, int j) const { return e[i][j]; }
	float& operator()(int i, int j)       { return e[i][j]; }

	const vec2& operator[](int i) const { return reinterpret_cast<const vec2&>(e[i][0]); }
		  vec2& operator[](int i)       { return reinterpret_cast<      vec2&>(e[i][0]); }

	operator const float *() const { return (const float *) e; }
};

mat2 operator + (const mat2 &m, const mat2 &n);
mat2 operator - (const mat2 &m, const mat2 &n); 
mat2 operator - (const mat2 &m);

mat2 operator * (const mat2 &m, const mat2 &n); //  
vec2 operator * (const vec2 &v, const mat2 &m); //   
mat2 operator * (const mat2 &m, const float x);

mat2 transpose(const mat2 &m);   // 
float det(const mat2 &m);		 // 
mat2 operator ! (const mat2 &m); //  


// ------------------------------------------------------------
//  3x3
//

struct mat3 {
private:
	float e[3][3];

public:
	mat3(){}
	mat3(const vec3 &row0, const vec3 &row1, const vec3 &row2){
		e[0][0] = row0.x;
		e[0][1] = row0.y;
		e[0][2] = row0.z;
		e[1][0] = row1.x;
		e[1][1] = row1.y;
		e[1][2] = row1.z;
		e[2][0] = row2.x;
		e[2][1] = row2.y;
		e[2][2] = row2.z;
	}
	mat3(const float m00, const float m01, const float m02,
		 const float m10, const float m11, const float m12,
		 const float m20, const float m21, const float m22){
		e[0][0] = m00;
		e[0][1] = m01;
		e[0][2] = m02;
		e[1][0] = m10;
		e[1][1] = m11;
		e[1][2] = m12;
		e[2][0] = m20;
		e[2][1] = m21;
		e[2][2] = m22;
	}

	float  operator()(int i, int j) const { return e[i][j]; }
	float& operator()(int i, int j)       { return e[i][j]; }

	const vec3& operator[](int i) const { return reinterpret_cast<const vec3&>(e[i][0]); }
		  vec3& operator[](int i)       { return reinterpret_cast<      vec3&>(e[i][0]); }

	operator const float *() const { return (const float *) e; }
};

mat3 operator + (const mat3 &m, const mat3 &n);
mat3 operator - (const mat3 &m, const mat3 &n);
mat3 operator - (const mat3 &m);

mat3 operator * (const mat3 &m, const mat3 &n);
vec3 operator * (const vec3 &v, const mat3 &m);
mat3 operator * (const mat3 &m, const float x);

mat3 transpose(const mat3 &m);
float det(const mat3 &m);
mat3 operator ! (const mat3 &m);


// ------------------------------------------------------------
//  4x4
//

struct mat4 {
private:
	float e[4][4];

public:
	mat4(){}
	mat4(const vec4 &row0, const vec4 &row1, const vec4 &row2, const vec4 &row3){
		e[0][0] = row0.x;
		e[0][1] = row0.y;
		e[0][2] = row0.z;
		e[0][3] = row0.w;
		e[1][0] = row1.x;
		e[1][1] = row1.y;
		e[1][2] = row1.z;
		e[1][3] = row1.w;
		e[2][0] = row2.x;
		e[2][1] = row2.y;
		e[2][2] = row2.z;
		e[2][3] = row2.w;
		e[3][0] = row3.x;
		e[3][1] = row3.y;
		e[3][2] = row3.z;
		e[3][3] = row3.w;
	}
	mat4(const float m00, const float m01, const float m02, const float m03,
		 const float m10, const float m11, const float m12, const float m13,
		 const float m20, const float m21, const float m22, const float m23,
		 const float m30, const float m31, const float m32, const float m33){
		e[0][0] = m00;
		e[0][1] = m01;
		e[0][2] = m02;
		e[0][3] = m03;
		e[1][0] = m10;
		e[1][1] = m11;
		e[1][2] = m12;
		e[1][3] = m13;
		e[2][0] = m20;
		e[2][1] = m21;
		e[2][2] = m22;
		e[2][3] = m23;
		e[3][0] = m30;
		e[3][1] = m31;
		e[3][2] = m32;
		e[3][3] = m33;
	}

	float  operator()(int i, int j) const { return e[i][j]; }
	float& operator()(int i, int j)       { return e[i][j]; }

	const vec4& operator[](int i) const { return reinterpret_cast<const vec4&>(e[i][0]); }
		  vec4& operator[](int i)       { return reinterpret_cast<      vec4&>(e[i][0]); }

	operator const float *() const { return (const float *) e; }

	void translate(const vec3 &v);
};

mat4 operator + (const mat4 &m, const mat4 &n);
mat4 operator - (const mat4 &m, const mat4 &n);
mat4 operator - (const mat4 &m);

mat4 operator * (const mat4 &m, const mat4 &n);
vec4 operator * (const vec4 &v, const mat4 &m);
mat4 operator * (const mat4 &m, const float x);

mat4 transpose(const mat4 &m);
mat4 operator ! (const mat4 &m);


// ------------------------------------------------------------
//     
//

mat2 rotate(const float angle);
mat4 rotateX(const float angle);
mat4 rotateY(const float angle);
mat4 rotateZ(const float angle);
mat4 rotateXY(const float angleX, const float angleY);
mat4 rotateYX(const float angleX, const float angleY);
mat4 rotateZXY(const float angleX, const float angleY, const float angleZ);
mat4 translate(const vec3 &v);
mat4 translate(const float x, const float y, const float z);
mat4 scale(const float x, const float y, const float z);
mat4 perspectiveMatrix(const float fov, const float zNear, const float zFar);
mat4 perspectiveMatrixX(const float fov, const int width, const int height, const float zNear, const float zFar);
mat4 perspectiveMatrixY(const float fov, const int width, const int height, const float zNear, const float zFar);
mat4 orthoMatrixX(const float left, const float right, const float top, const float bottom, const float zNear, const float zFar);
mat4 toD3DProjection(const mat4 &m);
mat4 toGLProjection(const mat4 &m);
mat4 cubeViewMatrix(const unsigned int side);
mat4 cubeProjectionMatrixGL(const float zNear, const float zFar);
mat4 cubeProjectionMatrixD3D(const float zNear, const float zFar);

mat2 identity2();
mat3 identity3();
mat4 identity4();

#endif

typedef mat2 float2x2;
typedef mat3 float3x3;
typedef mat4 float4x4;
typedef vec2 float2;
typedef vec3 float3;
typedef vec4 float4;

