#include "ParticlesEffects.h"



// ------------------------------------------------------------------
//   
//

PFRicochet::PFRicochet(vec2& p, vec2& normal, int n, vec3& col): ParticlesEmitter(p, col)
{
	back = false;			//  
	alpha = false;			// ,  
	Particle part;
	tex = tex_spark.id();

	part.pos = p;
	part.life = 25;
	part.start_life = 20;
	part.angle = 0;
	part.angle_speed = 0.0f;
	float s = frand(7.0f)+20.0f;
	part.size = vec2(s, s);
	part.speed = vec2(0.0f, 0.0f);

	particles.add(part);

	normal *= 3.2f;
	for (unsigned int i=0; i<n; i++)
	{
		part.pos = p;
		part.life = 20;
		part.start_life = 15;
		part.angle = 0;
		part.angle_speed = 0.0f;
		s = frand(8.0f)+10.0f;
		part.size = vec2(s, s);
		part.speed = vec2(normal.x + (frand(6.0f)-3.0f), normal.y + (frand(6.0f)-3.0f));

		particles.add(part);
	}
}

void PFRicochet::UpdateParticle(int i, float dt)
{
	particles[i].pos += particles[i].speed * dt;
}





// ------------------------------------------------------------------
// 
//

PFCrater::PFCrater(vec2& p, float size)
			   : ParticlesEmitter(p, vec3(1.0f, 1.0f, 1.0f))
{
	back = true;
	tex = tex_crater.id();
	alpha = true;

	// 
	Particle part;

	part.pos = p;
	part.life = 600;
	part.start_life = 60;
	part.angle = frand(360.0f);
	part.angle_speed = 0.0f;
	part.size = vec2(size);
	part.speed = vec2(0.0f, 0.0f);

	particles.add(part);
}





// ------------------------------------------------------------------
//  
//

PFExplosionWave::PFExplosionWave(vec2& pos, float size) : ParticlesEmitter(pos, vec3(1.0f, 1.0f, 1.0f))
{
	back = true;

	Particle part;

	part.pos = pos;
	part.life = size * 0.5f;
	part.start_life = 30;
	part.angle = frand(360.0f);
	part.angle_speed = 0.0f;
	part.size = vec2(size);
	part.speed = vec2(0.0f, 0.0f);

	particles.add(part);
}

void PFExplosionWave::Render()
{
	glEnable(GL_TEXTURE_2D);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	tex_shokwave1.bind();
	if (particles[0].life > 0.0f )
	{
		if (cam.boxVisible(particles[0].pos, particles[0].size))
		{
			float s = pow(particles[0].life / particles[0].start_life, 2.0f);
			vec4 col(color, s);
			glColor4fv(col);

			glPushMatrix();
			glTranslatef(particles[0].pos.x, particles[0].pos.y, 0.0f);
			glScalef(particles[0].size.x, particles[0].size.y, 1.0f);
			glRotatef(particles[0].angle, 0.0f, 0.0f, 1.0f);
			RenderQuad(vec2(-0.5f, -0.5f), vec2(1.0f, 1.0f));
			glPopMatrix();
		}
	}
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}

void PFExplosionWave::UpdateParticle(int i, float dt)
{
	particles[i].size += vec2(dt*4.0f);
}




// ------------------------------------------------------------------
//  
//

PFExplosionAnim::PFExplosionAnim(vec2& pos, float size): ParticlesEmitter(pos, vec3(1.0f, 1.0f, 1.0f))
{
	back = false;
	sa.init(pos, vec2(size), tex_explosion_anim.id(), 256.0f, 256.0f, 16, 1.0f, 64, 64, false, true, false);
}

void PFExplosionAnim::Update(float dt)
{
	sa.update(dt);
	if ( false == sa.getActive() ) active = false;
}

void PFExplosionAnim::Render()
{
	sa.render();
}






// ------------------------------------------------------------------
//  
//

PFCarRespawn::PFCarRespawn(vec2& pos): ParticlesEmitter(pos, vec3(1.0f, 1.0f, 1.0f))
{
	back = false;
	alpha = false;

	tex = tex_respawn.id();

	color = vec3(frand(0.15f)+0.3f, frand(0.2f)+0.8f, frand(0.4f)+0.1f);	//  

	Particle part;

	part.pos = pos;
	part.life = 40.0f;
	part.start_life = 20;
	part.angle = frand(360.0f);
	part.angle_speed = 5.0f;
	part.size = vec2(1);
	part.speed = vec2(0.0f, 0.0f);
	particles.add(part);


	part.pos = pos;
	part.life = 40.0f;
	part.start_life = 20;
	part.angle = frand(360.0f);
	part.angle_speed = -5.0f;
	part.size = vec2(1);
	part.speed = vec2(0.0f, 0.0f);

	particles.add(part);
}


void PFCarRespawn::Update(float dt)
{
	particles[0].angle += particles[0].angle_speed*dt;
	particles[1].angle += particles[1].angle_speed*dt;

	if ( particles[0].size.x < 100.0f )
		particles[0].size += dt*15.0f;
	if ( particles[1].size.x < 100.0f )
	particles[1].size += dt*15.0f;


	int active_particles = 0;
	for (unsigned int i=0; i<particles.getCount(); i++)
	{
		if (particles[i].life > 0.0f)
		{
			particles[i].life -= dt;
			++active_particles;
		}
	}

	if (0 == active_particles ) active = false;
}






// ------------------------------------------------------------------
//   
//

PFSmoke::PFSmoke(): ParticlesEmitter(vec2(0.0f), vec3(1.0f, 1.0f, 1.0f))
{
	loop = false;
	back = false;			//  
	alpha = true;
	owner_dead = false;
	tex = tex_smoke.id();
}

void PFSmoke::Update(float dt)
{
	int active_particles = 0;
	for (unsigned int i=0; i<particles.getCount(); i++)
	{
		if (particles[i].life > 0.0f)
		{
			particles[i].life -= dt;
			particles[i].pos += particles[i].speed * dt;
			active_particles++;
		}
		else particles.fastRemove(i);
	}
	if (0 == active_particles && true == owner_dead) active = false;
}

void PFSmoke::AddParticle(vec2& Pos, vec2& Speed, vec2& Size, float Life)
{
	Particle part;

	part.pos = Pos;
	part.speed = Speed;
	part.life = Life;
	part.start_life = 40;
	part.angle = frand(360);
	part.angle_speed = 0.0f;
	part.size = Size;

	particles.add(part);
}





// ------------------------------------------------------------------
//   
//

PFTiresTrail::PFTiresTrail(): ParticlesEmitter(vec2(0.0f), vec3(1.0f, 1.0f, 1.0f))
{
	tex = tex_tyres_trail.id();
	loop = false;
	back = true;
	alpha = true;
	off = false;
	active = true;
}

void PFTiresTrail::AddParticle(vec2& Pos, float Angle)
{
	Particle part;

	part.pos = Pos;
	part.speed = vec2(0.0f);
	part.life = 400;
	part.start_life = 50;
	part.angle = Angle*rad2deg;
	part.angle_speed = 0.0f;
	part.size = vec2(16, 64);

	particles.add(part);
}

void PFTiresTrail::Update(float dt)
{
	int active_particles = 0;
	for (unsigned int i=0; i<particles.getCount(); i++)
	{
		if (particles[i].life > 0.0f)
		{
			particles[i].life -= dt;
			active_particles++;
		}
		else particles.fastRemove(i);
	}
	if (0 == active_particles && true == off) active = false;
}


