#ifndef __SCROLL_CAMERA_2D__
#define __SCROLL_CAMERA_2D__

#include "../Math/vector.h"


class Camera
{
public:
	vec2 p;
	int   maxSizeX;
	int	  maxSizeY;
	int   width;
	int   height;

	float shake_power;
	float shake_time;

public:
	void centerCam(vec2& pos, float dt)
	{
		if (maxSizeX<width) width = maxSizeX;
		if (maxSizeY<height) height = maxSizeY;

		float cw, ch;

		cw      = pos.x - width  / 2.0f; //  
		ch      = pos.y - height / 2.0f;

		p.x = p.x + (cw - p.x)*0.1f * dt;
		p.y = p.y + (ch - p.y)*0.1f * dt;

		if (p.x < 0) p.x = 0.0f;
		if (p.y < 0) p.y = 0.0f;


		if (p.x > (maxSizeX-width))  p.x = (float) maxSizeX-width;
		if (p.y > (maxSizeY-height)) p.y = (float) maxSizeY-height;



		if (shake_time>0)
		{
			p.x += (frand(shake_power)-shake_power*0.5f)*shake_time*0.1f;
			p.y += (frand(shake_power)-shake_power*0.5f)*shake_time*0.1f;

			shake_time -= dt;
		}
	}

	void update(float dt)
	{
		if (maxSizeX<width) width = maxSizeX;
		if (maxSizeY<height) height = maxSizeY;

		if (p.x < 0) p.x = 0.0f;
		if (p.y < 0) p.y = 0.0f;


		if (p.x > (maxSizeX-width))  p.x = (float) maxSizeX-width;
		if (p.y > (maxSizeY-height)) p.y = (float) maxSizeY-height;


		if (shake_time>0)
		{
			p.x += (frand(shake_power)-shake_power*0.5f)*shake_time*0.1f;
			p.y += (frand(shake_power)-shake_power*0.5f)*shake_time*0.1f;

			shake_time -= dt;
		}

	}

	void Shake(float power, float time)
	{
		shake_power = power;
		shake_time  = time;
	}

	bool pointVisible(vec2& point)
	{
		return (point.x>p.x)&&(point.x<(p.x+width)) &&
			   (point.y>p.y)&&(point.y<(p.y+height));
	}

	bool circleVisible(vec2& point, float radius)
	{
		return (point.x+radius>p.x)&&(point.x-radius<p.x+width) &&
			   (point.y+radius>p.y)&&(point.y-radius<p.y+height);
	}

	bool boxVisible(vec2& v1, vec2& v2)
	{
		return (!pointVisible(v1-v2) && !pointVisible(vec2(v1.x+v2.x, v1.y-v2.y)) &&
			!pointVisible(v1+v2) && !pointVisible(vec2(v1.x-v2.x, v1.y+v2.y)))?false:true;
	}

	bool triangleVisible(vec2& a, vec2& b, vec2& c)
	{
		//return (!pointVisible(a) && !pointVisible(b) && !pointVisible(c))?false:true;
		return true;
	}

	bool lineVisible(vec2& a, vec2& b)
	{
		return (!pointVisible(a) && !pointVisible(b))?false:true;
	}
};


extern Camera cam; // 1  

#endif