#include "EditorDraw.h"



void drawPoint(vec2& p, bool selected, float alpha)
{
	if (!cam.pointVisible(p)) return;

	glDisable(GL_TEXTURE_2D);
	glPushMatrix();
	glTranslatef(-cam.p.x, -cam.p.y, 0.0f);

	if (selected)
		glColor4f(1.0f, 0.0f, 0.0f, alpha);
	else
		glColor4f(1.0f, 1.0f, 1.0f, alpha);

	glBegin(GL_POINTS);
	glVertex2f(p.x, p.y);
	glEnd();

	glPopMatrix();

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}


void drawLine(vec2& a, vec2& b, vec2& alpha)
{
	if (!cam.lineVisible(a, b)) return;

	glDisable(GL_TEXTURE_2D);
	glPushMatrix();
	glTranslatef(-cam.p.x, -cam.p.y, 0.0f);

	glBegin(GL_LINES);
	glColor4f(1.0f, 1.0f, 1.0f, alpha.x);
	glVertex2f(a.x, a.y);

	glColor4f(1.0f, 1.0f, 1.0f, alpha.y);
	glVertex2f(b.x, b.y);
	glEnd();

	glPopMatrix();

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}


void drawTriangle(Triangle* tri, bool selected, vec3& alpha)
{
	if (!cam.triangleVisible(tri->v1, tri->v2, tri->v3)) return;

	glDisable(GL_TEXTURE_2D);
	glPushMatrix();
	glTranslatef(-cam.p.x, -cam.p.y, 0.0f);

	glBegin(GL_LINE_LOOP);
	if (selected)
	{
		glColor4f(1.0f, 0.0f, 0.0f, alpha.x);
		glVertex2f(tri->v1.x, tri->v1.y);

		glColor4f(1.0f, 0.0f, 0.0f, alpha.y);
		glVertex2f(tri->v2.x, tri->v2.y);

		glColor4f(1.0f, 0.0f, 0.0f, alpha.z);
		glVertex2f(tri->v3.x, tri->v3.y);
	}
	else
	{
		glColor4f(1.0f, 1.0f, 1.0f, alpha.x);
		glVertex2f(tri->v1.x, tri->v1.y);

		glColor4f(1.0f, 1.0f, 1.0f, alpha.y);
		glVertex2f(tri->v2.x, tri->v2.y);

		glColor4f(1.0f, 1.0f, 1.0f, alpha.z);
		glVertex2f(tri->v3.x, tri->v3.y);
	}
	glEnd();

	glPopMatrix();
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}


void drawTexturedTriangle(GraphicTriangle* tri, bool selected, vec3& alpha)
{
	if (!cam.triangleVisible(tri->v1, tri->v2, tri->v3)) return;

	glEnable(GL_TEXTURE_2D);
	glPushMatrix();
	glTranslatef(-cam.p.x, -cam.p.y, 0.0f);

	glBegin(GL_TRIANGLES);
	if (selected)
	{
		glColor4f(1.0f, 0.0f, 0.0f, alpha.x);
		glTexCoord2f(tri->t1.x, tri->t1.y); glVertex2f(tri->v1.x, tri->v1.y);

		glColor4f(1.0f, 0.0f, 0.0f, alpha.y);
		glTexCoord2f(tri->t2.x, tri->t2.y); glVertex2f(tri->v2.x, tri->v2.y);

		glColor4f(1.0f, 0.0f, 0.0f, alpha.z);
		glTexCoord2f(tri->t3.x, tri->t3.y); glVertex2f(tri->v3.x, tri->v3.y);
	}
	else
	{
		glColor4f(1.0f, 1.0f, 1.0f, alpha.x);
		glTexCoord2f(tri->t1.x, tri->t1.y); glVertex2f(tri->v1.x, tri->v1.y);

		glColor4f(1.0f, 1.0f, 1.0f, alpha.y);
		glTexCoord2f(tri->t2.x, tri->t2.y); glVertex2f(tri->v2.x, tri->v2.y);

		glColor4f(1.0f, 1.0f, 1.0f, alpha.z);
		glTexCoord2f(tri->t3.x, tri->t3.y); glVertex2f(tri->v3.x, tri->v3.y);
	}
	glEnd();

	glPopMatrix();

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}


void drawQuad(vec2& p, vec2& size, bool selected, vec4& alpha)
{
	if (!cam.boxVisible(p, size)) return;

	glDisable(GL_TEXTURE_2D);
	glPushMatrix();
	glTranslatef(-cam.p.x, -cam.p.y, 0.0f);

	glBegin(GL_QUADS);
	if (selected)
	{
		glColor4f(1.0f, 0.0f, 0.0f, alpha.x);
		glVertex2f(p.x - size.x, p.y - size.y);

		glColor4f(1.0f, 0.0f, 0.0f, alpha.y);
		glVertex2f(p.x + size.x, p.y - size.y);

		glColor4f(1.0f, 0.0f, 0.0f, alpha.z);
		glVertex2f(p.x + size.x, p.y + size.y);

		glColor4f(1.0f, 0.0f, 0.0f, alpha.w);
		glVertex2f(p.x - size.x, p.y + size.y);
	}
	else
	{
		glColor4f(1.0f, 1.0f, 1.0f, alpha.x);
		glVertex2f(p.x - size.x, p.y - size.y);

		glColor4f(1.0f, 1.0f, 1.0f, alpha.y);
		glVertex2f(p.x + size.x, p.y - size.y);

		glColor4f(1.0f, 1.0f, 1.0f, alpha.z);
		glVertex2f(p.x + size.x, p.y + size.y);

		glColor4f(1.0f, 1.0f, 1.0f, alpha.w);
		glVertex2f(p.x - size.x, p.y + size.y);
	}
	glEnd();

	glPopMatrix();

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}


void drawTexturedQuad(vec2& p, vec2& size, bool selected, vec4& alpha)
{
	if (!cam.boxVisible(p, size)) return;

	glEnable(GL_TEXTURE_2D);
	glPushMatrix();
	glTranslatef(-cam.p.x, -cam.p.y, 0.0f);

	glBegin(GL_QUADS);
	if (selected)
	{
		glColor4f(1.0f, 0.0f, 0.0f, alpha.x);
		glTexCoord2f(0.0f, 1.0f); glVertex2f(p.x - size.x, p.y - size.y);

		glColor4f(1.0f, 0.0f, 0.0f, alpha.y);
		glTexCoord2f(1.0f, 1.0f); glVertex2f(p.x + size.x, p.y - size.y);

		glColor4f(1.0f, 0.0f, 0.0f, alpha.z);
		glTexCoord2f(1.0f, 0.0f); glVertex2f(p.x + size.x, p.y + size.y);

		glColor4f(1.0f, 0.0f, 0.0f, alpha.w);
		glTexCoord2f(0.0f, 0.0f); glVertex2f(p.x - size.x, p.y + size.y);
	}
	else
	{
		glColor4f(1.0f, 1.0f, 1.0f, alpha.x);
		glTexCoord2f(0.0f, 1.0f); glVertex2f(p.x - size.x, p.y - size.y);

		glColor4f(1.0f, 1.0f, 1.0f, alpha.y);
		glTexCoord2f(1.0f, 1.0f); glVertex2f(p.x + size.x, p.y - size.y);

		glColor4f(1.0f, 1.0f, 1.0f, alpha.z);
		glTexCoord2f(1.0f, 0.0f); glVertex2f(p.x + size.x, p.y + size.y);

		glColor4f(1.0f, 1.0f, 1.0f, alpha.w);
		glTexCoord2f(0.0f, 0.0f); glVertex2f(p.x - size.x, p.y + size.y);
	}
	glEnd();

	glPopMatrix();

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}

void drawTexturedQuad(vec2& p, vec2& size, vec4& alpha)
{
	glEnable(GL_TEXTURE_2D);

	glBegin(GL_QUADS);

	glColor4f(1.0f, 1.0f, 1.0f, alpha.x);
	glTexCoord2f(0.0f, 1.0f); glVertex2f(p.x, p.y);

	glColor4f(1.0f, 1.0f, 1.0f, alpha.y);
	glTexCoord2f(1.0f, 1.0f); glVertex2f(p.x + size.x, p.y);

	glColor4f(1.0f, 1.0f, 1.0f, alpha.z);
	glTexCoord2f(1.0f, 0.0f); glVertex2f(p.x + size.x, p.y + size.y);

	glColor4f(1.0f, 1.0f, 1.0f, alpha.w);
	glTexCoord2f(0.0f, 0.0f); glVertex2f(p.x, p.y + size.y);
	glEnd();

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}


void drawObject(MapObject* obj, bool selected)
{
	glDisable(GL_TEXTURE_2D);
	vec3 alpha;

	for (unsigned int i=0; i< obj->polygons.getCount(); i++)
	{
		alpha.x = obj->polygons[i].c1.w;
		alpha.y = obj->polygons[i].c2.w;
		alpha.z = obj->polygons[i].c3.w;
		drawTriangle(&(obj->polygons[i]), selected, alpha);
	}
}


void drawTexturedObject(MapObject* obj, bool selected)
{
	glEnable(GL_TEXTURE_2D);
	vec3 alpha;

	for (unsigned int i=0; i< obj->polygons.getCount(); i++)
	{
		alpha.x = obj->polygons[i].c1.w;
		alpha.y = obj->polygons[i].c2.w;
		alpha.z = obj->polygons[i].c3.w;
		drawTexturedTriangle(&(obj->polygons[i]), selected, alpha);
	}
}


void drawCircle(vec2& pos, float radius, float alpha)
{
	const float f = 30.0f * deg2rad;

	glDisable(GL_TEXTURE_2D);

	glPushMatrix();
	glTranslatef(pos.x-cam.p.x, pos.y-cam.p.y, 0.0f);

	glColor4f(1.0f, 1.0f, 1.0f, alpha);
	glBegin(GL_LINE_STRIP);
	for (int i = 0; i < 13; i++)
	{
		glVertex2f(radius*cosf((float)i * f), radius*sinf((float)i * f));
	}
	glEnd();
	glPopMatrix();

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}


void drawAABB(AABB* aabb)
{ //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	vec2 alpha(1.0f, 1.0f);
	drawLine(aabb->v1 - aabb->v2, vec2(aabb->v1.x + aabb->v2.x, aabb->v1.y - aabb->v2.y), alpha);
	drawLine(aabb->v1 + aabb->v2, vec2(aabb->v1.x + aabb->v2.x, aabb->v1.y - aabb->v2.y), alpha);
	drawLine(aabb->v1 + aabb->v2, vec2(aabb->v1.x - aabb->v2.x, aabb->v1.y + aabb->v2.y), alpha);
	drawLine(aabb->v1 - aabb->v2, vec2(aabb->v1.x - aabb->v2.x, aabb->v1.y + aabb->v2.y), alpha);
}


void calculateTextureCoord(GraphicTriangle* tri, int tex_width, int tex_height)
{
	tri->t1 = tri->v1/vec2((float)tex_width, (float)tex_height);
	tri->t2 = tri->v2/vec2((float)tex_width, (float)tex_height);
	tri->t3 = tri->v3/vec2((float)tex_width, (float)tex_height);
}


void calculateTextureCoord(MapObject* obj, int tex_width, int tex_height)
{
	for (unsigned int i=0; i< obj->polygons.getCount(); i++)
	{
		calculateTextureCoord(&(obj->polygons[i]), tex_width, tex_height);
	}
}


void calculateAABB(MapObject* obj, AABB* aabb)
{
	float minx=obj->polygons[0].v1.x,
		  maxx=obj->polygons[0].v1.x,
		  miny=obj->polygons[0].v1.y,
		  maxy=obj->polygons[0].v1.y;

	for (unsigned int i=0; i< obj->polygons.getCount(); i++)
	{
		if (obj->polygons[i].v1.x<minx) minx = obj->polygons[i].v1.x;
		if (obj->polygons[i].v1.x>maxx) maxx = obj->polygons[i].v1.x;

		if (obj->polygons[i].v2.x<minx) minx = obj->polygons[i].v2.x;
		if (obj->polygons[i].v2.x>maxx) maxx = obj->polygons[i].v2.x;

		if (obj->polygons[i].v3.x<minx) minx = obj->polygons[i].v3.x;
		if (obj->polygons[i].v3.x>maxx) maxx = obj->polygons[i].v3.x;


		if (obj->polygons[i].v1.y<miny) miny = obj->polygons[i].v1.y;
		if (obj->polygons[i].v1.y>maxy) maxy = obj->polygons[i].v1.y;

		if (obj->polygons[i].v2.y<miny) miny = obj->polygons[i].v2.y;
		if (obj->polygons[i].v2.y>maxy) maxy = obj->polygons[i].v2.y;

		if (obj->polygons[i].v3.y<miny) miny = obj->polygons[i].v3.y;
		if (obj->polygons[i].v3.y>maxy) maxy = obj->polygons[i].v3.y;
	}

	aabb->v1 = vec2( minx + (maxx-minx)*0.5f, miny + (maxy - miny)*0.5f);
	aabb->v2 = vec2( (maxx-minx)*0.5f, (maxy - miny)*0.5f);
}

