#include "Gui.h"
#include "../Omega/OMEGA_functions.h"

// --------------------------------------------------------------------
// GUIElement
//

void GUIElement::Initialize(const char* t)
{
	active = true;
	strcpy(text, t);
}

bool GUIElement::OnCursor(vec2 p)
{
	return    (p.x > pos.x) && (p.x < pos.x + size.x) &&
			  (p.y > pos.y) && (p.y < pos.y + size.y) && active;	
}

void GUIElement::SetMode(int m)
{
	mode = m;
}

void GUIElement::OnClick()
{}

void GUIElement::OnMouseOn()
{}

void GUIElement::OnMouseOver()
{}


void GUIElement::Render()
{}

void GUIElement::RenderText()
{
	font.drawText(textPos + pos, 0.2f, text);
}



// --------------------------------------------------------------------
// GUIPanel
//

void GUIPanel::Render()
{
	glColor4f(1, 1, 1, 1);
	setTexture(texture);
	RenderQuad(pos, size);
}


// --------------------------------------------------------------------
// GUILabel
//

void GUILabel::RenderText()
{
	font.drawText(textPos, size.x, text);
}


// --------------------------------------------------------------------
// GUIButton
//

void GUIButton::Initialize(const char* t)
{
	strcpy(text, t);
	text_size = 25.f;
	textPos = vec2(0.0f, 0.0f);
	active  = true;
	mode     = gui_mode_normal;
}

void GUIButton::OnClick()
{
	if (active)
	{
		SetMode(gui_mode_press);
		OnClickEvent();
	}
}

void GUIButton::OnMouseOn()
{
	 SetMode(gui_mode_rollover);
}

void GUIButton::OnMouseOver()
{
	SetMode(gui_mode_normal);
}

void GUIButton::Render()
{
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

	if (active)
	{

		if (mode == gui_mode_normal)
		{

			setTexture(tex_normal);
			RenderQuad(pos, size);

		}
		else
		if (mode ==  gui_mode_press)
		{
			setTexture(tex_press);
			RenderQuad(pos, size);
		}
			else
			{
				setTexture(tex_rollover);
				RenderQuad(pos, size);
			}
	}
}

void GUIButton::RenderText()
{
	font.drawText(textPos + pos, text_size, text);
}


// --------------------------------------------------------------------
// GUICheckBox
//


void GUICheckBox::Initialize(const char* t)
{
	strcpy(text, t);
	text_size = 25.f;
	textPos = vec2(size.x, 0.0f);
	active  = true;
	checked = false;
}

void GUICheckBox::OnClick()
{
	if (active)
	{
		checked = !checked;
	}
}

void GUICheckBox::OnMouseOn()
{}

void GUICheckBox::OnMouseOver()
{}

void GUICheckBox::Render()
{
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

	if (active)
	{
		if (checked == false)
		{

			setTexture(tex_normal);
			RenderQuad(pos, size);

		}
		else
		if (checked == true)
		{
			setTexture(tex_press);
			RenderQuad(pos, size);
		}
	}
}

void GUICheckBox::RenderText()
{
	font.drawText(textPos + pos, text_size, text);
}



// --------------------------------------------------------------------
// GUIEngine
//
int GUIEngine::AddElement(GUIElement* elem)
{
	elements.add( elem );
	return elements.getCount()-1;
}

int GUIEngine::update()
{
	for (unsigned int i = 0; i < elements.getCount() ; i++)
	{
		if (elements[i]->OnCursor(vec2(Mouse_X(), Mouse_Y())) && elements[i]->active)
		{
			elements[i]->OnMouseOn();
			if (Mouse_Click(M_LBUTTON))
			{
				elements[i]->OnClick();
				return i;
			}
		}
		else
			elements[i]->OnMouseOver();
	}
	return -1;
}

void GUIEngine::Render()
{
	glEnable(GL_TEXTURE_2D);
	for (unsigned int i = 0; i < elements.getCount(); i++)
		if ( elements[i]->active )
		{
			elements[i]->Render();
			elements[i]->RenderText();
		}
}

