#ifndef __GUI__
#define __GUI__

#include "OGLutils.h"
#include <string.h>
#include <stdio.h>
#include <windows.h>
#include <gl/gl.h>
#include "Array.h"
#include "font.h"


#define gui_mode_normal					1
#define gui_mode_rollover				2
#define gui_mode_press					3



typedef void (* POnClickEvent)();			    // 


// --------------------------------------------------------------------
// GUIElement
//
class GUIElement
{
public:
	vec2 pos;			// 
	vec2 size;			// 
	vec2 textPos;		//    
	bool active;        //

	int mode;

	char text[256];     // 

public:
    virtual void Initialize(const char* t);
	virtual bool OnCursor(vec2 p);
    virtual void SetMode(int m);
    virtual void OnClick();
    virtual void OnMouseOn();
    virtual void OnMouseOver();
    virtual void Render();
    virtual void RenderText();

	void setText(const char* format, ...)
	{
		va_list ap;

		va_start(ap, format);
		vsprintf(text, format, ap);
		va_end(ap);
	}
};


// --------------------------------------------------------------------
// GUIPanel
//
class GUIPanel: public GUIElement
{
public:
    int texture;

public:
    void Render();
};


// --------------------------------------------------------------------
// GUILabel
//
class GUILabel: public GUIElement
{
public:

public:
    void RenderText();
};


// --------------------------------------------------------------------
// GUIButton
//
class GUIButton: public GUIElement
{
public:
	int tex_normal;
	int tex_rollover;
	int tex_press;

	float text_size;

	POnClickEvent OnClickEvent;

public:
	void Initialize(const char* t);
    void OnClick();
    void OnMouseOn();
    void OnMouseOver();
	void Render();
    void RenderText();
};



// --------------------------------------------------------------------
// GUICheckBox
//
class GUICheckBox: public GUIElement
{
public:
	int tex_normal;
	int tex_press;

	float text_size;

	bool checked;

public:
	void Initialize(const char* t);
    void OnClick();
    void OnMouseOn();
    void OnMouseOver();
	void Render();
    void RenderText();
};




// --------------------------------------------------------------------
// GUIEngine
//
class GUIEngine
{
public:
    Array<GUIElement*>   elements;

public:

	int AddElement(GUIElement* elem);

    void Render();

	int update(); // -1 -   
				  //        

	void setText(int index, const char* format, ...)
	{
		va_list ap;
		va_start(ap, format);
		vsprintf(elements[index]->text, format, ap);
		va_end(ap);
	}
};


#endif