#include "ParticleSystem.h"


// ---------------------------------------------------------------------------
// ParticlesEmitter
//

ParticlesEmitter::ParticlesEmitter(vec2& p, vec3& col)
{
	pos     = p;
	color   = col;
	active = true;
}

void ParticlesEmitter::ResetParticle(int i){}

void ParticlesEmitter::UpdateParticle(int i, float dt){}

void ParticlesEmitter::Update(float dt)
{
	int active_particles = 0;
	for (unsigned int i=0; i<particles.getCount(); i++)
	{
		if (particles[i].life > 0.0f)
		{
			particles[i].life -= dt;
			UpdateParticle(i, dt);
			++active_particles;
		}
		else if (loop) ResetParticle(i);
	}
        
	if (( !loop ) && ( 0 == active_particles )) 
		active = false;
}

void ParticlesEmitter::Render()
{
	glEnable(GL_TEXTURE_2D);
	float s;

	setTexture(tex);

	if (!alpha) glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	else   	    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	  
	for (unsigned int i=0; i<particles.getCount(); i++)
	{
		if (particles[i].life > 0.0f )
		{
			if (cam.boxVisible(particles[i].pos/*+ pos*/, particles[i].size))
			{
				s = particles[i].life / particles[i].start_life;
				vec4 col(color, s);
				glColor4fv(col);

				glPushMatrix();
//					glTranslatef(pos.x, pos.y, 0.0f);
					glTranslatef(particles[i].pos.x, particles[i].pos.y, 0.0f);
					glRotatef(particles[i].angle, 0.0f, 0.0f, 1.0f);
					glScalef(particles[i].size.x, particles[i].size.y, 1.0f);
					
					RenderQuad(vec2(-0.5f, -0.5f), vec2(1.0f, 1.0f));
				glPopMatrix();
			}
		}

	}
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
}



// ---------------------------------------------------------------------------
// ParticlesEngine
//

int    ParticlesEngine::AddEmitter(ParticlesEmitter* pm)
{
	emitters.add(pm);
	return emitters.getCount()+1;
}

void   ParticlesEngine::DeleteEmitter(int i)
{
	ParticlesEmitter* em = emitters[i];
	delete em;
	emitters.fastRemove(i);
}

void   ParticlesEngine::Update(float dt)
{
	for (unsigned int i=0; i<emitters.getCount(); i++)
	{
		if (emitters[i]->active)emitters[i]->Update(dt);
		else					DeleteEmitter(i);
	}
}

void   ParticlesEngine::RenderFront()
{
	for (unsigned int i=0; i<emitters.getCount(); i++)
		if (!emitters[i]->back)
			emitters[i]->Render();
}

void   ParticlesEngine::RenderBack()
{
	for (unsigned int i=0; i<emitters.getCount(); i++)
		if (emitters[i]->back)
			emitters[i]->Render();
}
