
#include "TextureManager.h"
#include "Log.h"


TextureCache _cache;

bool Texture::load(const char *filename)
{   
	if (mID) //    c 
	{
		Delete();
	}


	Log.Write("Loading texture : %s - ", filename);


	//  
	mID = _cache.LoadIDFromCache( filename );
	if( mID > 0 )
	{
		Log.Write("(Search in cache) Texture has already loaded !\n");
		return true;
	}


	//    Image    
	Image image;
	if(image.ReadFile(filename) == false) //  !  .
	{
		mID = 0;
		return false;
	}


	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glGenTextures(1, &mID); //  ID


	switch ( mTexType ) 
	{
		case Tex1D : 
			glBindTexture(GL_TEXTURE_1D, mID); 

			if (mUseMipmap)
			{
				glTexParameterf( GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR );
				glTexParameterf( GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
			}
			else
			{
				glTexParameterf( GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
				glTexParameterf( GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
			}

			switch ( image.Bpp() )
			{
					case 8: if ( ! mUseMipmap )	glTexImage2D(GL_TEXTURE_1D, 0, GL_LUMINANCE, image.GetSizeX(), image.GetSizeY(), 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, image.GetPointer()); 
						    else           gluBuild2DMipmaps(GL_TEXTURE_1D, GL_LUMINANCE, image.GetSizeX(), image.GetSizeY(), GL_LUMINANCE, GL_UNSIGNED_BYTE, image.GetPointer());                           
							break;
					case 24:if ( ! mUseMipmap ) glTexImage2D(GL_TEXTURE_1D, 0, GL_RGB, image.GetSizeX(), image.GetSizeY(), 0, GL_RGB, GL_UNSIGNED_BYTE, image.GetPointer());
						    else		   gluBuild2DMipmaps(GL_TEXTURE_1D, GL_RGB, image.GetSizeX(), image.GetSizeY(), GL_RGB, GL_UNSIGNED_BYTE, image.GetPointer());  
							break;
					case 32:if ( ! mUseMipmap ) glTexImage2D(GL_TEXTURE_1D, 0, GL_RGBA, image.GetSizeX(), image.GetSizeY(), 0, GL_RGBA, GL_UNSIGNED_BYTE, image.GetPointer());
						    else		   gluBuild2DMipmaps(GL_TEXTURE_1D, GL_RGBA, image.GetSizeX(), image.GetSizeY(), GL_RGBA, GL_UNSIGNED_BYTE, image.GetPointer());  
							break;
			}
			break;

		case Tex2D : 
			glBindTexture(GL_TEXTURE_2D, mID); 
			
			if (mUseMipmap)
			{
				glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR );
				glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
			}
			else
			{
				glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
				glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
			}

			switch ( image.Bpp() )
			{
					case 8: if ( ! mUseMipmap )	glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE, image.GetSizeX(), image.GetSizeY(), 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, image.GetPointer()); 
						    else           gluBuild2DMipmaps(GL_TEXTURE_2D, GL_LUMINANCE, image.GetSizeX(), image.GetSizeY(), GL_LUMINANCE, GL_UNSIGNED_BYTE, image.GetPointer());                           
							break;
					case 24:if ( ! mUseMipmap ) glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, image.GetSizeX(), image.GetSizeY(), 0, GL_RGB, GL_UNSIGNED_BYTE, image.GetPointer());
						    else		   gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, image.GetSizeX(), image.GetSizeY(), GL_RGB, GL_UNSIGNED_BYTE, image.GetPointer());  
							break;
					case 32:if ( ! mUseMipmap ) glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, image.GetSizeX(), image.GetSizeY(), 0, GL_RGBA, GL_UNSIGNED_BYTE, image.GetPointer());
						    else		   gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, image.GetSizeX(), image.GetSizeY(), GL_RGBA, GL_UNSIGNED_BYTE, image.GetPointer());  
							break;
			}			
			break;

/*		case Tex3D : 
			glBindTexture(GL_TEXTURE_3D, mID); 
			
			if (mUseMipmap)
			{
				glTexParameterf( GL_TEXTURE_3D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR );
				glTexParameterf( GL_TEXTURE_3D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
			}
			else
			{
				glTexParameterf( GL_TEXTURE_3D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
				glTexParameterf( GL_TEXTURE_3D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
			}

			switch ( image.Bpp() )
			{
					case 8: if ( ! mUseMipmap )	glTexImage2D(GL_TEXTURE_3D, 0, GL_LUMINANCE, image.GetSizeX(), image.GetSizeY(), 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, image.GetPointer()); 
						    else           gluBuild2DMipmaps(GL_TEXTURE_3D, GL_LUMINANCE, image.GetSizeX(), image.GetSizeY(), GL_LUMINANCE, GL_UNSIGNED_BYTE, image.GetPointer());                           
							break;
					case 24:if ( ! mUseMipmap ) glTexImage2D(GL_TEXTURE_3D, 0, GL_RGB, image.GetSizeX(), image.GetSizeY(), 0, GL_RGB, GL_UNSIGNED_BYTE, image.GetPointer());
						    else		   gluBuild2DMipmaps(GL_TEXTURE_3D, GL_RGB, image.GetSizeX(), image.GetSizeY(), GL_RGB, GL_UNSIGNED_BYTE, image.GetPointer());  
							break;
					case 32:if ( ! mUseMipmap ) glTexImage2D(GL_TEXTURE_3D, 0, GL_RGBA, image.GetSizeX(), image.GetSizeY(), 0, GL_RGBA, GL_UNSIGNED_BYTE, image.GetPointer());
						    else		   gluBuild2DMipmaps(GL_TEXTURE_3D, GL_RGBA, image.GetSizeX(), image.GetSizeY(), GL_RGBA, GL_UNSIGNED_BYTE, image.GetPointer());  
							break;
			}
			break;
*/
	}
	
	image.Release(); //  

	if (mID == 0) return false; // ..,   !

	_cache.AddEntry(filename, mID); //  

	Log.Write("Ok!\n"); 

	return true;
}

/*
bool Texture::loadCubemap(const char *filesnames[6])
{

	if (mID) //    c 
	{
		Delete();
	}

	Log.Write("    :\n");

	glEnable(GL_TEXTURE_CUBE_MAP_ARB);

	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glGenTextures(1, &mID);

	glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, mID); 

	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);


	glTexParameteri( GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_S, GL_REPEAT );
	glTexParameteri( GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_T, GL_REPEAT );


	if (mUseMipmap)
	{
		glTexParameterf( GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR );
		glTexParameterf( GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	}
	else
	{
		glTexParameterf( GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
		glTexParameterf( GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	}


	for (int i = 0; i<6 ; i++ )
	{
		Image image;

		if(image.ReadFile(filesnames[i]) == false) //  !  .
		{
			mID = 0;
			return false;
		}

		Log.Write("		%s\n", filesnames[i]);

		switch ( image.Bpp() )
		{

			case 8: if ( ! mUseMipmap )	glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X + i, 0, GL_LUMINANCE, image.GetSizeX(), image.GetSizeY(), 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, image.GetPointer()); 
					else           gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_POSITIVE_X + i, GL_LUMINANCE, image.GetSizeX(), image.GetSizeY(), GL_LUMINANCE, GL_UNSIGNED_BYTE, image.GetPointer());                           
					break;
			case 24:if ( ! mUseMipmap ) glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X + i, 0, GL_RGB, image.GetSizeX(), image.GetSizeY(), 0, GL_RGB, GL_UNSIGNED_BYTE, image.GetPointer());
					else		   gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_POSITIVE_X + i, GL_RGB, image.GetSizeX(), image.GetSizeY(), GL_RGB, GL_UNSIGNED_BYTE, image.GetPointer());  
					break;
			case 32:if ( ! mUseMipmap ) glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X + i, 0, GL_RGBA, image.GetSizeX(), image.GetSizeY(), 0, GL_RGBA, GL_UNSIGNED_BYTE, image.GetPointer());
					else		   gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_POSITIVE_X + i, GL_RGBA, image.GetSizeX(), image.GetSizeY(), GL_RGBA, GL_UNSIGNED_BYTE, image.GetPointer());  
					break;
		}

		image.Release();
	}

	glDisable(GL_TEXTURE_CUBE_MAP_ARB);

	Log.Write("		Ok!\n");

	return true;
}
*/

void Texture::CreateEmpty(int width, int height, int NumComponents)
{
	if (mID != 0) Delete(); //       
						    //  

/*	unsigned char *texture; //   
	if (mTexType != Cubemap)
		texture = (unsigned char *) malloc(width * height * NumComponents); //    
*/
	glGenTextures(1, &mID); //  ID


	switch ( mTexType ) 
	{
		case Tex1D : 
			glBindTexture(GL_TEXTURE_1D, mID); 

			glTexParameterf( GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR );
			glTexParameterf( GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );

			switch ( NumComponents )
			{
					case 1:	glTexImage2D(GL_TEXTURE_1D, 0, GL_LUMINANCE, width, height, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, /*texture*/ NULL); break;
					case 3:	glTexImage2D(GL_TEXTURE_1D, 0, GL_RGB, width, height, 0, GL_RGB, GL_UNSIGNED_BYTE,			   /*texture*/ NULL); break;
					case 4:	glTexImage2D(GL_TEXTURE_1D, 0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE,		   /*texture*/ NULL); break;
			}
			break;

		case Tex2D : 
			glBindTexture(GL_TEXTURE_2D, mID); 
			
			glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR );
			glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );

			switch ( NumComponents )
			{
					case 1:	glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE, width, height, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, /*texture*/ NULL); break;
					case 3:	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, width, height, 0, GL_RGB, GL_UNSIGNED_BYTE,			   /*texture*/ NULL); break;
					case 4:	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE,		   /*texture*/ NULL); break;
			}			
			break;

/*		case Tex3D : 
			glBindTexture(GL_TEXTURE_3D, mID); 
			
			glTexParameterf( GL_TEXTURE_3D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR );
			glTexParameterf( GL_TEXTURE_3D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );

			switch ( NumComponents )
			{
				case 1:	gluBuild2DMipmaps(GL_TEXTURE_3D, GL_LUMINANCE, width, height, GL_LUMINANCE, GL_UNSIGNED_BYTE, NULL); break;
				case 3:	gluBuild2DMipmaps(GL_TEXTURE_3D, GL_RGB, width, height, GL_RGB, GL_UNSIGNED_BYTE, NULL); break;
				case 4:	gluBuild2DMipmaps(GL_TEXTURE_3D, GL_RGBA, width, height, GL_RGBA, GL_UNSIGNED_BYTE, NULL); break;
			}

			break;
*/
/*		case Cubemap : 
			glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, mID); 

			glTexParameterf( GL_TEXTURE_3D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR );
			glTexParameterf( GL_TEXTURE_3D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
			
			//   
			for (int i = 0; i<6 ; i++ )
			{
				switch ( NumComponents )
				{
					case 1:	glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X + i, 0, GL_LUMINANCE, width, height, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, NULL); break;
					case 3:	glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X + i, 0, GL_RGB, width, height, 0, GL_RGB, GL_UNSIGNED_BYTE,				NULL); break;
					case 4:	glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X + i, 0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE,			NULL); break;
				}
			}
			break;*/
	}

/*	if (mTexType != Cubemap)
		free(texture); //  
*/
}

void Texture::setFilter(TextureFilter tf)
{
	mTexFilter = tf;
	bind();
	switch (mTexFilter)
	{
	case TF_CLAMP: 
		{
//			glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );
//			glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );
			break;
		}
	case TF_REPEAT:
		{
			glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
			glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
			break;
		}
	case TF_NEAREST:
		{
			glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
			glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
			break;
		}
	case TF_LINEAR:
		{
            glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
            glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
			break;
		}
	case TF_BILINEAR:
		{
            glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST );
            glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
			break;
		}
	case TF_TRILINEAR:
		{
            glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR );
            glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
			break;
		}
	}
	unbind();
}

