// ------------------------------------------------------------------
//
//      :
//   BMP, TGA, JPG, JPEG, GIF
//
//	  :
//	  (1D, 2D...)    
//     Texture.
//
//	  MipMap      
//	  Texture.
//
//   : ViZgl
//
// ------------------------------------------------------------------

#ifndef __TEXTURE_MANAGER_H__
#define __TEXTURE_MANAGER_H__

#include <vector>

#include "../Image/Image.h"
#include <GL/glu.h>


// ------------------------------------------------------------------
//  .      !
//
struct TextureInfo
{
	std::string filename;
	unsigned int id;
};

class TextureCache
{
public:

	void AddEntry(const char* filename, unsigned int id)
	{
		TextureInfo newEntry;
		//  
		newEntry.filename = filename;
		newEntry.id = id;
		__cache.push_back( newEntry ); //   
	}

	unsigned int LoadIDFromCache(const char* filename)
	{
		std::vector<TextureInfo>::iterator it;
		it = __cache.begin();
		while(it != __cache.end()) //  
		{
			if( strcmp( it->filename.c_str(), filename ) == 0 ) //     
			{
				return it->id;	// !
			}
			it++;
		}
		return 0; //  
	}

	const char* LoadNameFromCache(const int _ID)
	{
		std::vector<TextureInfo>::iterator it;
		it = __cache.begin();
		while(it != __cache.end()) //  
		{
			if( it->id == _ID ) //     
			{
				return it->filename.c_str();	// !
			}
			it++;
		}
		return NULL; //  
	}

	
	bool DeleteFromCache(const int _id)
	{
		std::vector<TextureInfo>::iterator it;
		it = __cache.begin();
		while(it != __cache.end()) //  
		{
			if( it->id == _id ) //     
			{
				return true;	// !
			}
			it++;
		}
		return false;
	}
	

private:
	std::vector<TextureInfo> __cache; //  
};


// ------------------------------------------------------------------
//  
//
extern TextureCache _cache;


// ------------------------------------------------------------------
//   .
//
enum TextureFilter
{
	TF_CLAMP,
	TF_REPEAT,
	TF_NEAREST,
	TF_LINEAR,
	TF_BILINEAR,
	TF_TRILINEAR
};


// ------------------------------------------------------------------
//   
//
enum TexType
{
	Tex1D,
	Tex2D,
//	Tex3D,
	Cubemap,
};


// ------------------------------------------------------------------
// 
//
class Texture {

protected:

	GLuint			mID;
	int				mWidth;
	int				mHeight;
	TextureFilter	mTexFilter; //   TF_TRILINEAR
	TexType			mTexType;	//   Tex2D
	bool            mUseMipmap; //   true

	// MipMap'     

public:

	Texture(void)
		: mID(0)
		, mWidth(0)
		, mHeight(0)
		, mTexFilter(TF_TRILINEAR)
		, mTexType(Tex2D)
		, mUseMipmap(true)
	{}

	Texture(TexType type, bool Mipmap = true) { mTexType = type; mUseMipmap = Mipmap;}

	~Texture(void)	{ Delete(); }

	GLuint& id(void)	{ return mID; }

	int getWidth(void)	{ return mWidth;  }

	int getHeight(void) { return mHeight;  }

	void bind(void)
	{
		switch ( mTexType ) 
		{
			case Tex1D : glBindTexture(GL_TEXTURE_1D, mID); break;
			case Tex2D : glBindTexture(GL_TEXTURE_2D, mID); break;
//			case Tex3D : glBindTexture(GL_TEXTURE_3D, mID); break;
//			case Cubemap : glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, mID); break;
		}
	}

	void unbind(void)
	{
		switch ( mTexType ) 
		{
			case Tex1D : glBindTexture(GL_TEXTURE_1D, 0); break;
			case Tex2D : glBindTexture(GL_TEXTURE_2D, 0); break;
//			case Tex3D : glBindTexture(GL_TEXTURE_3D, 0); break;
//			case Cubemap:glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, 0); break;
		}
	}

	TextureFilter getFilter(void) { return mTexFilter; }

	TexType getType(void) { return mTexType; }

	void Delete(void)
	{
		if (mID != 0)
		{
			//_cache.DeleteFromCache(mID);
			glDeleteTextures(1, &mID); 
			mID = 0;
		}
	}

	void setID(unsigned int m){mID=m;}
	void setFilter(TextureFilter tf);
	bool load(const char *filename);            // 1D, 2D (,3D) 
//	bool loadCubemap(const char *filesnames[6]);// Cubemap 
	void CreateEmpty(int width, int height, int NumComponents);
};


#endif
