#include "Weapons.h"
#include "Utils/OGLutils.h"


// ----------------------------------------------------------
// Bullet
//

void Bullet::update(float dt)
{
	if ( !dead )
	{
		pos += vel * dt;
		timeToDead -= dt;
		if (timeToDead <= 0.0f)
		{
			kill();
		}

		if ( blt_smoke != NULL )
			switch ( type )
			{
			case MACHINEGUN:
					break;

			case FIREFLOW:
				vel *= dt*1.96f;
				angle += dt*deg2rad*5.0f;
				break;

			case ROCKETLAUNCHER:
				blt_smoke->AddParticle(pos, vec2(frand(1.0f)-0.5f, frand(1.0f)-0.5f) , vec2(12), 30.0f);
				break;

			case SHOTGUN:
				break;
			case MINE:
				break;
			}
	}
}

void Bullet::kill()
{
	if ( blt_smoke != NULL )
		blt_smoke->owner_dead = true;
	dead = true;
}
  

// ----------------------------------------------------------
// WeaponParam
//

void WeaponParam::init(const char* weapon_type)
{
	strcpy(this->weapon_type, weapon_type);

	IniFile      f("Config/Weapon.ini");

	char* tmp;

	tmp = f.GetString(weapon_type, "Texture", "");
	bullTex	   = _cache.LoadIDFromCache(tmp);

	multiply = (f.GetInt(weapon_type, "Multiply", 0)>0)?true:false;

	reloadTime = f.GetFloat(weapon_type,"Reload Time",0.0f);
	damage     = f.GetFloat(weapon_type,"Damage",0.0f);
	speed      = f.GetFloat(weapon_type,"Speed",0.0f);
	speedEps   = f.GetFloat(weapon_type,"Speed Eps", 0.0f);
	bullCount  = f.GetInt  (weapon_type,"Bullets Count", 0);
	shopSize   = f.GetInt  (weapon_type,"Shop Size", 0);
	bullLifeTime=f.GetInt  (weapon_type,"Life Time", 1);
	variation  = f.GetFloat(weapon_type,"Variation",0.0f);
}



// ----------------------------------------------------------
// BulletsManager
//

void BulletsManager::addBullet(Bullet& bull)
{
	bullets.add(bull);
}

void BulletsManager::update(float dt)
{
	for (unsigned int i = 0; i < bullets.getCount(); i++)
	{
		bullets[i].update(dt);
		if (bullets[i].dead) bullets.fastRemove(i);
	}
}

void BulletsManager::render()
{
	for (unsigned int i = 0; i < bullets.getCount(); i++)
	{
		if (!bullets[i].dead)
		{
			if (cam.circleVisible(bullets[i].pos, 15.0f))
			{
				vec2 Size;
				glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

				if ( paramWeapon[bullets[i].type].multiply )
					glBlendFunc(GL_SRC_ALPHA, GL_ONE);
				else
					glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);


				glPushMatrix();
				glTranslatef(bullets[i].pos.x, bullets[i].pos.y, 0.0f);
				setTexture(paramWeapon[bullets[i].type].bullTex);


				switch (bullets[i].type)
				{
				case MACHINEGUN:
					{
						Size = vec2(5, 3);

						glRotatef(bullets[i].angle * rad2deg, 0.0f, 0.0f, 1.0f);

						glBegin(GL_QUADS);
						glTexCoord2f(0.0f, 1.0f); glVertex2f( - Size.x, - Size.y );
						glTexCoord2f(0.0f, 0.0f); glVertex2f( - Size.x, Size.y );
						glTexCoord2f(1.0f, 0.0f); glVertex2f( Size.x,   Size.y );
						glTexCoord2f(1.0f, 1.0f); glVertex2f( Size.x,   - Size.y );
						glEnd();

						break;
					}
				case FIREFLOW:
					{
						float alpha = bullets[i].timeToDead / paramWeapon[bullets[i].type].bullLifeTime;
						Size = vec2(20.0f - pow(alpha, 4.0f)*20.0f);

						glRotatef(bullets[i].angle * rad2deg, 0.0f, 0.0f, 1.0f);
						vec3 c = lerp(bullets[i].endColor, bullets[i].startColor, pow(alpha, 1.7f));
						glColor4f(c.x, c.y, c.z, alpha);

						glBegin(GL_QUADS);
						glTexCoord2f(0.0f, 1.0f); glVertex2f( - Size.x, - Size.y );
						glTexCoord2f(0.0f, 0.0f); glVertex2f( - Size.x, Size.y );
						glTexCoord2f(1.0f, 0.0f); glVertex2f( Size.x,   Size.y );
						glTexCoord2f(1.0f, 1.0f); glVertex2f( Size.x,   - Size.y );
						glEnd();

						break;
					}
				case ROCKETLAUNCHER:
					{
						Size = vec2(25, 12);

						glRotatef(bullets[i].angle * rad2deg, 0.0f, 0.0f, 1.0f);

						glBegin(GL_QUADS);
						glTexCoord2f(0.0f, 1.0f); glVertex2f( - Size.x, - Size.y );
						glTexCoord2f(0.0f, 0.0f); glVertex2f( - Size.x, Size.y );
						glTexCoord2f(1.0f, 0.0f); glVertex2f( Size.x,   Size.y );
						glTexCoord2f(1.0f, 1.0f); glVertex2f( Size.x,   - Size.y );
						glEnd();

						break;
					}
				case SHOTGUN:
					{
						Size = vec2(3, 2);

						glRotatef(bullets[i].angle * rad2deg, 0.0f, 0.0f, 1.0f);

						glBegin(GL_QUADS);
						glTexCoord2f(0.0f, 1.0f); glVertex2f( - Size.x, - Size.y );
						glTexCoord2f(0.0f, 0.0f); glVertex2f( - Size.x, Size.y );
						glTexCoord2f(1.0f, 0.0f); glVertex2f( Size.x,   Size.y );
						glTexCoord2f(1.0f, 1.0f); glVertex2f( Size.x,   - Size.y );
						glEnd();

						break;
					}
				} // switch

				glPopMatrix();
			} // if
		} // if
	} // for

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}

void BulletsManager::init()
{
	paramWeapon[MACHINEGUN].init("Machine Gun");
	paramWeapon[FIREFLOW].init("Fire Gun");
	paramWeapon[ROCKETLAUNCHER].init("Rocket Launcher");
	paramWeapon[SHOTGUN].init("Shotgun");
	paramWeapon[MINE].init("Mine");
}


void BulletsManager::renderMine()
{
	for (unsigned int i = 0; i < bullets.getCount(); i++)
	{
		if (!bullets[i].dead && bullets[i].type == MINE)
		{
			if (cam.circleVisible(bullets[i].pos, 15.0f))
			{
				vec2 Size;
				glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

				if ( paramWeapon[bullets[i].type].multiply )
					glBlendFunc(GL_SRC_ALPHA, GL_ONE);
				else
					glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);


				glPushMatrix();
				glTranslatef(bullets[i].pos.x, bullets[i].pos.y, 0.0f);
				setTexture(paramWeapon[bullets[i].type].bullTex);


				Size = vec2(15, 15);

				glBegin(GL_QUADS);
				glTexCoord2f(0.0f, 1.0f); glVertex2f( - Size.x, - Size.y );
				glTexCoord2f(0.0f, 0.0f); glVertex2f( - Size.x, Size.y );
				glTexCoord2f(1.0f, 0.0f); glVertex2f( Size.x,   Size.y );
				glTexCoord2f(1.0f, 1.0f); glVertex2f( Size.x,   - Size.y );
				glEnd();


				glPopMatrix();
			}
		}
	}

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}

