#include "World.h"
#include "Utils/EditorDraw.h"


const float RESPAWN_TIME = 80;


//  
Array<Bot> World::bots;
Map World::map;
PFTiresTrail* World::tyres = NULL;
PFSmoke*	  World::carSmoke = NULL;

ParticlesEngine particlesEngine;


//  ,    Omega'    !
//   ;),       [0..100], ..
// value    [0..100]
#define VOL(value) (int)( (-6500 + value * 65) )


//    
void playSound(PSound snd, vec2& source_pos)
{
	vec2 v = source_pos - World::bots[0].pos;
	float len = length(v);
	if ( len </* 1024.0f */ 800.0f )
	{	//    :)
//		int id = Snd_Play( snd, v.x*10.0f, VOL((1200 - len)*0.12));
		int id = Snd_Play( snd, v.x*10.0f/*, VOLUME((1200 - len)*0.12)*/);
	}
}


//  i-   
void checkRespawn(int i, float dt)
{
	if ( World::bots[i].Active == false )
	{
		World::bots[i].respawnTime -= dt;
		if ( World::bots[i].respawnTime <= 0 )
		{	//     
			World::bots[i].pos = World::map.respawnPoints[(int)frand(World::map.respawnPoints.getCount())];
			World::bots[i].Active = true;
			World::bots[i].Health = 100;
			//  
			World::bots[i].shopCount[MACHINEGUN] = bulletsManager.paramWeapon[MACHINEGUN].shopSize;
			World::bots[i].shopCount[FIREFLOW] = 0;
			World::bots[i].shopCount[ROCKETLAUNCHER] = 0;
			World::bots[i].shopCount[MINE] = bulletsManager.paramWeapon[MINE].shopSize;

			PFCarRespawn* p = new PFCarRespawn(World::bots[i].pos);
			particlesEngine.AddEmitter(p);

			playSound(snd_respawn, World::bots[i].pos);
		}
	}
}


//  
void World::addRichochet( WeaponID wpn, vec2& pos, vec2& normal, bool isPlayer, bool use_sound)
{
	switch (wpn)
	{
	case SHOTGUN:
	case MACHINEGUN:
		{
			PFRicochet* pm = new PFRicochet(pos, normal, 2, vec3(1.0f));
			particlesEngine.AddEmitter(pm);
			if (use_sound)
			{
				float t = frand(3.0f);
				if ( t<1.0f )
					playSound(snd_bullet_ricochet1, pos);
				else
					if ( t>1.0f && t<2.0f)
						playSound(snd_bullet_ricochet2, pos);
					else
						playSound(snd_bullet_ricochet3, pos);
			}
			if ( isPlayer ) cam.Shake(2, 5.0f);
			break;
		}

	case FIREFLOW:
		{

			if ( isPlayer ) cam.Shake(5, 2.0f);
			break;
		}

	case ROCKETLAUNCHER:
		{
			addSmallExplosion(pos, 55.0f);
			PFRicochet* pm = new PFRicochet(pos, vec2(0.0f), 15, vec3(1.0f));
			particlesEngine.AddEmitter(pm);

			playSound(snd_rocket_blow, pos);

			if ( isPlayer ) cam.Shake(10, 5.0f);

			break;
		}
	case MINE:
		{
			addSmallExplosion(pos, 55.0f);
			PFRicochet* pm = new PFRicochet(pos, vec2(0.0f), 15, vec3(1.0f));
			particlesEngine.AddEmitter(pm);

			playSound(snd_rocket_blow, pos);
			if ( isPlayer ) cam.Shake(14, 5.0f);

			break;
		}
	}
}


//  
void World::addCarExplosion(vec2& pos)
{
	PFExplosionWave* pe = new PFExplosionWave( pos, 75.0f );
	PFCrater*		 pc = new PFCrater( pos, 100.0f );
	PFExplosionAnim* pea= new PFExplosionAnim( pos, 150.0f );

	particlesEngine.AddEmitter(pe);
	particlesEngine.AddEmitter(pc);
	particlesEngine.AddEmitter(pea);

	playSound(snd_explosion, pos);

	float len = length(pos - bots[0].pos);
	if ( len < 800 )
		cam.Shake((40 - len*0.08f), 20.0f);
}


//   :  ,   
void World::addSmallExplosion(vec2& pos, float size)
{
	PFExplosionWave* pe = new PFExplosionWave( pos, size-10.0f );
	PFExplosionAnim* pea= new PFExplosionAnim( pos, size );
	particlesEngine.AddEmitter(pe);
	particlesEngine.AddEmitter(pea);
}


void World::checkBotsCollisions(int i, int j)
{
	vec2 depth(0.0f);
	bool col1 = Circle(bots[i].pos, bots[i].carSize.x).colCircle(bots[j].pos, bots[j].carSize.x, &depth);
	bool col2 = Circle(bots[i].pos, bots[i].carSize.x).colCircle(bots[j].pos, bots[j].carSize.x, &depth);

	if ( col1 == true || col2 )
	{
		bots[i].force += -depth*0.2f;
		bots[j].force +=  depth*0.2f;
	}

}


void World::checkBotWithMap(int i)
{
	for ( int k=0; k<map.objects.getCount(); k++ )
	{
		if ( map.objects[k].aabb.colCircle(Circle(bots[i].pos, bots[i].carSize.x)) && map.objects[k].collided )	//  AABB
		{
			for ( int j=0; j<map.objects[k].polygons.getCount(); j++ )
			{ 
				vec2 normal(0.0f);
				float depth = 0.0f;

				if ( map.objects[k].polygons[j].colCircle(bots[i].pos, bots[i].carSize.x, &normal, &depth) )	//    
				{
					bots[i].pos += normal*depth;
				}
			}
		}
	}
}


void World::renderHUD()
{
	vec2 imageSize(45,45);
	float displace = 1.0f;
	float width = WINDOW_WIDTH-30;
	// 
	// 
	tex_mine_bonus.bind();
	drawTexturedQuad(vec2(width-imageSize.x*displace, 0.0f), imageSize, vec4(1.0f));
	font.drawText(vec2(width-imageSize.x+imageSize.x*0.25f, imageSize.y), 25, "%d", bots[0].shopCount[MINE]);

	displace += 1.0f;

	// 
	tex_rocket_bonus.bind();
	if ( ROCKETLAUNCHER == bots[0].currentWeapon)
	drawTexturedQuad(vec2(width-imageSize.x*displace, 0.0f), imageSize*1.2f, vec4(1.0f));
	else
	drawTexturedQuad(vec2(width-imageSize.x*displace, 0.0f), imageSize, vec4(0.3f));
	font.drawText(vec2(width-imageSize.x*displace, imageSize.y), 25, "%d", bots[0].shopCount[ROCKETLAUNCHER]);

	displace += 1.0f;

	// 
	tex_fire_bonus.bind();
	if ( FIREFLOW == bots[0].currentWeapon)
	drawTexturedQuad(vec2(width-imageSize.x*displace, 0.0f), imageSize*1.2f, vec4(1.0f));
	else
	drawTexturedQuad(vec2(width-imageSize.x*displace, 0.0f), imageSize, vec4(0.3f));
	font.drawText(vec2(width-imageSize.x*displace+3.0f, imageSize.y), 25, "%d", bots[0].shopCount[FIREFLOW]);

	displace += 1.0f;

	// 
	tex_shotgun_bonus.bind();
	if ( SHOTGUN == bots[0].currentWeapon)
	drawTexturedQuad(vec2(width-imageSize.x*displace, 0.0f), imageSize*1.2f, vec4(1.0f));
	else
	drawTexturedQuad(vec2(width-imageSize.x*displace, 0.0f), imageSize, vec4(0.3f));
	font.drawText(vec2(width-imageSize.x*displace, imageSize.y), 25, "%d", bots[0].shopCount[SHOTGUN]);

	displace += 1.0f;

	// 
	tex_bullet_bonus.bind();
	if ( MACHINEGUN == bots[0].currentWeapon)
	drawTexturedQuad(vec2(width-imageSize.x*displace, 0.0f), imageSize*1.2f, vec4(1.0f));
	else
	drawTexturedQuad(vec2(width-imageSize.x*displace, 0.0f), imageSize, vec4(0.3f));
	font.drawText(vec2(width-imageSize.x*displace, imageSize.y), 25, "%d", bots[0].shopCount[MACHINEGUN]);
}


void World::renderStatistic()
{
//	drawQuad( cam.p + vec2( 225, WINDOW_HEIGHT*0.4f ), vec2( 200, WINDOW_HEIGHT*0.5f ), false, vec4(0.2f, 0.3f, 0.6f, 0.3f));
	font.drawText(vec2(50.0f, 25.0f), 25.0f, "%s       %s  %s", "name", "kills", "death"); 

	for (unsigned int i=0; i < bots.getCount(); i++)
	{
		font.drawText(vec2(50.0f, 80.0f+i*25.0f), 25.0f, "%s",	 bots[i].name);
		font.drawText(vec2(190.0f, 80.0f+i*25.0f), 25.0f, "%d",  bots[i].killsCount);
		font.drawText(vec2(280.0f, 80.0f+i*25.0f), 25.0f, "%d",  bots[i].deathCount);
	}
}


void World::renderMini()
{	//      :(   )
	glColor4f(1.0f, 1.0f, 1.0f, 0.6f);
	tex_mini_map.bind();
	RenderQuad(vec2(0, WINDOW_HEIGHT-132), vec2(132, 132));

	// 
	glColor4f(1.0f, 1.0f, 1.0f, 0.7f);
	tex_ter_grass.bind();
	for ( int i=0; i<map.objects.getCount(); i++)
	{
		float dx = map.objects[i].aabb.v1.x/cam.maxSizeX*128.0f+2;
		float dy = map.objects[i].aabb.v1.y/cam.maxSizeY*128.0f+WINDOW_HEIGHT-132;

		float sx = map.objects[i].aabb.v2.x/cam.maxSizeX*128.0f+2;
		float sy = map.objects[i].aabb.v2.y/cam.maxSizeY*128.0f+2;

		RenderQuad(vec2(dx-sx, dy-sy) , vec2(sx*2.0f, sy*2.0f));	
	}

	Bonus* b=NULL;
	if ( bots[0].Health < 30 )
		b = bots[0].findClosestBonus(BNS_MEDKIT);

	// 
	glColor4f(0.4f, 1.0f, 0.3f, 1.0f);
	tex_white_spark.bind();
	for ( int i=0; i<map.bonuses.getCount(); i++)
	{
		if (map.bonuses[i].active == false) continue;
		float dx = map.bonuses[i].pos.x/cam.maxSizeX*128.0f+2;
		float dy = map.bonuses[i].pos.y/cam.maxSizeY*128.0f+WINDOW_HEIGHT-132;

		if ( b != NULL && map.bonuses[i].pos == b->pos )
		{
			glColor4f(1.0f, 0.3f, 0.0f, 1.0f);
			float t = sinf(U_GetTime()*0.007f)*15.0f;
			RenderQuad(vec2(dx-t*0.5f, dy-t*0.5f) , vec2(t,t));
			glColor4f(0.4f, 1.0f, 0.3f, 1.0f);
		}
		else
			RenderQuad(vec2(dx-2, dy-2) , vec2(4,4));		
	}

	// 
	glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
	tex_white_spark.bind();
	float dx = bots[0].pos.x/cam.maxSizeX*128.0f+4;
	float dy = bots[0].pos.y/cam.maxSizeY*128.0f+WINDOW_HEIGHT-132;
	RenderQuad(vec2(dx-3, dy-3) , vec2(6,6));	

	// 
	glColor4f(0.0f, 1.0f, 1.0f, 1.0f);
	tex_white_spark.bind();
	for (unsigned int i=1; i < bots.getCount(); i++)
	{
		float dx = bots[i].pos.x/cam.maxSizeX*128.0f+4;
		float dy = bots[i].pos.y/cam.maxSizeY*128.0f+WINDOW_HEIGHT-132;
		RenderQuad(vec2(dx-3, dy-3) , vec2(6,6));	
	}

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}


void World::update(float dt)
{
	bots[0].update(dt);				//  , ..    AI(    ;)
	checkRespawn(0, dt);

	//     
	if ( bots[0].isTyres() )
	{
		//   
		float size = bots[0].carSize.x - 20.0f;
		vec2 backSide = bots[0].pos - vec2(size*cosf(bots[0].Alpha), size*sinf(bots[0].Alpha));

		tyres->AddParticle(backSide, bots[0].Alpha);
	}

	// 
	if ( frand() < 0.2f )
	{
		vec2 p = bots[0].pos - vec2(bots[0].carSize.x*cosf(bots[0].Alpha), bots[0].carSize.x*sinf(bots[0].Alpha));
		vec2 v = -vec2(cosf(bots[0].Alpha)*frand(2.0f), sinf(bots[0].Alpha)*frand(2.0f));

		carSmoke->AddParticle(p, v , vec2(8), 20.0f);
	}

	for (unsigned int i=1; i < bots.getCount(); i++)
	{
		checkRespawn(i, dt);
		bots[i].update(dt);			// 
		bots[i].updateAI();			// AI


		if ( bots[i].isTyres() )	//     
		{
			//   
			float size = bots[i].carSize.x - 20.0f;
			vec2 backSide = bots[i].pos - vec2(size*cosf(bots[i].Alpha), size*sinf(bots[i].Alpha));

			tyres->AddParticle(backSide, bots[i].Alpha);
		}

		// 
		if ( frand() < 0.2f )
		{
			vec2 p = bots[i].pos - vec2(bots[i].carSize.x*cosf(bots[i].Alpha), bots[i].carSize.x*sinf(bots[i].Alpha));
			vec2 v = -vec2(cosf(bots[i].Alpha)*frand(2.0f), sinf(bots[i].Alpha)*frand(2.0f));

			carSmoke->AddParticle(p, v , vec2(8), 20.0f);
		}
	}

	particlesEngine.Update(dt);		// 
	bulletsManager.update(dt);		//  

	for (unsigned int i=0; i<map.bonuses.getCount(); i++)
		map.bonuses[i].update(dt);

	solveCollisions();				//  
}


void World::render()
{
	glEnable(GL_TEXTURE_2D);

	map.render();					//  

	glPushMatrix();
	glTranslatef(-cam.p.x, -cam.p.y, 0.0f);

	bulletsManager.renderMine();	//  , ..   
	particlesEngine.RenderBack();	//   

	for ( unsigned int i=0; i < bots.getCount(); i++ )
	{
		bots[i].render();			// 
	}

	bulletsManager.render();		//  
	particlesEngine.RenderFront();	//   

	glPopMatrix();

	map.renderBonuses();//   
	renderMini();

	if ( bots[0].Active == true )
	{
		renderHUD();		//  
	}
	else
	{	//    
		font.drawText(vec2(cam.width*0.5f-25, cam.height*0.5f), 40, "%1.1f", (bots[0].respawnTime*0.1f));
	}
}


void World::solveCollisions()
{	//   ,   )

	for ( unsigned int i=0; i < bots.getCount(); i++ )
	{
		if (bots[i].pos.x+bots[i].carSize.x > map.width ) bots[i].pos.x = map.width - bots[i].carSize.x;
		if (bots[i].pos.y+bots[i].carSize.x > map.height ) bots[i].pos.y = map.height - bots[i].carSize.x;
		if (bots[i].pos.x-bots[i].carSize.x < 0.0f ) bots[i].pos.x = bots[i].carSize.x;
		if (bots[i].pos.y-bots[i].carSize.x < 0.0f ) bots[i].pos.y = bots[i].carSize.x;
	}


	// 
	for ( unsigned int j=0; j < bots.getCount(); j++ )
	{

		//  
		for ( unsigned int i=0; i < bulletsManager.bullets.getCount(); i++ )
		{
			if ( bulletsManager.bullets[i].dead == false)
			{
				//  
				if ( bulletsManager.bullets[i].pos.x < 0.0f )
				{
					bulletsManager.bullets[i].kill();
					addRichochet(bulletsManager.bullets[i].type, vec2(0.0f, bulletsManager.bullets[i].pos.y), vec2(1.0f, 0.0f), false);
				}
				if ( bulletsManager.bullets[i].pos.y < 0.0f )
				{
					bulletsManager.bullets[i].kill();
					addRichochet(bulletsManager.bullets[i].type, vec2(bulletsManager.bullets[i].pos.x, 0.0f), vec2(0.0f, 1.0f), false);
				}
				if ( bulletsManager.bullets[i].pos.x > map.width )
				{
					bulletsManager.bullets[i].kill();
					addRichochet(bulletsManager.bullets[i].type, vec2(map.width , bulletsManager.bullets[i].pos.y), vec2(-1.0f, 0.0f), false);
				}
				if ( bulletsManager.bullets[i].pos.y > map.height )
				{
					bulletsManager.bullets[i].kill();
					addRichochet(bulletsManager.bullets[i].type, vec2(bulletsManager.bullets[i].pos.x , map.height), vec2(0.0f, -1.0f), false);
				}

				//  
				if ( bulletsManager.bullets[i].owner != (void*)&bots[j] &&			//      ,   
					 AABB(bots[j].pos, bots[j].carSize).pointInside(bulletsManager.bullets[i].pos, bots[j].Alpha) && //    OBB ;)
					 bots[j].Active)												//  
				{			
					//  ,     
					bots[j].Health -= bulletsManager.paramWeapon[bulletsManager.bullets[i].type].damage;
					bulletsManager.bullets[i].kill();	//   :)


					// 
					addRichochet(bulletsManager.bullets[i].type, bulletsManager.bullets[i].pos, vec2(0.0f, 0.0f));
					if ( bulletsManager.bullets[i].type == ROCKETLAUNCHER || bulletsManager.bullets[i].type == MINE )
					{
						//     
						for (int m=0; m<bots.getCount(); m++)
						{
							float len = length(bots[m].pos-bulletsManager.bullets[i].pos);
							if ( len < 150 && len != 0.0f &&  bulletsManager.bullets[i].owner != &bots[m] )
								bots[m].Health -= (150 - len)*0.2f;
						}
					}


					if ( bots[j].Health <= 0 )				//  
					{
						addCarExplosion(bots[j].pos);						   // 
						bots[j].Active = false;							       //  
						bots[j].respawnTime = RESPAWN_TIME;					   //  
						bots[j].deathCount++;								   //  ? :(
						((Car*)bulletsManager.bullets[i].owner)->killsCount++; //    ;)
					} 

	
				} // if


				//   
				for ( int k=0; k<map.objects.getCount(); k++ )
				{
					if ( map.objects[k].aabb.pointInside(bulletsManager.bullets[i].pos) && map.objects[k].collided )
					{
						for ( int m=0; m<map.objects[k].polygons.getCount(); m++ )
						{ 
							vec2 normal(0.0f);
							float depth(0.0f);
							if ( map.objects[k].polygons[m].pointInside(bulletsManager.bullets[i].pos, &normal, &depth) )
							{
								bulletsManager.bullets[i].kill();
								addRichochet(bulletsManager.bullets[i].type, bulletsManager.bullets[i].pos, normal, false);							
							}
						}
					}
				}
			}

		} // i


		//   
		for (unsigned int k=0; k < map.bonuses.getCount(); k++)
		{
			if ( Circle(bots[j].pos, bots[j].carSize.x).colCircle(map.bonuses[k].pos, 32) && map.bonuses[k].active)
			{
				playSound(snd_take_bonus, map.bonuses[k].pos);		// , 
				switch ( map.bonuses[k].type )						//   ,    
				{
				case BNS_MACHINEGUN:
					{
						bots[j].shopCount[BNS_MACHINEGUN] = bulletsManager.paramWeapon[BNS_MACHINEGUN].shopSize;
						map.bonuses[k].disable();
						break;
					}
				case BNS_FIREFLOW:
					{
						bots[j].shopCount[BNS_FIREFLOW] = bulletsManager.paramWeapon[BNS_FIREFLOW].shopSize;
						map.bonuses[k].disable();
						break;
					}
				case BNS_ROCKETLAUNCHER:
					{
						bots[j].shopCount[BNS_ROCKETLAUNCHER] = bulletsManager.paramWeapon[BNS_ROCKETLAUNCHER].shopSize;
						map.bonuses[k].disable();
						break;
					}
				case BNS_MINE:
					{
						bots[j].shopCount[BNS_MINE] = bulletsManager.paramWeapon[BNS_MINE].shopSize;
						map.bonuses[k].disable();
						break;
					}
				case BNS_MEDKIT:
					{
						bots[j].Health = 100;
						map.bonuses[k].disable();
						break;
					}
				case BNS_SHOTGUN:
					{
						bots[j].shopCount[BNS_SHOTGUN] = bulletsManager.paramWeapon[BNS_SHOTGUN].shopSize;
						map.bonuses[k].disable();
						break;
					}
				} // switch
			} // if
		} // for k

		checkBotWithMap(j);	//       

		//   (  )
/*
		for ( int i=0; i < bots.getCount(); i++)
			if ( i==j ) continue;
			else
				checkBotsCollisions(i, j);
*/
	} // for j

}


