#   pragma comment (lib, "opengl32.lib")
#   pragma comment (lib, "glu32.lib")

#include <windows.h>
#include <gl/gl.h>
#include "Resources.h"

#include "Omega/OMEGA_code.cpp"
#include "Omega/OMEGA_functions.h"

#include "GameMenu.h"
#include "World.h"

#include "Utils/EditorDraw.h"




Camera cam;
World world;
MenuState gameState = MS_MAINMENU;


Font font;
LOG Log;


void init ()
{
	srand(GetTickCount());
	Log.Enable(false);

	glClearColor ( 0.0, 0.0, 0.0, 1.0 );

	glEnable     ( GL_TEXTURE_2D );  //  

	glEnable(GL_BLEND);				 //  
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	loadConfig();				//  
	loadTextures();				//  
	loadSounds();				//  
	loadBotNames();				//  
	initGUI();					//  GUI
	bulletsManager.init();		//   
	font.Init(tex_font.id());	//  

	applyConfig();			//  
}

void display ()
{

	switch (gameState)
	{
	case MS_GAMEPLAY:
		{
			world.render();
			if ( Key_Down(K_TAB) || !world.bots[0].Active) world.renderStatistic();

			break;
		}

	case MS_MAINMENU:
		{
			glClear(GL_COLOR_BUFFER_BIT);
			renderMainMenu();

			break;
		}

	case MS_OPTIONS:
		{
			glClear(GL_COLOR_BUFFER_BIT);
			renderOptionsMenu();

			break;
		}
	case MS_PREPARE:
		{
			glClear(GL_COLOR_BUFFER_BIT);
			renderPrepareMenu();

			break;
		}
	}

	// 
	tex_cursor.bind();
	drawTexturedQuad(vec2(Mouse_X(), Mouse_Y())-16.0f, vec2(32.0f), vec4(1.0f));
}


void	animate ()
{
	const float dt = 0.5f;

	switch (gameState)
	{
	case MS_GAMEPLAY:
		{
			if ( Key_Press( K_ESCAPE ) ) gameState = MS_PREPARE;

			//  
    		if (Key_Down(K_W)) { world.bots[0].pressGas(); }
			if (Key_Down(K_S)) { world.bots[0].pressBreak(); }
			if (Key_Down(K_A)) { world.bots[0].turnLeft(); }
			if (Key_Down(K_D)) { world.bots[0].turnRight(); }


			// 
			if (Mouse_Down(M_LBUTTON) && world.bots[0].Active)
			{
				world.bots[0].shoot(vec2(Mouse_X(), Mouse_Y()) + cam.p);
			}

			//    
			if (Mouse_Down(M_RBUTTON) && world.bots[0].Active)
			{
				world.bots[0].moveTarget = &(vec2(Mouse_X(), Mouse_Y())+cam.p);
				world.bots[0].moveTo( 0.0f );
			}

			//   
			world.update(dt);

			//  
			cam.centerCam(world.bots[0].pos, dt);

			//  
			if ( Mouse_WheelDown() )
			{
				switch ( world.bots[0].currentWeapon )
				{	
				case MACHINEGUN:
					world.bots[0].currentWeapon = SHOTGUN; break;

				case FIREFLOW:
					world.bots[0].currentWeapon = ROCKETLAUNCHER; break;

				case ROCKETLAUNCHER:
					world.bots[0].currentWeapon = MACHINEGUN; break;

				case SHOTGUN:
					world.bots[0].currentWeapon = FIREFLOW; break;
				}
			}

			if ( Mouse_WheelUp() )
			{
				switch ( world.bots[0].currentWeapon )
				{	
				case MACHINEGUN:
					world.bots[0].currentWeapon = ROCKETLAUNCHER; break;

				case FIREFLOW:
					world.bots[0].currentWeapon = SHOTGUN; break;

				case ROCKETLAUNCHER:
					world.bots[0].currentWeapon = FIREFLOW; break;

				case SHOTGUN:
					world.bots[0].currentWeapon = MACHINEGUN; break;
				}
			}
			if (Key_Down(K_SPACE)) { world.bots[0].putMine(); }

			break;
		}

	case MS_MAINMENU:
		{
			if ( Key_Press( K_ESCAPE ) ) OE_Quit();
			updateMainMenu();

			break;
		}

	case MS_OPTIONS:
		{
			if ( Key_Press( K_ESCAPE ) ) gameState = MS_MAINMENU;
			updateOptionsMenu();

			break;
		}

	case MS_PREPARE:
		{
			if ( Key_Press( K_ESCAPE ) ) gameState = MS_MAINMENU;
			updatePrepareMenu();

			break;
		}
	}

}

void quit()
{
	unloadTextures();
}

int main ( int argc, char * argv [] )
{
	LoadOMEGA( "OMEGA.dll" );						//   

	Timer_Create( (void*)animate, 12, 1);			//   12 ms - 25 ms

	OE_RegProcedure( SYS_INIT, (void*)init );		//  
	OE_RegProcedure( SYS_DRAW, (void*)display );	//  
	OE_RegProcedure( SYS_QUIT, (void*)quit );		//   

	OE_SetScreenOptions( WINDOW_WIDTH, 
						 WINDOW_HEIGHT, 
						 32, 
						 0, 
						 false, 
						 false );					//  

	OE_SetWndName("Machine Fighting");
	OE_Disable(FLAG_COLOR_BUFFER_CLEAR);

	OE_Init();										// 

	return 0;	//   :)
}


